# FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
# License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
# Author: Klaus Haeming

macro( unique listname )
	set( unique_tmp "" )
	foreach( item ${ARGN} )
		if( "${unique_tmp}" MATCHES "${item}" )
		else( "${unique_tmp}" MATCHES "${item}" )
			set( unique_tmp "${unique_tmp}" "${item}" )
		endif( "${unique_tmp}" MATCHES "${item}" )
	endforeach( item )
	set( ${listname} "${unique_tmp}" )
endmacro( unique )

macro( print_properties targetname )
	message( STATUS "${targetname} property LOCATION: ${location}")
	get_target_property( CMAKE_BUILD_TYPEe ${targetname} TYPE )
	message( STATUS "${targetname} property TYPE: ${CMAKE_BUILD_TYPEe}")
endmacro( print_properties )

macro( export_library libname )
	get_target_property( location ${libname} LOCATION )
	set( ${libname}_LIBRARIES ${location} ${${libname}_LIBS} )
	set( ${libname}_FOUND ${${libname}_FOUND} CACHE INTERNAL "${libname} is present" FORCE )
	set( ${libname}_DEFINITIONS ${${libname}_DEFINITIONS} CACHE INTERNAL "${libname} definitions" FORCE )
	set( ${libname}_INCLUDE_DIR ${${libname}_INCLUDE_DIR} CACHE INTERNAL "${libname} include directories" FORCE )
	set( ${libname}_LIBRARY_DIR ${${libname}_LIBRARY_DIR} CACHE INTERNAL "${libname} library directories" FORCE )
	set( ${libname}_LIBRARIES ${${libname}_LIBRARIES} CACHE INTERNAL "${libname} libraries" FORCE )
endmacro( export_library )

macro( trace targetname )
	string( REPLACE ";" " " ${targetname}_USED_DEFINITIONS_L "${${targetname}_USED_DEFINITIONS}" )
	string( REPLACE ";" " " ${targetname}_USED_INCLUDE_DIR_L "${${targetname}_USED_INCLUDE_DIR}" )
	string( REPLACE ";" " " ${targetname}_USED_LIBRARY_DIR_L "${${targetname}_USED_LIBRARY_DIR}" )
	string( REPLACE ";" " " ${targetname}_USED_LIBRARIES_L "${${targetname}_USED_LIBRARIES}" )
	string( REPLACE ";" " " ${targetname}_USED_LIBS_L "${${targetname}_USED_LIBS}" )
	string( REPLACE ";" " " ${targetname}_USED_DEPS_L "${${targetname}_USED_DEPS}" )
	message( STATUS ${targetname} "_USED_DEFINITIONS: " ${${targetname}_USED_DEFINITIONS_L} )
	message( STATUS ${targetname} "_USED_INCLUDE_DIR: " ${${targetname}_USED_INCLUDE_DIR_L} )
	message( STATUS ${targetname} "_USED_LIBRARY_DIR: " ${${targetname}_USED_LIBRARY_DIR_L} )
	message( STATUS ${targetname} "_USED_LIBRARIES: " ${${targetname}_USED_LIBRARIES_L} )
	message( STATUS ${targetname} "_USED_LIBS: " ${${targetname}_USED_LIBS_L} )
	message( STATUS ${targetname} "_USED_DEPS: " ${${targetname}_USED_DEPS_L} )
endmacro( trace )

# macro for ${pkgname} to use library ${libname} that is calibrated through variables
macro( use_library targetname )
    foreach( libname ${ARGN} )
		if( ${libname}_FOUND )
			set( ${targetname}_USED_DEFINITIONS ${${targetname}_USED_DEFINITIONS} ${${libname}_DEFINITIONS} )
			set( ${targetname}_USED_INCLUDE_DIR ${${targetname}_USED_INCLUDE_DIR} ${${libname}_INCLUDE_DIR} )
			set( ${targetname}_USED_INCLUDE_DIR ${${targetname}_USED_INCLUDE_DIR} ":" )	
			set( ${targetname}_USED_LIBRARY_DIR ${${targetname}_USED_LIBRARY_DIR} ${${libname}_LIBRARY_DIR} )
			set( ${targetname}_USED_LIBS ${${targetname}_USED_LIBS} ${${libname}_LIBRARIES} )
			set( ${targetname}_USED_DEPS ${${targetname}_USED_DEPS} ${libname} )
		else( ${libname}_FOUND )
			message( FATAL_ERROR "Library "${libname}" required for "${targetname}" does not exist!" )
		endif( ${libname}_FOUND )
	endforeach( libname )
endmacro( use_library )

# macro for configureing target build
macro( configure_target targetname )

	unique( ${targetname}_USED_DEFINITIONS ${${targetname}_USED_DEFINITIONS} )
	unique( ${targetname}_USED_INCLUDE_DIR ${${targetname}_USED_INCLUDE_DIR} )
	unique( ${targetname}_USED_LIBRARY_DIR ${${targetname}_USED_LIBRARY_DIR} )
	unique( ${targetname}_USED_LIBRARIES ${${targetname}_USED_LIBRARIES} )
#	unique( ${targetname}_USED_LIBS ${${targetname}_USED_LIBS} )
	unique( ${targetname}_USED_DEPS ${${targetname}_USED_DEPS} )

	add_definitions( ${${targetname}_USED_DEFINITIONS} )
	include_directories( ${${targetname}_USED_INCLUDE_DIR} )
	link_directories( ${${targetname}_USED_LIBRARY_DIR} )
	target_link_libraries( ${targetname} ${${targetname}_USED_LIBS} )
	if( ${targetname}_USED_DEPS )
		add_dependencies( ${targetname} ${${targetname}_USED_DEPS} )
	endif( ${targetname}_USED_DEPS )

endmacro( configure_target )

