

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_CORECHANNELSET_H
#define ADORE_CORECHANNELSET_H

#include <vector>
#include <adrcore/imgprocessing/imageplane.h>
#include <adrcore/persistence/imgsaver.h>

namespace adore
{

	namespace core
	{

		/**
		 * A Set of channels
			 
		*/

		template <class imgT> class Channel;
		template <class imgT> class ImagePlane;

		template <class imgT>
		class ChannelSet
		{
			public:

				typedef Channel<imgT> ChannelT;
				typedef std::vector< ChannelT *> ContainerT;
				typedef typename ContainerT::iterator iterator;
				typedef typename ContainerT::const_iterator const_iterator;

			public:
				ChannelSet( ImagePlane<imgT> &img, bool del = false ): doDelete( del )
				{
					img.createChannels( *this );
				}

				explicit ChannelSet( bool del = false ): doDelete( del ), chans()
				{
				}

				ChannelSet( const ChannelSet<imgT> &set ): doDelete( false /*to prevent double deletion*/ ),
						chans( set.chans.size() )
				{
					for ( uint i = 0; i < chans.size(); ++i ) chans[i] = set.chans[i];
				}

				~ChannelSet()
				{
					wipe();
				}

				void add( ChannelSet &set )
				{
					for ( int s = 0; s < size(); ++s ) chans.push_back( set[s] );
				}

				void add( ChannelT *chan )
				{
					chans.push_back( chan );
				}

				void add( ImagePlane<imgT> &img )
				{
					img.createChannels( *this );
				}


				ChannelT *&operator[]( const int i )
				{
					return chans[i];
				}

				ChannelT *&operator()( const int i )
				{
					return chans[i];
				}

				ChannelT * const&operator[]( const int i ) const
				{
					return chans[i];
				}

				ChannelT * const&operator()( const int i ) const
				{
					return chans[i];
				}

				void clear()
				{
					wipe();
					chans.clear();
				}

				uint size() const
				{
					return chans.size();
				}

				iterator begin()
				{
					return chans.begin();
				}

				iterator end()
				{
					return chans.end();
				}

				const_iterator begin() const
				{
					return chans.begin();
				}

				const_iterator end() const
				{
					return chans.end();
				}

				int getWidth() const
				{
					return ( *begin() )->getWidth();
				}

				int getHeight() const
				{
					return ( *begin() )->getHeight();
				}

				const ROI &getROI() const
				{
					return ( *begin() )->getROI();
				}

				int getNumChans() const
				{
					return size();
				}

				bool empty() const
				{
					return size() == 0;
				}

				/**
					saves the ImgArray the first channel points to.
				*/
				void save ( const char *name ) const
				{
					if ( !empty() )
					{
						ImgSaver<imgT> saver;
						ImgArray *img = chans[0]->getImgArray();
						saver.saveImgArray ( name, img, chans[0]->getROI() );
					}
				}
				
				void setDeleteImages(const bool b){doDelete = b;}

			private:

				void wipe()
				{
					if ( doDelete ) for ( uint i = 0; i < chans.size(); ++i )
						{
							delete chans[i];
						}
				}

				bool doDelete;
				vector< ChannelT *> chans;
		};

	}
}

#endif
