

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_COREDIZENZOAT_H
#define ADORE_COREDIZENZOAT_H

#include<adrcore/imgprocessing/channelset.h>
#include<adrcore/imgprocessing/dizenzocommon.h>
#include<adrcore/util/calctype.h>

namespace adore
{

	namespace core
	{

		/**
		 * calculates locally a multichannel gradient
			 
		*/
		template<class inT>
		class DiZenzoAt
		{
			public:
				typedef typename signedT< typename calcT<inT>::type >::type calcT;

			public:
				DiZenzoAt( const ChannelSet<inT> &ch ):chans( ch )
				{
				}

				~DiZenzoAt()
				{
					;
				}

				/**
				 * calculates gxx, gxy and gyy
				 * do this before using one of the query functions
				 * @return false, if x and y are out of the image boundaries
				 */
				bool operator()( int x, int y )
				{
					if ( chans.size() < 1 || x < 1 || y < 1 || x >= chans[0]->getWidth() || y >= chans[0]->getHeight() ) return false;

					calcT gx = 0;
					calcT gy = 0;
					gxx = 0;
					gyy = 0;
					gxy = 0;
					for ( uint c = 0; c < chans.size(); ++c )
					{
						gx = *chans[c]->getPointer( x + 1, y ) - *chans[c]->getPointer( x - 1, y );
						gy = *chans[c]->getPointer( x, y + 1 ) - *chans[c]->getPointer( x, y - 1 );
						gxx += gx * gx;
						gyy += gy * gy;
						gxy += gx * gy;
					}

					gxx /= calcT( chans.size() );
					gyy /= calcT( chans.size() );
					gxy /= calcT( chans.size() );

					return true;
				}

				calcT getGXX() const
				{
					return gxx;
				}

				calcT getGYY() const
				{
					return gyy;
				}

				calcT getGXY() const
				{
					return gxy;
				}

				void om( calcT &mag, double &orient )
				{
					double winkelMal2 = getWinkelMal2( gxx, gyy, gxy );
					mag = calcT( getMagnitude( gxx, gyy, gxy, winkelMal2 ) );
					orient = 0.5 * winkelMal2;
					if ( orient == -halfPI ) orient = halfPI;
				}

				void omm( calcT &magX, calcT &magY, double &orient )
				{
					double winkelMal2 = getWinkelMal2( gxx, gyy, gxy );
					calcT mag = calcT( getMagnitude( gxx, gyy, gxy, winkelMal2 ) );
					orient = 0.5 * winkelMal2;
					if ( orient == -halfPI ) orient = halfPI;
					magX = mag * cos( orient );
					magY = mag * sin( orient );
				}

				void sm( calcT &mag, char &sector )
				{
					double winkelMal2 = getWinkelMal2( gxx, gyy, gxy );
					mag = calcT( getMagnitude( gxx, gyy, gxy, winkelMal2 ) );
					sector = getSector( winkelMal2 );
				}

				void smm( calcT &magX, calcT &magY, char &sector )
				{
					double winkelMal2 = getWinkelMal2( gxx, gyy, gxy );
					calcT mag = calcT( getMagnitude( gxx, gyy, gxy, winkelMal2 ) );
					sector = getSector( winkelMal2 );
					float orient = 0.5 * winkelMal2;
					if ( orient == -halfPI ) orient = halfPI;
					magX = mag * cos( orient );
					magY = mag * sin( orient );
				}

				calcT m()
				{
					return calcT( getMagnitude( gxx, gyy, gxy ) );
				}

				char s()
				{
					double winkelMal2 = getWinkelMal2( gxx, gyy, gxy );
					return getSector( winkelMal2 );
				}

				double o()
				{
					double winkelMal2 = getWinkelMal2( gxx, gyy, gxy );
					double orient = 0.5 * winkelMal2;
					if ( orient == -halfPI ) orient = halfPI;
					return orient;
				}

			private:

				const ChannelSet<inT> &chans;
				calcT gxx;
				calcT gyy;
				calcT gxy;

		};

	}

}

#endif
