

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_IMGPROCESSINGDIZENZOCOMMON_H
#define ADORE_IMGPROCESSINGDIZENZOCOMMON_H

#include <adrcore/util/mathutil.h>

/**
 * The sectors are numerated as follows
 * 3 2 1
 * 0 X 0
 * 1 2 3
 */

namespace adore
{
	namespace core
	{

		const char ANGLE_SECTOR[9] = {2, 3, 3, 0, 0, 1, 1, 2, 2};
		const float tanPIeighth = 0.41421356237309509F;
		const float halfPI = 0.5 * M_PI;

		/**
		 * computes an often used value
		 */
		inline float getWinkelMal2( const float &gxx, const float &gyy, const float &gxy )
		{
			return atan2f( 2.0F * gxy, gxx - gyy );
		}

		/**
		 * calculates multicolor gradient magnitude using the winkelMal2 value
		 */
		inline float getMagnitude( const float &gxx, const float &gyy,
		                           const float &gxy, const float &winkelMal2 )
		{
			return sqrtf( 0.5F*( gxx + gyy + cosf( winkelMal2 )*( gxx - gyy ) + 2.0F*gxy*sinf( winkelMal2 ) ) );
		}

		/**
		 * calculates multicolor gradient magnitude without using the winkelMal2 value.
		 */
		inline float getMagnitude( const float &gxx, const float &gyy,
		                           const float &gxy )
		{
			float y = 2.0F * gxy;
			float x = gxx - gyy;
			float sin, cos;
			tan2sincos( y, x, sin, cos );
			return sqrtf( 0.5F*( gxx + gyy + cos * x + sin * y ) );
		}

		inline uint8_t getSector( const float &winkelMal2 )
		{
			float w = ( ( 0.5F * winkelMal2 + halfPI ) * 8.0F ) / M_PI;
			return ANGLE_SECTOR[int( floorf( w ) )];
		}

		inline uint8_t getSectorOfOrientation( const float &o )
		{
			return getSector( 2.0f * o );
		}

	}
}

#endif

