

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_IMGPROCESSINGHARRIS_H
#define ADORE_IMGPROCESSINGHARRIS_H

#include <adrcore/imgprocessing/channelprocessor.h>
#include <adrcore/imgprocessing/imgfunctions.h>
#include <adrcore/types/aarray.h>
#include <adrcore/util/gaussians.h>
#include "harriscommon.h"

namespace adore
{

	namespace core
	{

		/**
		 
		*/
		template <class inT>
		class Harris : public ChannelProcessor< 3, 1, inT, typename harrisOutT<inT>::ouT>
		{
				typedef AArray<double, 2> T_weight;
			public:
				typedef typename harrisOutT<inT>::ouT ouT;


				Harris():ChannelProcessor< 3, 1, inT, ouT >(),
						weights( NULL ),
						hood( 1 ),
						k( 0.04 )
				{
				}

				~Harris()
				{
					;
				}

				void process( ChannelSet<inT> &in, Channel<ouT> *out );

				void setHood( const int &h )
				{
					hood = h;
				}

				int getHood() const
				{
					return hood;
				}

				void setK( const float &_k )
				{
					k = _k;
				}

				float getK() const
				{
					return k;
				}

				T_weight *getWeights() const
				{
					return weights;
				}

				void setWeight( const T_weight * const w )
				{
					weights = w;
				}

			protected:

				T_weight *weights;
				int hood;
				float k;
		};

		template <class inT>
		void Harris<inT>::process( ChannelSet<inT> &in, Channel<ouT> *out )
		{
			UsingROI r1( in[0] );
			UsingROI r2( in[1] );
			UsingROI r3( in[2] );
			in[0]->minROIDistance( hood );
			in[1]->minROIDistance( hood );
			in[2]->minROIDistance( hood );
			out->fill( 0 );
			out->setCenterdROI( in[0]->getROI().getWidth(), in[0]->getROI().getHeight() );

			int hWidth = hood + hood + 1;
			int norm = hWidth * hWidth;

			T_weight *weights = this->weights;
			if ( !weights )
			{
				weights = new T_weight( hWidth, hWidth );
				getGaussianWeights( *weights, hood );
			}

			typename Channel<inT>::Iterator2D gxxIt = in[0]->getIteratorBegin();
			typename Channel<inT>::Iterator2D gyyIt = in[1]->getIteratorBegin();
			typename Channel<inT>::Iterator2D gxyIt = in[2]->getIteratorBegin();
			typename Channel<ouT>::Iterator2D outIt = out->getIteratorBegin();
			typename Channel<ouT>::Iterator2D outEnd = out->getIteratorEnd();

//			cout << *weights;

			uint x = 0, y = 0;

			for ( ; outIt != outEnd; ++outIt, ++gxxIt, ++gyyIt, ++gxyIt )
			{
				float A = 0;
				float B = 0;
				float C = 0;

				in[0]->getCoors( gxxIt.getPointer(), x, y );

				in[0]->setROI( x - hood, y - hood, hWidth, hWidth );
				in[1]->setROI( x - hood, y - hood, hWidth, hWidth );
				in[2]->setROI( x - hood, y - hood, hWidth, hWidth );

				typename Channel<inT>::Iterator2D gxxHIt1 = in[0]->getIteratorBegin();
				typename Channel<inT>::Iterator2D gyyHIt  = in[1]->getIteratorBegin();
				typename Channel<inT>::Iterator2D gxyHIt  = in[2]->getIteratorBegin();
				typename Channel<inT>::Iterator2D gxxHIt2 = in[0]->getIteratorEnd();
				typename T_weight::iterator wit = weights->begin();

				for ( ; gxxHIt1 != gxxHIt2; ++gxxHIt1, ++gyyHIt, ++gxyHIt, ++wit )
				{
					double w = *wit;
					A += w * *gxxHIt1;
					B += w * *gyyHIt;
					C += w * *gxyHIt;
				}

				A /= norm;
				B /= norm;
				C /= norm;

				float trace = A + B;
				trace *= trace;
				float result = k * trace;
				result = A * B - C * C - result;

				*outIt = ouT( result );
			}

			if ( !this->weights )
			{
				delete weights;
			}
		}


	}

}

#endif
