

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_IMGPROCESSINGHYSTERESISTHRESHOLDER_H
#define ADORE_IMGPROCESSINGHYSTERESISTHRESHOLDER_H

#include <stack>
#include <adrcore/imgprocessing/channelprocessor.h>

namespace adore
{

	namespace core
	{

		/**
		 
		*/
		template <class inT, class ouT>
		class HysteresisThresholder : public ChannelProcessor< 1, 1, inT, ouT >
		{
			public:
				HysteresisThresholder():ChannelProcessor< 1, 1, inT, ouT >(),
						maxima( NULL ),
						low( 0 )
				{
					;
				}

				~HysteresisThresholder()
				{
					;
				}

				void process( Channel<inT> *in, Channel<ouT> *out );

				void setMaxima( std::stack<int> *const m )
				{
					maxima = m;
				}

				std::stack<int> *getMaxima() const
				{
					return maxima;
				}

				void setThreshold( const inT &t )
				{
					low = t;
				}

				inT getThreshold() const
				{
					return low;
				}

			protected:

				std::stack<int> *maxima;
				inT low;
		};

		template <class inT, class ouT>
		void HysteresisThresholder<inT, ouT>::process( Channel<inT> *in, Channel<ouT> *out )
		{
			out->fill( 0 );

			while ( maxima->size() )
			{
				int srcOff = maxima->top();

//				div_t dv = div( srcOff, in->getWStep() );
//				int dstOff = dv.rem + out->getWStep() * dv.quot;
				int dstOff = out->translateOffset( *in, srcOff );

//				int mx, my, sx, sy;
//				in->getCoors( srcOff, mx, my );
//				out->getCoors( dstOff, sx, sy );
//				printf( "(%d, %d), (%d, %d)\n", mx, my, sx, sy );

				maxima->pop();

//				printf( "offsets are %d - %d\n", srcOff, dstOff );

				ouT &dVal = *out->getPointer( dstOff );

				if ( dVal == 0 )
				{
					dVal = imgMax<ouT>();
//					printf( "src/dst: %d/%d\n", srcOff, dstOff );
					for ( int i = 0; i < 8; ++i )//iterate the CORONA indices
					{
						int offSrc = in->getOffsetRel( i, srcOff );
						int offDst = out->getOffsetRel( i, dstOff );
//						printf( "i: %d  -  src/dst: %d/%d\n", i, offSrc, offDst );
						if ( *in->getPointer( offSrc ) > low && *out->getPointer( offDst ) == 0 ) maxima->push( offSrc );
					}
//					printf( "\n\n" );
				}
			}
		}

	}
}

#endif
