

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_COREKERNELBINOM_3_7__3_3_H
#define ADORE_COREKERNELBINOM_3_7__3_3_H

#include <adrcore/imgprocessing/kerneloperatorbase.h>
#include <adrcore/types/aarray.h>
#include <adrcore/util/calctype.h>

namespace adore
{

	namespace core
	{

		/**
		 * binomic kernel with delta_t = 3/7
			 
		*/
		template <class inT, class ouT = inT>
		class KernelBinom_3_7__3_3 : public KernelOperatorBase
		{
			public:
				typedef typename signedT< typename calcT<inT>::type >::type calcT;

				typedef inT img_in_type;
				typedef ouT img_out_type;

			public:
				KernelBinom_3_7__3_3()
				{
					setTopHood( 1 );
					setLeftHood( 1 );
					setBottomHood( 1 );
					setRightHood( 1 );
				}

				~KernelBinom_3_7__3_3()
				{
					;
				}

				/**
				 * Convolves.
				 * The iterator has to iterate through iterators that iterate lines :)
				 * That means:
				 * LineIterator = *IteratorT
				 * value = *LineIterator
				 */
				template<class IteratorT>
				ouT convolve( IteratorT it )
				{
					typedef typename IteratorT::value_type lineIterator;
					lineIterator lit1 = *it++;
					lineIterator lit2 = *it++;
					lineIterator lit3 = *it++;

					calcT a = *lit1; ++lit1;
					calcT b = *lit1; ++lit1;
					calcT c = *lit1; ++lit1;
					calcT d = *lit2; ++lit2;
					calcT e = *lit2; ++lit2;
					calcT f = *lit2; ++lit2;
					calcT g = *lit3; ++lit3;
					calcT h = *lit3; ++lit3;
					calcT i = *lit3; ++lit3;

//					calcT result = calcT( float( a + ( b << 2 ) + c + ( d << 2 ) + ( e << 3 ) + ( f << 2 ) + g + ( h << 2 ) + i ) / 28.0F + 0.5F );
					calcT result = ( a + ( b << 2 ) + c + ( d << 2 ) + ( e << 3 ) + ( f << 2 ) + g + ( h << 2 ) + i ) / 28;

					return ouT( result );
				}

			private:

		};

	}
}

#endif
