

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_COREMULTIITERATOR_H
#define ADORE_COREMULTIITERATOR_H

#include <vector>
#include <adrcore/util/calctype.h>

namespace adore
{

	namespace core
	{

		/**
		 * A convenient iterator that increments multiple iterators
		 * This iterator is read only
		 */
		template <class AnIteratorT>
		class MultiIterator
		{
			public:

				typedef AnIteratorT IteratorT;
				typedef AnIteratorT iterator;

			private:

				typedef vector<IteratorT> CollT;

			public:

				typedef typename IteratorT::value_type imgT;
				typedef typename IteratorT::value_type value_type;
				typedef typename signedT<typename calcT<imgT>::type>::type calcT;

			public:

				MultiIterator( int numDims ):val( numDims = 3 )
				{
					iters.reserve( numDims );
				}

				/**
				 * dereferences iterator i
				 */
				imgT &operator[]( int i )
				{
//					printf( "%d\n", ( int )*iters[i] );
					return *iters[i];
				}

				IteratorT iteratorOfChannel( int i )
				{
					return iters[i];
				}

				/**
				 * dereferences to a Vector. A reference on an internal
				 * vector is returned. That means: Don't use more than one at a time
				 * or use (*iter).copy()
				 */
				const AArray<calcT, 1> &operator *()
				{
					for ( int d = val.size() - 1; d >= 0; --d )
					{
						val( d ) = ( *this )[d];
					}
					return val;
				}

				void operator++()
				{
					for ( int d = iters.size() - 1; d >= 0; --d )
					{
						++iters[d];
					}
				}

				void operator--()
				{
					for ( int d = iters.size() - 1; d >= 0; --d )
					{
						--iters[d];
					}
				}

				void operator+=( int off )
				{
					for ( int d = iters.size() - 1; d >= 0; --d )
					{
						iters[d] += off;
					}
				}

				void operator-=( int off )
				{
					for ( int d = iters.size() - 1; d >= 0; --d )
					{
						iters[d] -= off;
					}
				}


				void add( const IteratorT &iter )
				{
					push_back( iter );
				}

				void push_back( const IteratorT &iter )
				{
					iters.push_back( iter );
				}

				void clear()
				{
					iters.clear();
				}

				int size()
				{
					return iters.size();
				}

			private:
				CollT iters;
				AArray<calcT, 1> val;
		};


	}

}

#endif
