

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin
/// Author: Klaus Haeming


#ifndef KLTTRACKER_H
#define KLTTRACKER_H

#include <map>
#include <vector>

#include <adrcore/viewmatch/tracker.h>
#include "../../../thirdparty/klt/klt.h"
typedef unsigned int uint;

namespace adore
{
	namespace core
	{

		class ViewRelations;
		class NView;
		class ViewBase;
		class FeaturePoint;

		class KltTracker : public Tracker
		{

			public:
				/**
				 * @param mindist min dist between points
				 * @param sm smooth
				 * @param li lighting insensitive
				 */
				KltTracker ( int mindist = 10, int w_width = 7, int w_height = 7,
				             bool sm = true, bool li = false, int check = -1, int minval = 100,
				             uint maxpts = 1000, uint id = 0, int sRange = 15 );

				void track ( NViewMap& map ) { trackPairs ( map ); }

			private:
				const int minDist, winWidth, winHeight;
				const bool smoothing, lighting;
				const int checking;
				const int minValue;
				const int searchRange;
				uint maxPoints;
				uint fid;//feature id

				void setupKlt ( KLT_TrackingContext& context ) const;

				void trackPairs ( NViewMap& map );
				uint maxNeighbours ( const ViewRelations& relations ) const;
				void fillValues ( const KLT_FeatureList& f_list, std::vector<int>& values ) const;
				void fillValues ( const KLT_FeatureList& f_list, const std::vector<int>& old_values,
				                  std::vector<int>& values ) const;
				void insertCorrs ( int id1, int id2, NViewMap& map,
				                   KLT_FeatureTable& f_table, std::map<uint, std::vector<FeaturePoint*> >& fpoints,
				                   const std::vector<int>& values );

				void makeImages ( const NViewMap& map, std::map<ViewT*, imgT*>& images );
				void releaseImages ( std::map<ViewT*, imgT*>& images );

				imgT* toGray ( ViewT* v );
		};

	} // namespace core
} // namespace adore

#endif

