

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef ERRORFUNCTORS3VIEWS_H
#define ERRORFUNCTORS3VIEWS_H

#include <adrcore/types/aarray.h>
#include <adrcore/nview/ttensorcalculatorppp.h>
#include <adrcore/nview/lineartriangulator.h>
#include <adrcore/nview/point3dcorrector.h>
#include <adrcore/types/camera.h>
#include <adrcore/util/mathutil.h>

#include <vector>
#include <boost/tuple/tuple.hpp>

namespace adore
{
	namespace core
	{
		using namespace boost::tuples;

		template <class Solution>
		class TTWithCorrector
		{
				typedef tuple<PointCorr*, FeaturePoint *, FeaturePoint *, FeaturePoint *> CorrT;
				typedef vector< CorrT > CorrsT;
				TTensorCalculator tcalc;
				LinearTriangulator triangle;
				Point3DCorrector correct;
				AVector3D point3D;
				vector<AVector*> points;

			public:
				TTWithCorrector() : tcalc(), triangle(), correct ( 3 ), point3D ( 4 ), points ( 3 ) {}

				TTWithCorrector ( const TTWithCorrector& ) : tcalc(), triangle(), correct ( 3 ), point3D ( 4 ), points ( 3 ) {}

				double operator() ( Solution& sol, double thresh, vector<int>& candInliers, CorrsT& corrs,  int numCandidates, vector<Camera *> cam, bool optimize3D )
				{
					const ATensor3 & tt = sol.getTTensor();

					double error = 0;
					candInliers.clear();

					AVector2D epi1 = tcalc.calculateEpipole ( tt, true );
					AVector2D epi2 = tcalc.calculateEpipole ( tt, false );
					//cam[0] is identity
					cam[1]->getProjMat() = tcalc.calculateCameraMatrix ( tt, epi1, epi2, TTensorCalculator::CAMERA_P1 );
					cam[2]->getProjMat() = tcalc.calculateCameraMatrix ( tt, epi1, epi2, TTensorCalculator::CAMERA_P2 );

					for ( int idx = 0; idx < numCandidates; ++idx )
					{
						points[0] = corrs[idx].get<1>();
						points[1] = corrs[idx].get<2>();
						points[2] = corrs[idx].get<3>();

						triangle ( cam, points, point3D );
						if ( optimize3D ) correct ( cam, points, point3D );

						AVector2D optPt = cam[0]->project ( point3D );
						inhomogenize<2> ( optPt );
						double dist = optPt.sqrDist ( *points[0] );

						optPt = cam[1]->project ( point3D );
						inhomogenize<2> ( optPt );
						dist += optPt.sqrDist ( *points[1] );

						optPt = cam[2]->project ( point3D );
						inhomogenize<2> ( optPt );
						dist += optPt.sqrDist ( *points[2] );

						if ( dist < thresh )
						{
							error += dist;
							candInliers.push_back ( idx );
						}
						else
						{
							error += thresh;
						}
					}
					return error;
				}
		};

		template <class Solution>
		class TTSampson
		{
				typedef tuple<PointCorr*, FeaturePoint *, FeaturePoint *, FeaturePoint *> CorrT;
				typedef vector< CorrT > CorrsT;
				AVector T;
				AMatrix A, J, JJT;
				AVector e, temp;
				double eps;

			public:
				TTSampson() :T ( 27 ), A ( 4, 27 ), J ( 4, 6 ), JJT ( 4, 4 ), e ( 4 ), temp ( 4 ),
						eps ( std::numeric_limits<double>::epsilon() ) {}

				TTSampson ( const TTSampson& ) :T ( 27 ), A ( 4, 27 ), J ( 4, 6 ), JJT ( 4, 4 ), e ( 4 ), temp ( 4 ), eps ( std::numeric_limits<double>::epsilon() ) {}

				double operator() ( Solution& sol, double thresh, vector<int>& candInliers, CorrsT& corrs,  int numCandidates, vector<Camera *>, bool )
				{
					double error = 0;
					candInliers.clear();
					const ATensor3 & tt = sol.getTTensor();

					T =
					        tt ( 0, 0, 0 ), tt ( 0, 0, 1 ), tt ( 0, 0, 2 ), tt ( 0, 1, 0 ), tt ( 0, 1, 1 ), tt ( 0, 1, 2 ),
					        tt ( 0, 2, 0 ), tt ( 0, 2, 1 ), tt ( 0, 2, 2 ), tt ( 1, 0, 0 ), tt ( 1, 0, 1 ), tt ( 1, 0, 2 ),
					        tt ( 1, 1, 0 ), tt ( 1, 1, 1 ), tt ( 1, 1, 2 ), tt ( 1, 2, 0 ), tt ( 1, 2, 1 ), tt ( 1, 2, 2 ),
					        tt ( 2, 0, 0 ), tt ( 2, 0, 1 ), tt ( 2, 0, 2 ), tt ( 2, 1, 0 ), tt ( 2, 1, 1 ), tt ( 2, 1, 2 ),
					        tt ( 2, 2, 0 ), tt ( 2, 2, 1 ), tt ( 2, 2, 2 );


					A = 0; J = 0;
					A ( 0, 18 ) = A ( 1, 19 ) = A ( 2, 21 ) = A ( 3, 22 ) = 1;
					for ( int idx = 0; idx < numCandidates; ++idx )
					{
						const AVector & p1 = *corrs[idx].get<1>();
						const AVector & p2 = *corrs[idx].get<2>();
						const AVector & p3 = *corrs[idx].get<3>();

						A ( 0, 0 ) = A ( 1, 1 ) = A ( 2, 3 ) = A ( 3, 4 ) = p1 ( 0 );
						A ( 0, 2 ) = A ( 2, 5 ) = -p1 ( 0 ) *p3 ( 0 );
						A ( 0, 6 ) = A ( 1, 7 ) = -p1 ( 0 ) *p2 ( 0 );
						A ( 0, 8 ) =   p1 ( 0 ) *p2 ( 0 ) *p3 ( 0 );
						A ( 0, 9 ) = A ( 1, 10 ) = A ( 2, 12 ) = A ( 3, 13 ) = p1 ( 1 );
						A ( 0, 11 ) = A ( 2, 14 ) = -p1 ( 1 ) *p3 ( 0 );
						A ( 0, 15 ) = A ( 1, 16 ) = -p1 ( 1 ) *p2 ( 0 );
						A ( 0, 17 ) =  p1 ( 1 ) *p2 ( 0 ) *p3 ( 0 );
						A ( 0, 20 ) = A ( 2, 23 ) = -p3 ( 0 );
						A ( 0, 24 ) = A ( 1, 25 ) = -p2 ( 0 );
						A ( 0, 26 ) =  p2 ( 0 ) *p3 ( 0 );

						A ( 1, 2 ) = A ( 3, 5 ) = -p1 ( 0 ) *p3 ( 1 );
						A ( 1, 8 ) =   p1 ( 0 ) *p2 ( 0 ) *p3 ( 1 );
						A ( 1, 11 ) = A ( 3, 14 ) = -p1 ( 1 ) *p3 ( 1 );
						A ( 1, 17 ) =  p1 ( 1 ) *p2 ( 0 ) *p3 ( 1 );
						A ( 1, 20 ) = A ( 3, 23 ) = -p3 ( 1 );
						A ( 1, 26 ) = p2 ( 0 ) *p3 ( 1 );

						A ( 2, 6 ) = A ( 3, 7 ) = -p1 ( 0 ) *p2 ( 1 );
						A ( 2, 8 ) =  p1 ( 0 ) *p2 ( 1 ) *p3 ( 0 );
						A ( 2, 15 ) = A ( 3, 16 ) = -p1 ( 1 ) *p2 ( 1 );
						A ( 2, 17 ) = p1 ( 1 ) *p2 ( 1 ) *p3 ( 0 );
						A ( 2, 24 ) = A ( 3, 25 ) = -p2 ( 1 );
						A ( 2, 26 ) = p2 ( 1 ) *p3 ( 0 );

						A ( 3, 8 ) = p1 ( 0 ) *p2 ( 1 ) *p3 ( 1 );
						A ( 3, 17 ) = p1 ( 1 ) *p2 ( 1 ) *p3 ( 1 );
						A ( 3, 26 ) = p2 ( 1 ) *p3 ( 1 );

						matMul2 ( A, T, e );
						if ( dot2 ( e, e ) < eps )
						{
							continue; // dist __IS__ very small
						}

						J ( 0, 0 ) = T ( 0 ) - p3 ( 0 ) *T ( 2 ) - p2 ( 0 ) *T ( 6 ) + p2 ( 0 ) *p3 ( 0 ) *T ( 8 );
						J ( 1, 0 ) = T ( 1 ) - p3 ( 1 ) *T ( 2 ) - p2 ( 0 ) *T ( 7 ) + p2 ( 0 ) *p3 ( 1 ) *T ( 8 );
						J ( 2, 0 ) = T ( 3 ) - p3 ( 0 ) *T ( 5 ) - p2 ( 1 ) *T ( 6 ) + p2 ( 1 ) *p3 ( 0 ) *T ( 8 );
						J ( 3, 0 ) = T ( 4 ) - p3 ( 1 ) *T ( 5 ) - p2 ( 1 ) *T ( 7 ) + p2 ( 1 ) *p3 ( 1 ) *T ( 8 );
						J ( 0, 1 ) = T ( 9 )  - p3 ( 0 ) *T ( 11 ) - p2 ( 0 ) *T ( 15 ) + p2 ( 0 ) *p3 ( 0 ) *T ( 17 );
						J ( 1, 1 ) = T ( 10 ) - p3 ( 1 ) *T ( 11 ) - p2 ( 0 ) *T ( 16 ) + p2 ( 0 ) *p3 ( 1 ) *T ( 17 );
						J ( 2, 1 ) = T ( 12 ) - p3 ( 0 ) *T ( 14 ) - p2 ( 1 ) *T ( 15 ) + p2 ( 1 ) *p3 ( 0 ) *T ( 17 );
						J ( 3, 1 ) = T ( 13 ) - p3 ( 1 ) *T ( 14 ) - p2 ( 1 ) *T ( 16 ) + p2 ( 1 ) *p3 ( 1 ) *T ( 17 );
						J ( 0, 2 ) = -p1 ( 0 ) *T ( 6 ) + p1 ( 0 ) *p3 ( 0 ) *T ( 8 ) - p1 ( 1 ) *T ( 15 ) + p1 ( 1 ) *p3 ( 0 ) *T ( 17 ) - T ( 24 ) + p3 ( 0 ) *T ( 26 );
						J ( 1, 2 ) = -p1 ( 0 ) *T ( 7 ) + p1 ( 0 ) *p3 ( 1 ) *T ( 8 ) - p1 ( 1 ) *T ( 16 ) + p1 ( 1 ) *p3 ( 1 ) *T ( 17 ) - T ( 25 ) + p3 ( 1 ) *T ( 26 );
						J ( 2, 3 ) = J ( 0, 2 ); J ( 3, 3 ) = J ( 1, 2 );
						J ( 0, 4 ) = J ( 1, 5 ) = -p1 ( 0 ) *T ( 2 ) + p1 ( 0 ) *p2 ( 0 ) *T ( 8 ) - p1 ( 1 ) *T ( 11 ) + p1 ( 1 ) *p2 ( 0 ) *T ( 17 ) - T ( 20 ) + p2 ( 0 ) *T ( 26 );
						J ( 2, 4 ) = J ( 3, 5 ) = -p1 ( 0 ) *T ( 5 ) + p1 ( 0 ) *p2 ( 1 ) *T ( 8 ) - p1 ( 1 ) *T ( 14 ) + p1 ( 1 ) *p2 ( 1 ) *T ( 17 ) - T ( 23 ) + p2 ( 1 ) *T ( 26 );

						matMul2_BT ( J, J, JJT );
						if ( false == invert ( JJT ) )
						{
							continue; // dist __IS__ very small
						}
						matMul2_AT ( JJT, e, temp );

						double dist = dot2 ( temp, e );

						if ( dist < thresh )
						{
							error += dist;
							candInliers.push_back ( idx );
						}
						else error += thresh;
					}
					return error;
				}
		};

	}
}

#endif // ERRORFUNCTORS3VIEWS_H
