

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "filterransac2viewhelper.h"
#include "errorfunctors2views.h"

#define DBGFILTER DBGALL

namespace adore
{

	namespace core
	{

		void get2ndEpi ( AMatrix &fmat, AVector &epipole, SVDStoraged &store3 )
		{
			transInPlace ( fmat );
			FMatrixCalculator::calculateEpipolarPoint ( fmat, epipole, store3 );
			transInPlace ( fmat );
		}

		template<class Solution, template<typename> class ErrorFn>
		int FilterRANSAC2ViewHelper<Solution, ErrorFn>::
		isOriented ( Solution &sol, AVector &epiPoint, const vector<int> &in )
		{
			AMatrix &fmat = sol.getFMat();
			int score = 0;

			for ( int i = 0; i < ( int ) in.size(); ++i )
			{
				AVector tmpPoint1 = sol.getCorrectedPoint ( *corrs[in[i]].get<1>() );
				AVector tmpPoint2 = sol.getCorrectedPoint ( *corrs[in[i]].get<2>() );
				bool o = FMatrixCalculator::isOriented ( fmat, epiPoint, tmpPoint1, tmpPoint2 );
				score += o ? 1 : -1;
			}

			if ( score < 0 )
			{
				DBGOUT ( DBGFILTER, "FilterRANSAC2ViewFMatrix::isOriented: score was negative: flipping epipole\n" );
				score = -score;
				epiPoint *= -1;
			}

			return score;
		}

		template<class Solution, template<typename> class ErrorFn>
		bool FilterRANSAC2ViewHelper<Solution, ErrorFn>::
		updateSolution ( Solution &sol, const vector<int> &base, float thresh, bool opt )
		{
			//threshold on min number of correct oriented points
			//if applying sample, all must fulfill the orientation constraint
			//if optimizing, this threshold is less important
			int oThresh = opt ? 1 : numSamples;

			AMatrix &fmat = sol.getFMat();
			get2ndEpi ( fmat, epiPoint, store3 );
			int oScore = isOriented ( sol, epiPoint, base );
			DBGOUT ( DBGFILTER, "orientation score was %d\n", oScore );

			if ( oScore < oThresh )
			{
				DBGOUT ( DBGFILTER, "fmat not oriented\n" );
				return false;
			}

			double err = computeErrors ( sol, epiPoint, thresh, candInliers, corrs, client->getNumCandidates() );

			DBGErr = err;

			int in = ( int ) candInliers.size();
			bool okay = err < client->getError();

			if ( okay )
			{
				DBGOUT ( DBGFILTER, "good enough (in: %d/%d, err: %.10f/%.10f)\n", in, client->getNumIn(), err, client->getError() );
				bestSolution = sol;
				client->setNumIn ( in );
				client->setError ( err );
				inliers.resize ( candInliers.size() );
				copy ( candInliers.begin(), candInliers.end(), inliers.begin() );
				return true;
			}
			else
			{
				DBGOUT ( DBGFILTER, "NOT good enough (in: %d/%d, err: %.10f/%.10f)\n", in, client->getNumIn(), err, client->getError() );
			}

			return false;
		}

//		instantiation
		template class FilterRANSAC2ViewHelper<FMatSolution, FundamentalErrorsWithCorrector >;
		template class FilterRANSAC2ViewHelper<FMatSolution, SampsonF >;
		template class FilterRANSAC2ViewHelper<RDFMatSolution, FundamentalErrorsWithCorrector >;

	}//ns
}
