

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "filterransacprojectivityhelper.h"
#include "errorfunctors2views.h"

#define DBGFILTER DBGALL

namespace adore
{

	namespace core
	{

		template<class Solution, class ErrorFn>
		bool FilterRANSACProjectivityHelper<Solution, ErrorFn>::
		updateSolution ( Solution &sol, const vector<int> &base, float thresh, bool opt )
		{

			double err = computeErrors( sol.getH(), thresh, candInliers, corrs, client->getNumCandidates() );
			DBGErr = err;

			int in = ( int ) candInliers.size();
			bool okay = ( err < client->getError() );

			if ( okay )
			{
				DBGOUT ( DBGFILTER, "good enough (in: %d/%d, err: %.10f/%.10f)\n", in, client->getNumIn(), err, client->getError() );
				bestSolution = sol;
				client->setNumIn ( in );
				client->setError ( err );
				inliers.resize ( candInliers.size() );
				copy ( candInliers.begin(), candInliers.end(), inliers.begin() );
				return true;
			}
			else
			{
				DBGOUT ( DBGFILTER, "NOT good enough (in: %d/%d, err: %.10f/%.10f)\n", in, client->getNumIn(), err, client->getError() );
			}

			return false;
		}

//		instantiation
		template class FilterRANSACProjectivityHelper<HSolution, ProjectiveErrorsWithCorrector >;
		template class FilterRANSACProjectivityHelper<HSolutionC, ProjectiveErrorsWithCorrector >;
		template class FilterRANSACProjectivityHelper<HSolution, SampsonH >;
		template class FilterRANSACProjectivityHelper<HSolutionC, SampsonH >;

	}//ns
}
