

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_BABUNDLEADJUSTERSIMPLE_H
#define ADORE_BABUNDLEADJUSTERSIMPLE_H

#include <adrcore/types/camera.h>
#include <adrcore/minimization/lmsparsesimple.h>

#include <adrcore/parameterization/simplecamparam.h>
#include <adrcore/parameterization/zfixedpointparam.h>
#include <adrcore/parameterization/angleaxisfocalcamparam.h>
#include <adrcore/parameterization/simple3dpointparam.h>
#include <adrcore/parameterization/quaternioncamparam.h>

#include <vector>

namespace adore
{
	namespace core
	{

		template <typename Switch> class Parameterizer
		{
		public:
			void toParams(vector<Camera *> & cams, vector<AVector3D*> & pts3D,
			  vector<AVector> & lineCams, vector<AVector> & linePts3D) const;

			void fromParams(vector<Camera *> & cams, vector<AVector3D*> & pts3D,
			  vector<AVector> & lineCams, vector<AVector> & linePts3D) const;

			void create ( vector<AVector> & vec, uint num, uint dim ) const
			{
				vec.resize ( num );
				for ( vector<AVector>::iterator it = vec.begin(); it != vec.end(); ++it )
				{
					( *it ).resize( dim );
				}
			}
		};

		template<typename Switch> inline void Parameterizer<Switch>::toParams(
                        vector<Camera *> &, vector<AVector3D*> &,
			vector<AVector> &, vector<AVector> &) const
		{
			assert(false);
		}

		template<> inline void Parameterizer<BAMinFunctionSimple>::toParams(
                        vector<Camera *> & cams, vector<AVector3D*> & pts3D,
			vector<AVector> & lineCams, vector<AVector> & linePts3D) const
		{
			SimpleCamParam cParam;
			ZFixedPointParam pParam;

			create ( lineCams, cams.size(), cParam.getSize() );
			create ( linePts3D, pts3D.size(), pParam.getSize() );

			cParam.toParam ( cams, lineCams );
			pParam.toParam ( pts3D, linePts3D );
		}

		template<> inline void Parameterizer<BAMinFunctionSimple_RTF>::toParams(
                        vector<Camera *> & cams, vector<AVector3D*> & pts3D,
			vector<AVector> & lineCams, vector<AVector> & linePts3D) const
		{
			AngleAxisFocalCamParam cParam;
			Simple3DPointParam pParam;

			create ( lineCams, cams.size(), cParam.getSize() );
			create ( linePts3D, pts3D.size(), pParam.getSize() );

			cParam.toParam ( cams, lineCams );
			pParam.toParam ( pts3D, linePts3D );
		}

		template<> inline void Parameterizer<BAMinFunctionSimple_Quat_Full>::toParams(
                        vector<Camera *> & cams, vector<AVector3D*> & pts3D,
			vector<AVector> & lineCams, vector<AVector> & linePts3D) const
		{
			QuaternionCamParam cParam;
			Simple3DPointParam pParam;

			create ( lineCams, cams.size(), cParam.getSize() );
			create ( linePts3D, pts3D.size(), pParam.getSize() );

			cParam.toParam ( cams, lineCams );
			pParam.toParam ( pts3D, linePts3D );
		}

		template<> inline void Parameterizer<BAMinFunctionSimple_Quat_F>::toParams(
                        vector<Camera *> & cams, vector<AVector3D*> & pts3D,
			vector<AVector> & lineCams, vector<AVector> & linePts3D) const
		{
			QuaternionCamParam_F cParam;
			Simple3DPointParam pParam;

			create ( lineCams, cams.size(), cParam.getSize() );
			create ( linePts3D, pts3D.size(), pParam.getSize() );

			cParam.toParam ( cams, lineCams );
			pParam.toParam ( pts3D, linePts3D );
		}

		template<typename Switch> inline void Parameterizer<Switch>::fromParams(
                        vector<Camera *> &, vector<AVector3D*> &,
			vector<AVector> &, vector<AVector> &) const
		{
			assert(false);
		}

		template<> inline void Parameterizer<BAMinFunctionSimple>::fromParams(
                        vector<Camera *> & cams, vector<AVector3D*> & pts3D,
			vector<AVector> & lineCams, vector<AVector> & linePts3D) const
		{
			SimpleCamParam cParam;
			ZFixedPointParam pParam;
			cParam.fromParam ( lineCams, cams );
			pParam.fromParam ( linePts3D, pts3D );
		}

		template<> inline void Parameterizer<BAMinFunctionSimple_RTF>::fromParams(
                        vector<Camera *> & cams, vector<AVector3D*> & pts3D,
			vector<AVector> & lineCams, vector<AVector> & linePts3D) const
		{
			AngleAxisFocalCamParam cParam;
			Simple3DPointParam pParam;
			cParam.fromParam ( lineCams, cams );
			pParam.fromParam ( linePts3D, pts3D );
		}

		template<> inline void Parameterizer<BAMinFunctionSimple_Quat_Full>::fromParams(
                        vector<Camera *> & cams, vector<AVector3D*> & pts3D,
			vector<AVector> & lineCams, vector<AVector> & linePts3D) const
		{
			QuaternionCamParam cParam;
			Simple3DPointParam pParam;
			cParam.fromParam ( lineCams, cams );
			pParam.fromParam ( linePts3D, pts3D );
		}

		template<> inline void Parameterizer<BAMinFunctionSimple_Quat_F>::fromParams(
                        vector<Camera *> & cams, vector<AVector3D*> & pts3D,
			vector<AVector> & lineCams, vector<AVector> & linePts3D) const
		{
			QuaternionCamParam_F cParam;
			Simple3DPointParam pParam;
			cParam.fromParam ( lineCams, cams );
			pParam.fromParam ( linePts3D, pts3D );
		}

		/**
		 * This class is a wrapper around the sparse LM algo.
		 * It essentially reformats the input, solves it and reformats it back again.
		 * missing 2D points should be no problem.
		 *  
		*/
		template<class ErrorFuncT, class MinFunctionT>
		class BundleAdjusterSimple
		{
			protected:

				LMSparseSimple<ErrorFuncT, MinFunctionT> lm;

				vector<AVector> lineCams;
				vector<AVector> linePts3D;
				vector<int> missing;
				int maxIterations;
			public:

				/**
				 * Constructor.
				 */
				BundleAdjusterSimple(int max_it = 10000) :
					missing ( 0 ),
					maxIterations(max_it)
				{
				}

				BundleAdjusterSimple( const BundleAdjusterSimple &ba ) :
						missing ( ba.missing )//TODO:correct?
				{
				}

				/**
				 * Adjusts the cameras and the 3D-points to find a maximum likelihood estimation that minimizes
				 * the reprojection error according to the given 2D-points.
				 * this version can not cope with missing 2D points
				 * @param cams The cameras
				 * @param pts3D The 3D-points
				 * @param pts The 2D-points
				 */
				void modify ( vector<Camera *> &cams, vector<AVector3D*> &pts3D, ErrorFuncT &errorFunc, double thresh = FLT_EPSILON );

				/**
				 * returns the error of the last call of this algorithm
				 * @return the error
				 */
				double getLastError() const
				{
					return lm.getLastError();
				}

				/**
				 * returns the number of iterations needed by the last call of this algorithm.
				 * @return the number of iterations
				 */
				int getLastIterations() const
				{
					return lm.getLastIterations();
				}

				/**
				 * returns the error before starting of minimization
				 * @return the error
				 */
				double getInitialError() const
				{
					return lm.getInitialError();
				}

				int getMaxIterations() const { return maxIterations; }

				void setMaxIterations(int max_it) { maxIterations = max_it; }

			protected:

				void setMissingIndices ( const vector< int >& theValue )
				{
					missing = theValue;
				}

				void create ( vector<AVector> & vec, uint num, uint dim )
				{
					vec.resize ( num );
					for ( vector<AVector>::iterator it = vec.begin(); it != vec.end(); ++it )
					{
						( *it ).resize( dim );
					}
				}
		};


		template<class ErrorFuncT, class MinFunctionT>
		void BundleAdjusterSimple<ErrorFuncT, MinFunctionT>::modify (
		        vector<Camera *> &cams,
                        vector<AVector3D*> &pts3D,
		        ErrorFuncT &errorFunc, double thresh )
		{
			if ( cams.size() < 1 || pts3D.size() < 1 ) return;

			if ( cams.size() != lineCams.size() || pts3D.size() != linePts3D.size() )
			{
				DBGOUT ( DBGNORMAL, "BundleAdjusterSimple: reinitializing the lm-algo\n" );
//				lm.init( cams.size(), 12, lastCamFixed ? 1 : 0, pts3D.size(), 3, 2 );
				lm.init ( cams.size(), pts3D.size() );
			}
//			else
//			{
//				printf( "Vecs: %d/%d; %d/%d\n", cams.size(), lineCams.size(), pts3D.size(), linePts3D.size() );
//			}


//			int numPts2D = ( cams.size()  + ( lastCamFixed ? 1 : 0 ) ) * pts3D.size();


			Parameterizer<MinFunctionT> parameterizer;

			parameterizer.toParams(cams, pts3D, lineCams, linePts3D);

			lm.solve ( lineCams, linePts3D, missing, errorFunc, thresh, maxIterations );

			DBGOUT( DBGNORMAL, "BundleAdjusterSimple: Residual: %.20f - %.20f after %d iterations\n", lm.getInitialError(), lm.getLastError(), lm.getLastIterations() );

			parameterizer.fromParams(cams, pts3D, lineCams, linePts3D);

		}

	}

}

#endif
