

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_MINIMIZATIONCOVARSQUARER2_H
#define ADORE_MINIMIZATIONCOVARSQUARER2_H

#include <adrcore/types/aarray.h>
#include <adrcore/minimization/lmconstants.h>
#include<adrcore/util/matmul2.h>

namespace adore
{

	namespace core
	{

		/**
			 
		*/
		template <class InvCovarT>
		class CovarSquarer2
		{
				typedef typename InvCovarT::scalar_type number_t;

			public:

				void covarProduct( const AArray<number_t, 2> &A, const InvCovarT &invCovar, AArray<number_t, 2> &result ) const
				{
					covarProduct( A, invCovar, A, result );
				}

				void covarProduct( const AArray<number_t, 2> &A, const InvCovarT &invCovar, const AArray<number_t, 2> &B, AArray<number_t, 2> &result ) const
				{
					ADORE_UNUSED( invCovar );

//					std::cout << "Covar Squarer: covar not used" << endl;
					matMul2_AT( A, B, result );
				}

				void covarProduct( const AArray<number_t, 2> &A, const InvCovarT &invCovar, const AArray<number_t, 1> &B, AArray<number_t, 1> &result ) const
				{
					ADORE_UNUSED( invCovar );

					matMul2_AT( A, B, result );
				}
		};

		template <class number_t>
		class CovarSquarer2<AArray<number_t, 2> >
		{
			public:

				void covarProduct( const AArray<number_t, 2> &A,
				                   const AArray<number_t, 2> &invCovar, AArray<number_t, 2> &result ) const
				{
					covarProduct( A, invCovar, A, result );
				}

				void covarProduct( const AArray<number_t, 2> &A,
				                   const AArray<number_t, 2> &invCovar,
				                   const AArray<number_t, 2> &B, AArray<number_t, 2> &result ) const
				{
					AArray<number_t, 2> covar = matMul( invCovar, B );
					matMul2_AT( A, covar, result );
				}

				void covarProduct( const AArray<number_t, 2> &A,
				                   const AArray<number_t, 2> &invCovar,
				                   const AArray<number_t, 1> &B, AArray<number_t, 1> &result ) const
				{
					AArray<number_t, 1> covar = matMul( invCovar, B );
					matMul2_AT( A, covar, result );
				}
		};

		template <class number_t>
		class CovarSquarer2<AArray<number_t, 1> >
		{
			public:
				void covarProduct( const AArray<number_t, 2> &A,
				                   const AArray<number_t, 1> &invCovar, AArray<number_t, 2> &result ) const
				{
					covarProduct( A, invCovar, result );
				}

				void covarProduct( const AArray<number_t, 2> &A,
				                   const AArray<number_t, 1> &invCovar,
				                   const AArray<number_t, 2> &B, AArray<number_t, 2> &result ) const
				{
					AArray<number_t, 2> covar( B.rows(), invCovar.cols() );
					covar = B;
					//multiply diagonal elements
					for ( int row = 0; row < invCovar.rows(); ++row )
						covar( row, row ) = covar( row, row ) * invCovar( row );

					matMul2_AT( A, covar, result );
				}

				void covarProduct( const AArray<number_t, 2> &A,
				                   const AArray<number_t, 1> &invCovar,
				                   const AArray<number_t, 1> &B, AArray<number_t, 1> &result ) const
				{
					AArray<number_t, 1> covar( B.rows() );
					covar = B;
					//multiply diagonal elements
					for ( int row = 0; row < invCovar.rows(); ++row )
						covar( row ) = covar( row ) * invCovar( row );

					matMul2_AT( A, covar, result );
				}
		};

	}

}

#endif
