

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "automerger.h"

namespace adore
{

	namespace core
	{

		bool AutoMerger::merge ( NViewMap &nviewmap, ViewKnitter &knit, const bool delSourceNViews )
		{
			if ( knit.done() ) return true;

			typedef ViewKnitter::Tuple Tuple;
			typedef ViewKnitter::VTuple VTuple;

			while ( !knit.done() )
			{
				DBGOUT ( DBGNORMAL, "AutoMerger: pos in plan: %d/%d\n", knit.getPos(), knit.numSteps() );
				bool okay = merger.merge ( nviewmap, knit, delSourceNViews );
				if ( !okay ) return false;

				//polish the merging results
				for ( ViewKnitter::const_iterator it = knit.begin(); it != knit.end(); ++it )
				{
					const Tuple &tup = *it;
					VTuple *inAorB = tup.get<2>();
					NView *nv = nviewmap.getNView( inAorB->begin(), inAorB->end() );

					bpr2.modify( *nv );
					//DBGOUT(DBGNORMAL, "AutoMerger: BPR2 removed %d/%d corrs\n", bpr2.getRemoved(), nv->getNumPointCorrs());
					printf( "AutoMerger: BPR2 removed %d/%d corrs\n", bpr2.getRemoved(), nv->getNumPointCorrs() );

					if ( nv->getNumViews() > 3 )
					{
						//bpr3.modify(*nv);
						//DBGOUT(DBGNORMAL, "AutoMerger: BPR3 removed %d/%d corrs\n", bpr3.getRemoved(), nv->getNumPointCorrs());
						//printf("AutoMerger: BPR3 removed %d/%d corrs\n", bpr3.getRemoved(), nv->getNumPointCorrs());
						printf( "AutoMerger: BPR3 disabled\n" );
					}

					ba.modify( *nv );
					DBGOUT( DBGNORMAL, "AutoMerger: BA after merging %d views needs %d iterations\nresidual(before - after): %.30f - %.30f\n", nv->getNumViews(), ba.getLastIterations(), ba.getInitialError(), ba.getLastError() );
				}

				knit.next();
			}
			return true;
		}

	}

}
