

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_NVIEWFMATRIXCALCULATORPVP_H
#define ADORE_NVIEWFMATRIXCALCULATORPVP_H

#include <vector>
#include <adrcore/util/debug.h>
#include <adrcore/types/camera.h>
#include <adrcore/types/rect.h>
#include <adrcore/types/view.h>
#include <adrcore/types/pointcorr.h>
#include <adrcore/nview/lineartriangulator.h>
#include <adrcore/nview/correspondencecorrector.h>
#include <adrcore/nview/fmatrixcalculator.h>

namespace adore
{
	namespace core
	{

		using namespace std;

		/**
		 * This class calculates the fundamental matix/matrices for a Pair of Vectors of Points (PVP).
		 * The point's coordinates must be accessible via point[0] and point[1].
		 * the vector elements must also be accessible via the index operator [].
		 *
		 * example: PointT=double* and IteratorT=vector&lt;double *&gt;::iterator
		 *  
		*/
		class FMatrixCalculatorPVP : public FMatrixCalculator
		{
			public:

				/**
				 * This Constructor intializes everything you will need to calculate a fundamental matrix.
				 */
				FMatrixCalculatorPVP() : FMatrixCalculator()
				{
				}

				/**
				 * Calculates the fundamental matrix using the information provided via init() or the constructor.
				 */
				template <class CorrIterator, class ViewIterator >
				int calculateFundamentalMatrix ( CorrIterator it, int size, ViewIterator vit )
				{
					ViewBase *v1 = *vit; ++vit;
					ViewBase *v2 = *vit; --vit;//vit is used later
					CorrIterator cit = it;

					int row = 0;
					FMatrixCalculator::init ( size );
					for ( int i = 0; i < size; ++i, ++cit )
					{
						PointCorr *corr = *cit;
						if ( !corr ) continue;
						FeaturePoint *fp1 = corr->getFeature ( v1 );
						if ( !fp1 ) continue;
						FeaturePoint *fp2 = corr->getFeature ( v2 );
						if ( !fp2 ) continue;

						if ( ( *fp1 ) ( 2 ) != 1 || ( *fp2 ) ( 2 ) != 1 )
						{
							cout << "fp1: " << ( *fp1 ) << ", fp2: " << ( *fp2 ) << endl;
							assert( ( *fp1 ) ( 2 ) == 1 );
							assert( ( *fp2 ) ( 2 ) == 1 );
						}
						setRowFromPointCorrespondences ( row, ( *fp1 ) ( 0 ), ( *fp1 ) ( 1 ), ( *fp2 ) ( 0 ), ( *fp2 ) ( 1 ) ); ++row;
					}
					assert( row > 6 );
					FMatrixCalculator::init ( row );//resize

					int ret = calculateFundamentalMatrixBase();

//					if ( bundleAdjust && ret > 0 )
//					{
//						bundleAdjustFMat( it, size, vit, result1, *this );
//					}

					return ret;
				}

			private:

				/**
				    Given two corresponding points, one row of the matrix defining the
				    elements of the fundamental matrix by Af=0 has to be:
				    (x'x) (x'y) (x') (y'x) (y'y) (y') (x) (y) (1)

				    This functions sets one row of matrix A.
				*/
				void setRowFromPointCorrespondences ( int row, double x1, double y1, double x2, double y2 )
				{
					ptsMat ( row, 0 ) =  x1 * x2;
					ptsMat ( row, 1 ) =  y1 * x2;
					ptsMat ( row, 2 ) =       x2;
					ptsMat ( row, 3 ) =  x1 * y2;
					ptsMat ( row, 4 ) =  y1 * y2;
					ptsMat ( row, 5 ) =       y2;
					ptsMat ( row, 6 ) =  x1;
					ptsMat ( row, 7 ) =  y1;
					ptsMat ( row, 8 ) =  1;
				}

		};

	}
}

#endif
