

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef GLOBALBUNDLEADJUSTER_H
#define GLOBALBUNDLEADJUSTER_H

#include <adrcore/types/nviewmap.h>

#include <boost/tuple/tuple.hpp>

#include <vector>

namespace adore {
	namespace core {

		/**
		 *  
		 */
		class GlobalBundleAdjuster
		{
		public:
			/**
			 * @param num_initial_it number of iterations of initial guess(min 8)
			 * @param thresh_ratio threshold after initial guess = threshold * thresh_ratio
			 * @param use_2_corrs use 2-views correspondences
			 */
			GlobalBundleAdjuster(uint num_initial_it, double thresh_ratio, bool use_2_corrs);

			/**
			 * @param map nviews and views assumed to be normalized. map assumed to have 2-Views and 3-Views with
			 * calculated f. matrices and t. tensors.
			 */
			void adjust(NViewMap& map);

		private:
			typedef boost::tuples::tuple<double, double, double, double> Pt4_t;
			typedef boost::tuples::tuple<Pt4_t, Pt4_t, Pt4_t> C3x4_t;

			uint m_num_initial_it;
			double m_thresh_ratio;
			bool m_use_2_corrs;

			// place all overlaping NViews(tripples) with all correspondences in one NView
			NView * stitchAllTripplets(NViewMap& map);

			// create initial identity cameras and 3d-points(add to all 2d-points z-dimension and set z value to 1.0)
			void init3D(NView * nview);

			// set subsequently z-values to 1.0 and -1.0 and check errors for each z initial value.
			// if the first guess has smaller error, restore points and cameras adjusted by 1. run.
			void storeInitialGuessAndReinit(NView * nview, std::vector<Pt4_t>& first_points, std::vector<C3x4_t>& first_cams);
			void replayInitialGuess(NView * nview, const std::vector<Pt4_t>& first_points, const std::vector<C3x4_t>& first_cams);

			// remove points with reprojection error > threshold
			void removeBadPoints(NView * nview, double threshold);

		public:
			// 'good' value for num_initial_it
			static uint getGoodInitialIt() { return 16u; }

			// 'good' value for thresh_ratio
			static double getGoodThreshRatio() { return 0.8; }
		};

	} // core
} // adore

#endif // GLOBALBUNDLEADJUSTER_H
