

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_CORELINEARQCALCULATORFL_H
#define ADORE_CORELINEARQCALCULATORFL_H

#include <adrcore/nview/autocalibratorcommon.h>

namespace adore
{

	namespace core
	{

		/**
			Calculates Q lineary by assuming a variable focal length and general cameras
			 
		*/
		class LinearQCalculatorFL
		{
			private:
				static int getRows ( const int numCams )
				{
					return 4 * numCams;
				}

			private:
				const int rows;
				const int cols;

				AMatrix left;
				AVector right;
				AVector x;
				SVDSolveStorage<double, 1> store;

			public:
				LinearQCalculatorFL ( int numCams ) :
						rows ( getRows ( numCams ) ),
						cols ( 9 ),
						left ( rows, cols ),
						right ( rows ),
						x ( cols ),
						store ( left, right )
				{
					;
				}

				int calcDualQuadric ( NView::CameraIterator camIt, const int numCams, AMatrix &Q1, AMatrix &Q2 );
				/**
				 * fills in the values for a linear equation to solve autocalibration
				 */
				void getLinearEquation ( NView::CameraIterator camIt, const int numCams, AMatrix &left, AVector &right );

				/**
				 * returns 4 lambdas that satify det(Q1 - l * Q2) = 0
				 */
				int getDet0Solutions ( AMatrix &Q1, AMatrix &Q2, double l[4] );

		};

	}

}

#endif
