

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "mergerhminfunc.h"
#include <adrcore/util/mathutil.h>

namespace adore
{
	namespace core
	{

		MergerHMinFunc::MergerHMinFunc ( vector< vector< AVector2D* > > *p2D ) :
				method ( HMIN_SINGLE ),
				pts3D1 ( NULL ),
				pts3D2 ( NULL ),
				cams1 ( NULL ),
				cams2 ( NULL ),
				H44 ( 4, 4 ),
				invH44 ( 4, 4 )
		{
			setFeats ( p2D );
//			reproFunc = &MergerHMinFunc::reprojection;
		}


		AVector MergerHMinFunc::operator() ( const AVector &H ) const
		{
			assert ( pts3D1 );
			assert ( pts3D2 );
			assert ( cams1 );
			assert ( cams2 );
			AVector result( 2 * pts3D2->size() * cams1->size() );
			this->operator()( H, result );
			return result;
		}

		void MergerHMinFunc::operator() ( const AVector &H, AVector &result ) const
		{
			assert ( pts3D1 );
			assert ( pts3D2 );
			assert ( cams1 );
			assert ( cams2 );

			int idx = 0;
			for ( int row = 0; row < 4; ++row )
				for ( int col = 0; col < 4; ++col )
				{
					H44 ( row, col ) = H ( idx++ );
				}

			switch ( method )
			{
				case HMIN_SINGLE:
					reprojection ( H44, result );
					return;
				case HMIN_BOTH:
					reprojectionBoth ( H44, result );
					return;
				default:
					exit ( 1 );
			}
		}

		void MergerHMinFunc::reprojection ( const AMatrix &H, AVector &result ) const
		{
			assert( ( int )result.size() == ( int )( 2 * pts3D2->size() * cams1->size() ) );
			int row = 0;

			for ( uint p = 0; p < pts3D2->size(); ++p )
			{
				for ( uint c = 0; c < cams1->size(); ++c )
				{
					double dx1, dy1;
					if ( ( *feats ) [c][p] )
					{
//						AMatrix PH = matMul ( * ( *cams1 ) [c], H );
						AMatrix PH(3, 4);
						matMul2 ( * ( *cams1 ) [c], H, PH );
//						AVector3D vector3 = matMul ( PH, * ( *pts3D2 ) [p] );//project
						AVector3D vector3(3);
						matMul2 ( PH, * ( *pts3D2 ) [p], vector3 );//project

						double fz = ( * ( *feats ) [c][p] ) ( 2 );
						assert ( fz != 0 );
						double fx = ( * ( *feats ) [c][p] ) ( 0 ) / fz;
						double fy = ( * ( *feats ) [c][p] ) ( 1 ) / fz;

						double z = vector3 ( 2 );
						assert ( z != 0 );

						dx1 = vector3 ( 0 ) / z - fx;
						dx1 *= dx1;
						dy1 = vector3 ( 1 ) / z - fy;
						dy1 *= dy1;
					}
					else
					{
						dx1 = dy1 = 0;
					}

					result ( row++ ) =  dx1;
					result ( row++ ) =  dy1;
				}
			}
		}

		void MergerHMinFunc::reprojectionBoth ( const AMatrix &H, AVector &result ) const
		{
			assert( ( int )result.size() == ( int )( 2 * pts3D2->size() * cams1->size() ) );
			invH44 = H;
			invert ( invH44 );

			int row = 0;

			AMatrix PH(3, 4);
			AVector3D vector3(3);

			for ( uint p = 0; p < pts3D2->size(); ++p )
			{
				for ( uint c = 0; c < cams1->size(); ++c )
				{
					matMul2 ( * ( *cams1 ) [c], H, PH );
					matMul2 ( PH, * ( *pts3D2 ) [p], vector3 );

					double fz = ( * ( *feats ) [c][p] ) ( 2 );
					assert ( fz != 0 );
					double fx = ( * ( *feats ) [c][p] ) ( 0 ) / fz;
					double fy = ( * ( *feats ) [c][p] ) ( 1 ) / fz;

					double z = vector3 ( 2 );
					assert ( z != 0 );

					double dx1 = vector3 ( 0 ) / z - fx;
					dx1 *= dx1;
					double dy1 = vector3 ( 1 ) / z - fy;
					dy1 *= dy1;

					matMul2 ( * ( *cams2 ) [c], invH44, PH );
					matMul2 ( PH, * ( *pts3D1 ) [p], vector3 );

					z = vector3 ( 2 );
					assert ( z != 0 );

					double dx2 = vector3 ( 0 ) / z - fx;
					dx2 *= dx2;
					double dy2 = vector3 ( 1 ) / z - fy;
					dy2 *= dy2;

					result ( row++ ) =  dx1 + dx2;
					result ( row++ ) =  dy1 + dy2;
				}
			}
		}


	}
}
