

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "parallelautomerger.h"

namespace adore
{

	namespace core
	{

		bool ParallelAutoMerger::merge ( NViewMap &map, ViewKnitter &knit, const bool delSourceNViews )
		{
			if ( knit.done() ) return true;

			typedef ViewKnitter::Tuple Tuple;
			typedef ViewKnitter::VTuple VTuple;

			while ( !knit.done() )
			{
				DBGOUT ( DBGNORMAL, "AutoMerger: pos in plan: %d/%d\n", knit.getPos(), knit.numSteps() );

				//polish the merging results
				set<NView *> sourceNViews;
				for ( ViewKnitter::const_iterator it = knit.begin(); it != knit.end(); ++it )
				{
					const Tuple &tup = *it;

					VTuple *inA = tup.get<0>();
					VTuple *inB = tup.get<1>();
					VTuple *inAorB = tup.get<2>();

					NView *nv1 = map.getNView( inA->begin(), inA->end() );
					assert( nv1 );
					NView *nv2 = map.getNView( inB->begin(), inB->end() );
					assert( nv2 );
					NView *result = map.createNView( inAorB->begin(), inAorB->end() );
					assert( result );

					JobThread *t = pool.getIdle();
					MJob *j = static_cast<MJob *> ( t->getJob() );
					j->set( nv1, nv2, result, tup );
					t->doJob( j );

					if ( delSourceNViews )
					{
						sourceNViews.insert( nv1 );
						sourceNViews.insert( nv2 );
					}
				}
				//wait for one level to complete before starting the next
				pool.joinJobs();

				if ( delSourceNViews )
				{
					set<NView *>::iterator it = sourceNViews.begin();
					while ( it != sourceNViews.end() )
					{
						//save the iterator first to prevent invalidation
						set<NView *>::iterator oldIt = it; ++it;
						NView *nv = *oldIt;
						sourceNViews.erase( oldIt );
						assert_msg( nv->getViewResetMode() != DELETE_VIEW, "Dangerous VIEW-RESET-Mode in merger. The merging-result will point to nonexisting views." );
						assert_msg( nv->getCorrResetMode() == DELETE_CORR, "Leaky CORR-RESET-Mode. Old corrs wont get deleted." );
						map.setNView( nv->getViewsBegin(), nv->getViewsEnd(), NULL );
						delete nv;
					}
				}

				knit.next();
			}
			return true;
		}

	}

}
