

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_NVIEWPOINT3DCALCULATOR_H
#define ADORE_NVIEWPOINT3DCALCULATOR_H

#include <adrcore/types/nview.h>
#include <adrcore/nview/nviewoperator.h>
#include <adrcore/types/aarray.h>
#include <adrcore/nview/lineartriangulator.h>
#include <adrcore/nview/fmatrixcalculator.h>
#include <adrcore/nview/ttensorcalculator.h>
#include <adrcore/nview/point3dcorrector.h>
#include <adrcore/nview/correspondencecorrector.h>

namespace adore
{
	namespace core
	{

		/**
		 * You got a NView data structure with computed Tensors and Correspondences? You want to calculate
		 * the 3D-points now? Then this is the right class for you.
		 *
		 *  
		*/
		class Point3DCalculator : public NViewOperator
		{
			public:
				/**
				 * Constructor.
				 */
				Point3DCalculator();

				Point3DCalculator( const Point3DCalculator &pc );

				/**
				 * Destructor.
				 */
				~Point3DCalculator();

				NViewOperator *clone()
				{
					return new Point3DCalculator( *this );
				}

				/**
				 * This function calculates 3D-Points via the following procedure:
				 * - Triangulate (DLT)
				 * - optinonal LM-minimization on the reprojection error.
				 *
				 * In case of the trifocal tensor the 3D-Points should are adjusted using LM-minimization on
				 * the reprojection error.
				 *
				 * In case of the fundamental matrix this is omitted. The CorrespondenceCorrector is used instead.
				 * @param nview the NView
				 */
				void modify( NView &nview );

				/**
				 * specify, if the DLT result should be optimized by LM-minimization
				 * @param mini if true, a subsequent LM-minimization is carried out.
				 */
				void setMinimizeError( bool mini )
				{
					minimize = mini;
				}

				/**
				 * Figure out, if a subsequent LM-minimization is carried out.
				 * @return true, if a subsequent LM-minimization is carried out.
				 */
				bool getMinimizeError()
				{
					return minimize;
				}

			private:
				LinearTriangulator triangulator;
				CorrespondenceCorrector correctFM;
				FMatrixCalculator fcalc;
				TTensorCalculator tcalc;
				vector< vector<FeaturePoint *> > feats;
				bool minimize;
		};

	}

}

#endif
