

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "point3dcorrjacobi.h"

namespace adore
{
	namespace core
	{

		Point3DCorrJacobi::Point3DCorrJacobi( vector<Camera *> *cams ):
				camvec( NULL )
		{
			if ( cams ) setCams( cams );
		}


		Point3DCorrJacobi::~Point3DCorrJacobi()
		{
		}

		void Point3DCorrJacobi::operator()( const AVector &pt3D, AMatrix& result ) const
		{
			int resultRow = 0;
//			AMatrix result( 2 * camvec->size(), 3 );

			for ( uint i = 0; i < camvec->size(); ++i )
			{
				const AMatrix & c = ( *camvec )[i]->getProjMat();

				double denom = c( 2, 0 ) * pt3D( 0 ) + c( 2, 1 ) * pt3D( 1 ) + c( 2, 2 ) + c( 2, 3 ) * pt3D( 2 );
				double denomSqr =  denom * denom;
				double numer1 = c( 0, 0 ) * pt3D( 0 ) + c( 0, 1 ) * pt3D( 1 ) + c( 0, 2 ) + c( 0, 3 ) * pt3D( 2 );
				double numer2 = c( 1, 0 ) * pt3D( 0 ) + c( 1, 1 ) * pt3D( 1 ) + c( 1, 2 ) + c( 1, 3 ) * pt3D( 2 );
				double factor1 = numer1 / denomSqr;
				double factor2 = numer2 / denomSqr;

				result( resultRow, 0 ) = c( 0, 0 ) / denom - c( 2, 0 ) * factor1;
				result( resultRow, 1 ) = c( 0, 1 ) / denom - c( 2, 1 ) * factor1;
				result( resultRow, 2 ) = c( 0, 3 ) / denom - c( 2, 3 ) * factor1;

				++resultRow;

				result( resultRow, 0 ) = c( 1, 0 ) / denom - c( 2, 0 ) * factor2;
				result( resultRow, 1 ) = c( 1, 1 ) / denom - c( 2, 1 ) * factor2;
				result( resultRow, 2 ) = c( 1, 3 ) / denom - c( 2, 3 ) * factor2;

				++resultRow;
			}

//			return result;
		}

	}
}
