

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_CORERDFMATRIXCALCULATOR_H
#define ADORE_CORERDFMATRIXCALCULATOR_H

#include <adrcore/types/nview.h>
#include <adrcore/util/c++0x.h>
#include "fmatrixcalculator.h"

namespace adore
{

	namespace core
	{

		/**
			Calculates Radial Distortion and Fundamental Matrix
			 
		*/
		class RDFMatrixCalculator
		{
			private:
				typedef double T_double;
				typedef AArray<T_double,1> Arr1D;
				typedef AArray<T_double,2> Arr2D;

				Arr2D D1, D2, D3;
				Arr2D A, B;
				Arr1D lambdas;
				Arr2D eigenvecs;
				int numSolutions;
				int row;//current row of D1 - D3
				FMatrixCalculator fcalc;

			public:
				RDFMatrixCalculator():
						D1( 9, 9 ), D2( 9, 9 ), D3( 9, 9 ),
						A( 18, 18 ), B( 18, 18 ),
						lambdas( 18 ), eigenvecs( 18, 18 ),
						numSolutions( 0 ), row( 0 )
				{
					;
				}

				/**
				 * returns the number of solutions
				 */
				template<class CorrIterator, class ViewIterator>
				int calculate( CorrIterator it, const int numValues, ViewIterator vit,
				               const float minThresh = -1.5F,
				               const float maxThresh = 3.0F )
				{
					init( numValues );

					ViewBase *v1 = *vit; ++vit;
					ViewBase *v2 = *vit;
					for ( int i = 0; i < numValues; ++i, ++it )
					{
						if ( ! ( *it ) ) continue;

						FeaturePoint *fp1 = ( *it )->getFeature( v1 );
						if ( ! fp1 ) continue;

						FeaturePoint *fp2 = ( *it )->getFeature( v2 );
						if ( ! fp2 ) continue;

						addCorrespondence( *fp1, *fp2 );
					}

					return solve( numValues, minThresh, maxThresh );
				}


				template<class Iterator>
				int calculate( Iterator it1, Iterator it2, const int numValues,
				               const float minThresh = -1.5F,
				               const float maxThresh = 3.0F )
				{
					init( numValues );

					for ( int i = 0; i < numValues; ++i, ++it1, ++it2 )
					{
						if ( ! *it1 ) continue;
						AVector &fp1 = **it1;

						if ( ! *it2 ) continue;
						AVector &fp2 = **it2;

						addCorrespondence( fp1, fp2 );
					}

					return solve( numValues, minThresh, maxThresh );
				}


				/**
				 * returns the fundamental matrix and lambda of solution i
				 * if there is no ith solution, the lambda will be zero
				 */
				tuple<AMatrix, float> getSolution( int i );

				/**
				 * returns the number of solutions
				 */
				int getNumSolutions()
				{
					return numSolutions;
				}

				double getError( AMatrix &fmat, float lambda );

			private:

				void init( const int numValues );

				void addCorrespondence( const AVector &fp1, const AVector &fp2 );

				int solve( const int numValues, const float minThresh, const float maxThresh );

		};

	}

}

#endif
