

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_PARAMETERIZATIONSIMPLECAMPARAM_H
#define ADORE_PARAMETERIZATIONSIMPLECAMPARAM_H

#include <adrcore/types/camera.h>

namespace adore
{

	namespace core
	{

		/**
			 
		*/
		class SimpleCamParam
		{
			public:
				int getSize()
				{
					return 12;
				}

				void fromParam( const vector<AVector> &param, vector<Camera *> &cams ) const
				{
					for ( uint i = 0; i < param.size(); ++i ) //param.size is the limiting factor, as
						//excess cameras may be trated as fixed ones
					{
						fromParam( param[i], *cams[i] );
					}
				}

				void toParam( const vector<Camera *> &cams, vector<AVector> &param ) const
				{
					for ( uint i = 0; i < param.size(); ++i ) //param.size is the limiting factor, as
						//excess cameras may be trated as fixed ones
					{
						toParam( *cams[i], param[i] );
					}
				}

				void fromParam( const AVector &param, Camera &cam ) const
				{
					fromParam( param, cam.getProjMat() );
				}

				void fromParam( const AVector &vec, AMatrix &cam ) const
				{
					cam( 0, 0 ) = vec( 0 );
					cam( 1, 0 ) = vec( 1 );
					cam( 2, 0 ) = vec( 2 );
					cam( 0, 1 ) = vec( 3 );
					cam( 1, 1 ) = vec( 4 );
					cam( 2, 1 ) = vec( 5 );
					cam( 0, 2 ) = vec( 6 );
					cam( 1, 2 ) = vec( 7 );
					cam( 2, 2 ) = vec( 8 );
					cam( 0, 3 ) = vec( 9 );
					cam( 1, 3 ) = vec( 10 );
					cam( 2, 3 ) = vec( 11 );
				}

				void toParam( const Camera &cam, AVector &vec ) const
				{
					toParam( cam.getProjMat(), vec );
				}

				void toParam( const AMatrix &cam, AVector &vec ) const
				{
					vec( 0 ) = cam( 0, 0 );
					vec( 1 ) = cam( 1, 0 );
					vec( 2 ) = cam( 2, 0 );
					vec( 3 ) = cam( 0, 1 );
					vec( 4 ) = cam( 1, 1 );
					vec( 5 ) = cam( 2, 1 );
					vec( 6 ) = cam( 0, 2 );
					vec( 7 ) = cam( 1, 2 );
					vec( 8 ) = cam( 2, 2 );
					vec( 9 ) = cam( 0, 3 );
					vec( 10 ) = cam( 1, 3 );
					vec( 11 ) = cam( 2, 3 );
				}

		};

	}

}

#endif
