

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_PARAMETERIZATIONSOFTCONTRAINTS_H
#define ADORE_PARAMETERIZATIONSOFTCONTRAINTS_H

#include <vector>
#include <adrcore/parameterization/constants.h>
#include <adrcore/types/aarray.h>
#include <adrcore/util/debug.h>

namespace adore
{

	namespace core
	{

		using namespace std;

		/**
		 
		*/
		class SoftConstraint
		{
			public:

				SoftConstraint ( CALIBRATION_ENTITY where = FL, double v = 1.0, double r = 1.0 )
				{
					set ( where, v, r );
				}

				SoftConstraint( const SoftConstraint &sc ):row( sc.row ), col( sc.col ), val( sc.val ), range( sc.range )
				{}

				~SoftConstraint()
				{
					;
				}

				void set ( CALIBRATION_ENTITY where, double v, double r )
				{
					assert_msg ( r != 0,
					             "the range is not allowed to be zero. Use PROVIDED parameterization instead.\n" );

					switch ( where )
					{
						case FL:  row = 1; col = 1; break;
						case AR:   row = 0; col = 0; break;
						case SKEW: row = 0; col = 1; break;
						case PPX:  row = 0; col = 2; break;
						case PPY:  row = 1; col = 2; break;
					}
					val = v;
					range = fabs ( r );
				}

				void getError ( AMatrix &m, AVector &param, int paramOff )
				{
					double v = 0.0;
					if ( row == 0 && col == 0 )
					{
						v = m ( 0, 0 ) / m ( 1, 1 );
					}
					else
					{
						v = m ( row, col );
					}
					double result = ( val - v ) / range;
                                        param ( paramOff ) =  result;
				}


				void getGoal ( AVector &goal, int off )
				{
					goal ( off ) =  val;
				}


			private:
				int row;
				int col;
				double val;
				double range;
		};

		class SoftConstraintSet
		{
			public:

				SoftConstraintSet()
				{
					constraints.reserve( 10 );
				}

				SoftConstraintSet( const SoftConstraintSet &sc ):constraints( sc.constraints )
				{}

				~SoftConstraintSet()
				{
					reset();
				}

				void reset()
				{
//					for ( uint v = 0; v < constraints.size(); ++v )
//					{
//						for ( uint i = 0; i < constraints[v].size(); ++i )
//						{
//							delete constraints[v][i];
//						}
//						constraints[v].clear();
//					}
					constraints.clear();
				}

				void addConstraint ( SoftConstraint s, uint viewIdx )
				{
					if ( constraints.size() <= viewIdx ) constraints.resize ( viewIdx + 1 );
					constraints[viewIdx].push_back ( s );

#ifndef NDEBUG
					printf ( "%zd views known\n", constraints.size() );
					for ( uint i = 0; i < constraints.size(); ++i ) printf ( "%zd constraints in view %d\n", constraints[i].size(), i );
#endif
				}

				int getNumConstraints ( uint viewIdx )
				{
					assert ( viewIdx < constraints.size() );
					return constraints[viewIdx].size();
				}

				int getNumConstraints()
				{
					int sum = 0;
					for ( uint i = 0; i < constraints.size(); ++i ) sum += constraints[i].size();
					return sum;
				}

				uint getNumViews()
				{
					return constraints.size();
				}


				void getError ( AMatrix &m, AVector &err, int offset, uint viewIdx )
				{
					assert ( viewIdx < constraints.size() );

					for ( uint i = 0; i < constraints[viewIdx].size(); ++i )
					{
						constraints[viewIdx][i].getError ( m, err, offset++ );
					}
				}

				void getGoal ( AVector &goal, int offset, uint viewIdx )
				{
					assert ( viewIdx < constraints.size() );

					for ( uint i = 0; i < constraints[viewIdx].size(); ++i )
					{
						constraints[viewIdx][i].getGoal ( goal, offset++ );
					}
				}


			private:

				vector< vector<SoftConstraint> > constraints;
		};


	}

}

#endif
