

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef PERSISTENCE_BOOST_VIEW_H
#define PERSISTENCE_BOOST_VIEW_H

#include <boost/serialization/version.hpp>

#include <adrcore/types/view.h>
#include <adrcore/types/featuretypes.h>

#include <adrcore/persistence/persist_boost_common.h>
#include <adrcore/persistence/boost/imageplane.h>
#include <adrcore/persistence/boost/aarray.h>
#include <adrcore/persistence/boost/featurepoint.h>
#include <adrcore/persistence/boost/featureline.h>

#include <adrcore/util/scaleshifttrafo.h>

BOOST_CLASS_VERSION( adore::core::View<persist::IMG_TYPE>, 2 )

namespace boost
{
	namespace serialization
	{
		using namespace adore;
		using namespace adore::core;
		using namespace persist;

		template<typename Archive>
		void save( Archive & ar, const View<persist::IMG_TYPE>& v, unsigned int version )
		{
			saveImage( ar, v );
			ar << ( const uint& )v.getId();
			ar << ( const double& )v.getX();
			ar << ( const double& )v.getY();
			ar << ( const double& )v.getWidth();
			ar << ( const double& )v.getHeight();
			ar << ( const double& )v.getSameDist();
			ar << ( const bool& )v.getNormalized();
			///trafo
			ar << v.getTrafo();
			///feats
			for ( uint i = 0; i < maxFeatureTypeNo; ++i )
			{
				ar << ( const int& )v.getNumFeaturePoints( i );
				if ( v.hasFeatures( i ) )
				{
					ViewBase::PointsConstIterator fii = v.getPointsBegin( i );
					ViewBase::PointsConstIterator fend = v.getPointsEnd( i );
					for ( ; fii != fend; ++fii ) ar << ( const FeaturePoint& )**fii;
				}
				///triangles
				uint tri_vec_len = ( uint )v.getNumTriangles( i );
				ar << tri_vec_len;
				if ( 0 < tri_vec_len )
				{
					ViewBase::TrianglesConstIterator trit = v.getTrianglesBegin( i );
					ViewBase::TrianglesConstIterator trend = v.getTrianglesEnd( i );
					for ( ; trit != trend; ++trit ) ar << *trit;
				}
			}
			/// lines
			ar << ( const uint& )v.getNumSegments();
			ViewBase::LinesConstIterator lit = v.getLinesBegin();
			ViewBase::LinesConstIterator lend = v.getLinesEnd();
			while (lit != lend)
			{
				const FeatureLine* fl = (*lit).get();
				ar << *fl;
				++lit;
			}
		}

		template<typename Archive>
		void load( Archive & ar, View<persist::IMG_TYPE>& view, unsigned int /*version*/ )
		{
			string image_path; ar >> image_path;
			view.setImagePath( image_path );
			ImagePlane<IMG_TYPE>* plane = loadImage( ar, image_path );
			if ( plane )
			{
				view.setImage( plane );
				view.setDeleteImage( true );
			}

			uint view_id; ar >> view_id;
			view.setId( view_id );

			double x, y, width, height, same_dist;
			ar >> x; ar >> y;
			ar >> width; ar >> height; ar >> same_dist;
			view.setDimensions( x, y, width, height );
			view.setSameDist( same_dist );
			bool normalized; ar >> normalized;
			view.setNormalized( normalized );

			///trafo
			AMatrix & trafo = view.getTrafo();
			ar >> trafo;
			AMatrix & invTrafo = view.getInvTrafo();
			invTrafo = trafo;
			invertNormalizationMatrix ( invTrafo );

			if ( !plane )   // FAILED TO LOAD IMAGE
			{
				if ( normalized )
				{
					double scale = view.getInvTrafo()( 0, 0 );
					width = ceil( scale * view.getWidth() );
					height = ceil( scale * view.getHeight() );
				}
				plane = new ImagePlane<IMG_TYPE>( ( int )width, ( int )height, 1 );
				plane->fill( 128 );
				view.setImage( plane );
				view.setDeleteImage( true );
			}

			for ( uint u = 0; u < maxFeatureTypeNo; ++u )
			{
				int feat_cnt; ar >> feat_cnt;
				if ( feat_cnt )
				{
					for ( int j = 0; j < feat_cnt; ++j )
					{
						FeaturePoint* fp = new FeaturePoint( u );
						ar >> (FeaturePoint&)*fp;
						view.add( fp, false /*no check*/ );
					}
				}
				uint tri_vec_len; ar >> tri_vec_len;
				if ( !tri_vec_len ) continue;
				vector<unsigned> tris;
				for ( uint tt = 0; tt < tri_vec_len; ++tt )
				{
					unsigned vertex; ar >> vertex;
					tris.push_back( vertex );
				}
				view.setTriangles( tris.begin(), tris.end(), u );
			}
			/// lines
			uint line_count; ar >> line_count;
			for(uint i = 0; i < line_count; ++i)
			{
				FeatureLine* fl = new FeatureLine;
				ar >> *fl;
				view.addSegment(ViewBase::segmentT(fl));
			}
		}

		template<typename Archive>
		inline void serialize( Archive & ar, View<persist::IMG_TYPE>& v, const unsigned int file_version )
		{
			split_free( ar, v, file_version );
		}

	}
}

#endif // PERSISTENCE_BOOST_VIEW_H
