

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef ADORE_LOADER_H
#define ADORE_LOADER_H

#include <string>
#include <vector>

#include <adrcore/imgprocessing/imgarray.h>
#include <adrcore/persistence/pngpersistence.h>
#include <adrcore/persistence/jpegpersistence.h>


namespace adore
{
	namespace core
	{

		enum ImgFileType
		{
			Unknown,
			Jpeg,
			Png
		};

		class ImgLoaderBase
		{
			public:
				bool isImage ( string filename );

				void getInfo ( string filename, int &width, int &height, int &channels );
				void getInfo ( string filename, const vector<char>& data, int &width, int &height, int &channels );

			protected:
				ImgFileType extract_suffix ( string filename );

		};

		template <class T>
		class ImgLoader : public ImgLoaderBase
		{
			public:
				bool loadToBuffer ( string filename, ImgArray *buffer, int border, ImgFileType filetype );
				bool loadToBuffer ( const string& filename, const vector<char>& data,
				                    ImgArray *buffer, int border, ImgFileType filetype );
		};


		template <class T> bool ImgLoader<T>::loadToBuffer ( string filename,ImgArray *buffer,int border=0,ImgFileType filetype=Unknown )
		{
			if ( filetype==Unknown ) filetype=extract_suffix ( filename );
			switch ( filetype )
			{
				case Unknown:
					return false;
				case Jpeg:
				{
					return readJPEG(filename.c_str(), buffer->imgData, buffer->widthStep);
				}
				case Png:
				{
					return readPNG(filename.c_str(), buffer->imgData, buffer->widthStep);
				}
				default:
					cout << "ImgLoader::loadToBuffer: unexpected case" << endl;
					exit ( 1 );
			}
			return false;
		}

		template <class T>
		bool ImgLoader<T>::loadToBuffer ( const string& filename, const vector< char > & data,
		                                  ImgArray * buffer, int border = 0, ImgFileType filetype = Unknown )
		{
			if ( filetype==Unknown ) filetype=extract_suffix ( filename );
			switch ( filetype )
			{
				case Unknown:
					return false;
				case Jpeg:
				{
					JPEGPersistence loader;
					return loader.read(&data[0], data.size(), buffer->imgData, buffer->widthStep);
				}
				case Png:
				{
//					PngLoader<T> pngloader;
//					cout <<"Load png image data to buffer!"<<endl;
//					return pngloader.loadToBuffer ( filename,buffer,border );
				}
				default:
					cout << "ImgLoader::loadToBuffer: unexpected case" << endl;
					exit ( 1 );
			}
			return false;
		}

		inline bool isImage ( string filename )
		{
			ImgLoaderBase ilb;
			return ilb.isImage ( filename );
		}

	}
} //ns

#endif
