

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include "persist.h"
#include "persist_boost.h"
#include <adrcore/persistence/boost/nviewmap.h>

#include <iostream>
#include <fstream>
#include <cstring>
#include <ctime>
#include <cstdio>

#include <boost/spirit/include/classic_core.hpp>

#include <adrcore/util/filelocator.h>

namespace persist
{

	using namespace std;
	using namespace boost::spirit::classic;

	static const char * const file_header = "NView Header v. 0.1";
	static const char * const file_obj = "NView Object v. 0.1";
	static int fheader_sz = 19;
	static int fobj_sz = 19;

	void PersistBoost::open ( const string& name )
	{
		filename = name;
		lastState = checkFile();
		if ( Header == lastState ) fillObjectList();
#ifndef NDEBUG
		int cnt = objList.size();
		for ( int i = 0; i < cnt; ++i )
		{
			cout << get<0>( objList[i] ) << " " << get<1>( objList[i] ) << " " << get<2>( objList[i] ) << endl;
		}
#endif
	}

	PersistBoost::FileType PersistBoost::checkFile() const
	{
		//ObjIStream reads uncompressed data as well
		ObjIStream ifs ( filename.c_str() );
		if ( !ifs )
		{
			DBGOUT( DBGNORMAL, "PersistBoost: File does not exists, creating new\n" );
			return New;
		}
		char header[32];
		ifs.getline ( header, 32 );
		if ( 0 == strncmp ( header, file_header, fheader_sz ) )
		{
			DBGOUT( DBGNORMAL, "PersistBoost: File does exist, it's a Header file\n" );
			return Header;
		}
		else if ( 0 == strncmp ( header, file_obj, fobj_sz ) )
		{
			DBGOUT( DBGNORMAL, "PersistBoost: File does exist, it's an Object file\n" );
			return Object;
		}
		return Invalid;
	}

	void PersistBoost::close()
	{
		objList.clear();
		lastState = Invalid;
	}

	vector< string > persist::PersistBoost::objectNames() const
	{
		vector< string > list;
		int cnt = objList.size();
		for ( int i = 0; i < cnt; ++i )
			list.push_back ( get<OBJNAME> ( objList[i] ) );
		return list;
	}

	void PersistBoost::saveNViewMap ( const NViewMap & nvm, const string& objname,
	                                  bool simple, const Options& opts, const string srcFile ) const
	{
		if ( PersistBoost::Invalid == lastState ) return;

		string obj_filename ( objname );

		if ( obj_filename.empty() ) obj_filename = default_obj_name;
		obj_filename = PersistBoost::generateObjFileName ( filename, obj_filename );

		ObjDirOStream odfs;
		if ( PersistBoost::New == lastState && !simple )
		{
			DBGOUT( DBGNORMAL, "PersistBoost: laststate is PersistBoost::New\n" );
			odfs.open ( filename.c_str() );
			if ( !odfs ) return;
			odfs << file_header << "\n";
			odfs << objname << " \"" << obj_filename << "\"";
			if ( srcFile.empty() ) odfs << "\n"; else odfs << " \"" << srcFile<< "\"\n";
			odfs.close();
		}
		else if ( PersistBoost::Header == lastState )
		{
			DBGOUT( DBGNORMAL, "PersistBoost: laststate is PersistBoost::Header\n" );
			odfs.open ( filename.c_str() );
			if ( !odfs ) return;
			odfs << file_header << "\n";
			int cnt = objList.size();
			bool del_found = false;
			for ( int i = 0; i < cnt; ++i )
			{
				odfs << get<OBJNAME> ( objList[i] ) << " \"" << get<PATH> ( objList[i] );
				if ( get<SRCFILE> ( objList[i] ).empty() ) //test, if we have a source file
				{
					DBGOUT( DBGNORMAL, "PersistBoost: SRCFILE is empty\n" );
					odfs << "\"\n";
				}
				else
				{
					DBGOUT( DBGNORMAL, "PersistBoost: SRCFILE is NOT empty\n" );
					odfs << "\" \"" << get<SRCFILE> ( objList[i] ) << "\"\n";
				}
				if ( !del_found && get<OBJNAME> ( objList[i] ) == objname )
				{
					obj_filename = get<PATH> ( objList[i] );
					del_found = true;
				}
			}
			if ( !del_found )
			{
				DBGOUT( DBGNORMAL, "PersistBoost: del_found is false (%s, %s)\n", obj_filename.c_str(), srcFile.c_str() );
				odfs << objname << " \"" << obj_filename << "\"";
				if ( srcFile.empty() ) odfs << "\n"; else odfs << " \"" << srcFile << "\"\n";
			}

			odfs.close();
		}
		else obj_filename = filename;

		ObjOStream ofs;
		ofs.open ( obj_filename.c_str() );
		if ( !ofs ) return;
		ofs << file_obj << "\n";
		if ( opts.embedImages() )
		{
			NView::ViewIterator ii = nvm.getAllViewsBegin();
			NView::ViewIterator end = nvm.getAllViewsEnd();
			for ( ; ii != end; ++ii )
			{
				ViewBase* v = *ii;
				string path ( v->getImagePath() );
				if ( embed_mark != path.substr ( 0, embed_mark_l ) )
					v->setImagePath ( embed_mark + path );
			}
		}
		BoostOutput oa ( ofs );
		BoostData data;
		data.options = opts;
		OutputHelper oh(&oa, &data);
		oa << nvm;
	}

	string PersistBoost::generateObjFileName ( const string& filename, const string& objname )
	{
		size_t pos = filename.find_last_of ( "." );
		if ( string::npos == pos )
			pos = filename.length();

		string path ( filename.substr ( 0, pos ) );

		time_t rawtime; time ( &rawtime );
		string ct ( ctime ( &rawtime ) );
		int cnt = ct.size() - 1; ct.resize ( cnt );
		for ( int i = 0; i < cnt; ++i )
		{
			if ( ct[i] == string::const_reference ( " " ) || ct[i] == string::const_reference ( ":" ) )
				ct[i] = string::const_reference ( "_" );
		}
		path += "_" + objname + "_" + ct + ".che";
		return path;
	}

	bool PersistBoost::loadNViewMap ( NViewMap & nvm, const string& objname ) const
	{
		if ( New == lastState || Invalid == lastState )
			return false;

		BoostData data;
		ObjIStream ifs;
		if ( Header == lastState && !objList.empty() )
		{
			int cnt = objList.size();
			for ( int i = 0; i < cnt; ++i )
			{
				if ( objname == get<OBJNAME> ( objList[i] ) )
				{
					adore::core::FileLocator locator;
					locator.addPath ( filename.c_str() );
					string obj_file = locator.getLocation ( get<PATH> ( objList[i] ).c_str() );
					ifs.open ( obj_file.c_str() );
					data.absolute_archive_path = obj_file;
					break;
				}
			}
			if ( !ifs ) return false;
		}
		else
		{
			ifs.open ( filename.c_str() );
			if ( !ifs ) return false;
			data.absolute_archive_path = filename;
		}

		char line[fobj_sz + 10];
		ifs.getline ( line, fobj_sz + 10 );
		if ( 0 != strncmp ( line, file_obj, fobj_sz ) )
			return false;

		BoostInput ia ( ifs );
		InputHelper ih(&ia, &data);
		try
		{
			ia >> nvm;
		}
		catch ( ... )
		{
			cout << "(PersistBoost::loadNViewMap): archive corrupted?!\n";
			return false;
		} /// TODO HANDLE THIS RETURN VALUE! NViewmap state is UNDEFINED!
		return true;
	}

	string PersistBoost::getSrcFile ( string objname ) const
	{
		int cnt = objList.size();
		for ( int i = 0; i < cnt; ++i )
		{
			if ( objname == get<OBJNAME> ( objList[i] ) )
			{
				adore::core::FileLocator locator;
				locator.addPath ( filename.c_str() );
				string src_file = locator.getLocation ( get<SRCFILE> ( objList[i] ).c_str() );
				return src_file;
			}
		}
		return "";
	}

	void PersistBoost::deleteObject ( string objname )
	{
		int cnt = objList.size();
		for ( int i = 0; i < cnt; ++i )
		{
			if ( objname == get<OBJNAME> ( objList[i] ) )
			{
				adore::core::FileLocator locator;
				locator.addPath ( filename.c_str() );
				string src_file = locator.getLocation ( get<PATH> ( objList[i] ).c_str() );
				if(!locator.exists(src_file.c_str())) {
					cout << "(PersistBoost::deleteObject): no file to delete (header corrupted?)" << endl;
					return;
				}
				if( remove( src_file.c_str() ) != 0 ) {
					perror( "Error deleting file" );
					return;
				}
				cout << src_file << " is deleted." << endl;

				objList.erase(objList.begin() + i);
				ObjDirOStream odfs;
				odfs.open ( filename.c_str() );
				if ( !odfs ) {
					cout << "(PersistBoost::deleteObject): could not write header." << endl;
					return;
				}
				odfs << file_header << "\n";
				int cnt2 = objList.size();
				for ( int j = 0; j < cnt2; ++j )
				{
					odfs << get<OBJNAME> ( objList[j] ) << " \"" << get<PATH> ( objList[j] );
					if ( get<SRCFILE> ( objList[j] ).empty() ) //test, if we have a source file
					{
						odfs << "\"\n";
					}
					else
					{
						odfs << "\" \"" << get<SRCFILE> ( objList[j] ) << "\"\n";
					}
				}
				odfs.close();
				return;
			}
		}
		cout << "(PersistBoost::deleteObject): no file matching to object name found" << endl;
	}

	////////////////////////////////////////////////////////////
	////	parsing
	////////////////////////////////////////////////////////////


	struct HeaderParser : public grammar< HeaderParser >
	{
//		mutable PersistBoost::ObjectList objList;
		mutable PersistBoost::ParseTuple *curobj;

		struct add_name
		{
			const HeaderParser& hp;
			add_name ( const HeaderParser& h ) : hp ( h ) {}
			void operator() ( char const* str, char const* end ) const
			{
				get<PersistBoost::OBJNAME> ( *hp.curobj ) = string ( str, end );
			}
		};
		struct add_path
		{
			const HeaderParser& hp;
			add_path ( const HeaderParser& h ) : hp ( h ) {}
			void operator() ( char const* str, char const* end ) const
			{
				str++; end--; /// removing ""
				get<PersistBoost::PATH> ( *hp.curobj ) = string ( str, end );
			}
		};
		struct add_srcObj
		{
			const HeaderParser& hp;
			add_srcObj ( const HeaderParser& h ) : hp ( h ) {}
			void operator() ( char const* str, char const* end ) const
			{
				str++; end--; /// removing ""
				get<PersistBoost::SRCFILE> ( *hp.curobj ) = string ( str, end );
			}
		};

		void setCurObj( PersistBoost::ParseTuple *obj )
		{
			curobj = obj;
		}

		template <typename ScannerT> struct definition
		{
			definition ( HeaderParser const & self )
			{

				object = name [ add_name ( self ) ] >> path [ add_path ( self ) ] >> !path [ add_srcObj ( self ) ];

				name = +var_ch >> *var_ch;

				var_ch = alnum_p | ch_p ( "_" );

				path = ch_p ( "\"" ) >> txt >> ch_p ( "\"" );

				txt = + ( print_p - ch_p ( "\"" ) ) >> * ( print_p - ch_p ( "\"" ) );

			}
			rule<ScannerT> object, name, var_ch, path, txt;
			rule<ScannerT> const& start() const
			{
				return object;
			}
		};
	};

	bool PersistBoost::fillObjectList()
	{
		static const int line_size = 4096;
		ObjDirIStream ifs ( filename.c_str() );
		if ( !ifs ) return false;
		char line[line_size];
		ifs.getline ( line, line_size );
		if ( 0 != strncmp ( line, file_header, fheader_sz ) )
			return false;

		bool done = false;
		HeaderParser parser;
		parse_info<> info;
		while ( !done )
		{
			ifs.getline ( line, line_size );
			objList.push_back( ParseTuple() );
			parser.setCurObj( &objList.back() );
			info = parse ( line, parser, space_p/*a skip parser from boost::spirit*/ );
			if ( !info.full )
			{
				objList.pop_back();//remove empty tail
				done = true;
			}
		}
		return true;
	}
}
