

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef PERSIST_BOOST_COMMON_H
#define PERSIST_BOOST_COMMON_H

#include <string>
#include <map>

#include <boost/archive/text_iarchive.hpp>
#include <boost/archive/text_oarchive.hpp>

#include <adrcore/persistence/persist.h>

namespace persist
{
	typedef uint8_t IMG_TYPE;
	typedef boost::archive::text_oarchive BoostOutput;
	typedef boost::archive::text_iarchive BoostInput;

	static const std::string embed_mark( "embedded:" );
	static const int embed_mark_l = embed_mark.length();

	struct BoostData {
		std::string absolute_archive_path;
		Options options;
	};

	template <typename Archive>
	class OptionsHelper
	{
		typedef typename std::map<Archive*, BoostData*> OptionsMap;
		static OptionsMap options;

		static void set(Archive* ar, BoostData* data)
		{
			options[ar] = data;
		}

		static void erase(Archive* ar)
		{
			typename OptionsMap::iterator it = options.find(ar);
			if(it != options.end())
				options.erase(it);
		}

		Archive* archive;
		public:
		OptionsHelper(Archive* ar, BoostData* data)
				: archive(ar)
		{
			set(archive, data);
		}

		~OptionsHelper()
		{
			erase(archive);
		}

		static BoostData* get(Archive* ar)
		{
			typename OptionsMap::const_iterator it = options.find(ar);
			if(it == options.end())
				throw("Options not found");
			return it->second;
		}

		static const std::string& getAbsPath(Archive* ar)
		{
			BoostData* data = get(ar);
			return data->absolute_archive_path;
		}

		static Options& getOptions(Archive* ar)
		{
			BoostData* data = get(ar);
			return data->options;
		}

	};

	typedef OptionsHelper<BoostOutput> OutputHelper;
	typedef OptionsHelper<BoostInput> InputHelper;

	template <class Archive> typename OptionsHelper<Archive>::OptionsMap OptionsHelper<Archive>::options;

}// namespace persist

#endif // PERSIST_BOOST_COMMON_H
