

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef PNGPERSISTENCE_H
#define PNGPERSISTENCE_H

#include <stdint.h>
#include <string>
#include <vector>
#include <fstream>
#include <iostream>
#include <cstdlib>
#include <png.h>

using namespace std;

class PNGPersistence
{
public:
	typedef uint8_t pngT;

private:
	png_infop end_info;
	png_infop info_ptr;
	png_structp png_ptr;

public:
	bool getInfo ( const char* filename, int &width, int &height, int &channels );
	bool getInfo( const uint8_t *data, size_t size, int& width, int& height, int& channels );
	bool read( const char *filename, pngT *buffer, int wstep );
	bool read( const uint8_t *data, size_t size, pngT* buffer, int wstep );
	bool write( const char* filename, pngT * buffer, int nChannels, int width, int height, int wstep);
	bool write( vector<uint8_t>& output, pngT * buffer, int nChannels, int width, int height, int wstep);

private:
	bool checkfile ( FILE* fp );
	bool initInfoStructs();
};

template<typename T> inline bool readPNG( const char *, T*, int ) { return false; }

template<> inline bool readPNG( const char *filename, char* buffer, int wstep )
{
	PNGPersistence reader;
	return reader.read(filename, reinterpret_cast<uint8_t*>(buffer), wstep);
}

template<typename T> inline bool readPNG( const uint8_t *, size_t, T*, int ) { return false; }

template<> inline bool readPNG( const uint8_t *data, size_t size, char* buffer, int wstep )
{
	PNGPersistence reader;
	return reader.read(data, size, reinterpret_cast<uint8_t*>(buffer), wstep);
}

template<typename T> inline bool writePNG( const char *, T *, int, int, int, int ) { return false; }

template<> inline bool writePNG( const char * filename, char * buffer, int channels, int width, int height, int wstep )
{
	PNGPersistence writer;
	return writer.write(filename, reinterpret_cast<uint8_t*>(buffer), channels, width, height, wstep);
}

template<typename T> inline bool writePNG( vector<uint8_t>&, T *, int, int, int, int ) { return false; }

template<> inline bool writePNG( vector<uint8_t>& output, char * buffer, int channels, int width, int height, int wstep )
{
	PNGPersistence writer;
	return writer.write(output, reinterpret_cast<uint8_t*>(buffer), channels, width, height, wstep);
}

#endif
