

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include "zoutbuf.h"

#include <iostream>
#include <cstring>
#include <locale>

#include <zlib.h>

namespace zstream
{

	using namespace std;

	zoutbuf::zoutbuf()
			: lastError( 0 ), file( 0 )/*, flushCounter(0)*/
	{
		setp( buffer, buffer + buf_size );
	}

	zoutbuf::~ zoutbuf()
	{
		sync();
		close();
	}

	void zoutbuf::open( const char * name )
	{
		if ( file ) close();
		file = gzopen( name, "wb1f" );
	}

	bool zoutbuf::is_open() const
	{
		return 0 != file;
	}

	zoutbuf * zoutbuf::close()
	{
		flushBuffer();
		lastError = gzclose( file );
		file = 0;
		if ( Z_OK != lastError )
		{
			if ( lastError == Z_ERRNO )
			{
				perror( "zoutbuf::close(): error in underlying lib" );
			}
			else
			{
				cout << "zoutbuf::close(): error - " << lastError << endl;
				if ( lastError == Z_STREAM_ERROR )
				{
					cout << "this is a Z_STREAM_ERROR: file was NULL, or did not refer to an open compressed file stream." << endl;
				}
				else if ( lastError == Z_BUF_ERROR )
				{
					cout << "this is a Z_BUF_ERROR: no compression progress is possible during buffer flush (see deflate())." << endl;
				}
				else
				{
					cout << "unknown error" << endl;
				}
			}

			return 0;
		}
		return this;
	}

	int zoutbuf::overflow( int c )
	{
		if ( EOF != c )
		{
			flushBuffer();
			*pptr() = c;
			pbump( 1 );
		}
		return c;
	}

	int zoutbuf::sync()
	{
		flushBuffer();
		return 0;
	}


	streamsize zoutbuf::xsputn( const char * s, streamsize n )
	{
		int cnt = n;
		while ( cnt )
		{
			int pos = pptr() - pbase();
			int next_pos = pos + cnt;
			if ( next_pos >= buf_size )
			{
				size_t num = buf_size - pos;
				memcpy( pptr(), s + ( n - cnt ), num );
				pbump( num );
				flushBuffer();
				cnt -= num;
			}
			else
			{
				memcpy( pptr(), s + ( n - cnt ), cnt );
				pbump( cnt );
				cnt -= cnt;
			}
		}

		return n;
	}

	void zoutbuf::flushBuffer()
	{
		int num = pptr() - pbase();
		if ( 0 >= num ) return;
		if ( file ) gzwrite( file, pbase(), num );
		pbump( -num );
		/*	flushCounter += num;
			if(data_window <= flushCounter) {
				lastError = gzflush(file, Z_FULL_FLUSH);
				flushCounter = 0;
			}*/
	}

} //zstream

