

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADOREINTEGERRANDOMSET_H
#define ADOREINTEGERRANDOMSET_H

#include <stdlib.h>
#include <list>
#include <vector>
#include <cassert>
#include <adrcore/random/random.h>
namespace adore
{
	namespace core
	{

		using namespace std;

		class NumberStore
		{
			private:
				vector<int> numbers;
				vector<int> swaps;
			public:
				NumberStore( int size = 100, int numSwaps = 10 )
				{
					assert( size >= numSwaps );
					numbers.reserve( size );
					swaps.reserve( numSwaps );
				}

				void init( int lower, int upper )
				{
					for ( ;lower <= upper; ++lower )
					{
						numbers.push_back( lower );
					}
				}

				void swap( int idx )
				{
					int temp = numbers[swaps.size()];
					numbers[swaps.size()] = numbers[idx];
					numbers[idx] = temp;
					swaps.push_back( idx );
				}

				int getRandomIndex( uint max )
				{
					assert( max >= swaps.size() );
					assert( max < numbers.size() );
					return swaps.size() + ( int ) ( float( max - swaps.size() ) * intRand() / ( RAND_MAX + 1.0 ) );
				}

				int getNumberAt( int idx )
				{
					return numbers[idx];
				}

				void rewind()
				{
					for ( int i = swaps.size() - 1; i >= 0; --i )
					{
						int temp = numbers[i];
						numbers[i] = numbers[swaps[i]];
						numbers[swaps[i]] = temp;
					}
					swaps.clear();
				}
		};

		template<class Iterator>
		void createChoice( Iterator it, int size, int max, NumberStore &store )
		{
			for ( int i = 0; i < size ; ++i, ++it )
			{
				int idx = store.getRandomIndex( max );
				*it = store.getNumberAt( idx );
				store.swap( idx );
			}
			store.rewind();
		}



		/**
		This class generates a random subset of distinc integers inside a given interval.
		There are two methods availiable:

		n Numbers in [m, m+k], n &lt; k

		METHOD1 (for small intervals):
		Space: O(k)
		Time: O(n)

		METHOD2 (for large intervals):
		Space: O(n)
		Time: O(n*n)

		 
		*/
		class IntegerRandomSet
		{
			public:
				enum METHOD_TYPE
				{
					DEFAULT,    //uses default method
					METHOD1,    //uses an array of all integers in the given intervall
					METHOD2     //maintains a set of intervals
				};

				/**
				    Instantiates a new IntegerRandomSet.
				    Default method is METHOD1.
				*/
				IntegerRandomSet();

				/**
				    Destruktor
				*/
				~IntegerRandomSet();

				/**
				    Computes the random subset an stores it in "array".
				*/
				void createChoice( vector<int> &array, int lower, int upper, vector<int>* mask = NULL, const METHOD_TYPE m = DEFAULT );

				/**
				    Sets the method.
				*/
				void setMethod( const METHOD_TYPE& theValue )
				{
					if ( theValue == DEFAULT ) method = METHOD1;
					method = theValue;
				}

				/**
				    Gets the method.
				*/
				int getMethod() const
				{
					return method;
				}


			private:
				/**
				    Stores the type of method.
				*/
				METHOD_TYPE method;

				//Variables for method one
				list<int> q;
				int remaining;
				int idx;
				int numNumbers;
				int *array;

				/**
				    DBGOUT: Prints the values of private members
				*/
				void print();

				/**
				    DBGOUT: Prints the intervals in q
				*/
				void printList();

		};

	}
}

#endif
