

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "random.h"
#include <boost/random.hpp>
#include <ctime>
#include <vector>
#include <sys/time.h>
namespace adore
{
	namespace core
	{

//		--------------------------------RandomSafe------------------------------------
		RandomSafe::~RandomSafe()
		{
			vector<elem*>::iterator p=threads.begin();
			while ( p!=threads.end() )
			{
				delete ( *p )->rP;
				delete ( *p );
				p++;
			}
        }

		RandomProvider* RandomSafe::provider()
		{
			l.lock();
			pthread_t user=pthread_self();
			vector<elem*>::iterator p=threads.begin();
			while ( p!=threads.end() )
			{
				if ( user==( *p )->id )break;
				p++;
			}
			if ( p!=threads.end() )
			{
				l.unlock();
				return ( *p )->rP;
			}
			else
			{
				elem *tmp=new elem();
				tmp->id=user;
				tmp->rP=new RandomProvider();
				tmp->rP->generator.seed( dRP.genUniInt() );
				threads.push_back( tmp );
				l.unlock();
				return tmp->rP;
			}
		}
		void RandomSafe::delThread()
		{
			l.lock();
			pthread_t user=pthread_self();
			vector<elem*>::iterator p=threads.begin();
			while ( p!=threads.end() )
			{
				if ( user==( *p )->id )break;
				p++;
			}
			if ( p!=threads.end() )
			{
				printf( "gelöscht" );
				threads.erase( p );
			}
			l.unlock();
		};
//		------------------------------------------------------------------------------

		RandomSafe sR;
		RandomProvider dRP; //defaultRandomProvider
		/**
		 *	a random number between 0 and 1 (inclusive)
		 */
		double dblRand( RandomProvider &rp )
		{
			if ( &rp==&dRP )return sR.provider()->genUniReal();
			return rp.genUniReal();
		}

		/**
		 *	a random number between min and max
		 */
		double dblRand( double min, double max, RandomProvider &rp )
		{
			return min + ( max - min ) * dblRand( rp );
		}

		int intRand( RandomProvider &rp )
		{
			if ( &rp==&dRP )
				return sR.provider()->genUniInt();
			return rp.genUniInt();
		}
		/**
		 *	a random number between min and max
		 */
		int intRand( int min, int max, RandomProvider &rp )
		{
			return min + int ( ( max - min + 1 ) * ( intRand( rp ) / ( ADR_RAND_MAX + 1.0 ) ) );
		}

		void setSeed( unsigned int s, RandomProvider &rp )
		{
			if ( &rp==&dRP )sR.provider()->generator.seed( s );
			else rp.generator.seed( s );
		}

		void setSeed( RandomProvider &rp )
		{
			setSeed( static_cast<unsigned int>( getTimeSeed() ), rp );
		}
		void setGlobalSeed( unsigned int s )
		{
			sR.setGlobalSeed( s );
		}
		void cleaner( void* c )
		{
			sR.delThread();
		}

		unsigned int randomSeedCallCount(0);
		
		void setRandomSeedCallCount(const int c)
		{
			randomSeedCallCount = c;
		}
		
		unsigned int getTimeSeed()
		{
			timespec t1;
			clock_gettime(CLOCK_REALTIME, &t1);
			unsigned int clockval = t1.tv_nsec * t1.tv_sec;
			return (clockval << 16) | (randomSeedCallCount++);
		}
	}
}
