

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_RANDOMUTILSRANSACCLIENT_H
#define ADORE_RANDOMUTILSRANSACCLIENT_H

#include<vector>
#include<adrcore/util/debug.h>

namespace adore
{
	namespace core
	{

		using namespace std;

		/**
		 
		*/
		class RANSACClient
		{
			private:

				double err;
				int numIn;
				bool okay;
				int numCandidates;

			public:
				RANSACClient():err( -1 ), numIn( -1 ), okay( false ), numCandidates( 0 )
				{
					;
				}

				RANSACClient( const RANSACClient& rc ):err( rc.err ), numIn( rc.numIn ), okay( rc.okay ), numCandidates( rc.numCandidates ) {}

				virtual ~RANSACClient()
				{
					;
				}

				virtual RANSACClient *clone()
				{
					error( "missing implementation\n" );
				}

				virtual int getSampleSize() = 0;

				/**
				* a new sample is ready. This gives you the opportunity to prepare for the invokation of getNumberOfInliers(...)
				* As the sample is a protected class member, no argument is necessary.
				* @return please deliver the number of results. &lt; 0 should mark an error.
				*/
				virtual bool applySample( const vector<int> &sample ) = 0;

				/**
				* This method is called after ransac has finished.
				* Subclasses may use this to tidy up.
				*/
				virtual void ransacDone() = 0;

				/**
				* the lo-ransac step. optimize the result by applying a local search or something
				* shall return false if no optimization took place
				*/
				virtual bool optimize() = 0;

				double getError() const
				{
					return err;
				}

				int getNumIn() const
				{
					return numIn;
				}

				bool isOkay() const
				{
					return okay;
				}

				int getNumCandidates() const
				{
					return numCandidates;
				}

				void setNumIn( const int theValue )
				{
					numIn = theValue;
				}

				void setError( const double theValue )
				{
					err = theValue;
				}

				void setOkay( const bool theValue )
				{
					okay = theValue;
				}

				void setNumCandidates( const int theValue )
				{
					numCandidates = theValue;
				}

		};

	}

}

#endif
