

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_RANDOMUTILSRANSACINTERFACE_H
#define ADORE_RANDOMUTILSRANSACINTERFACE_H

#include<vector>
#include<adrcore/util/debug.h>

namespace adore
{
	namespace core
	{

		using namespace std;

		/**
		 
		*/
		class RANSACInterface
		{
			public:
				RANSACInterface() {}

				virtual ~RANSACInterface() {}

				/**
				 * Implement this to present an algorithm that tells us the Error. A first choice may br the number of
				 * outliers, a better one would be
				 *
				 * result = sum(p(e²)) with
				 * p(e²) = e² &lt; T² ? e² : T²
				 *
				 * The protected member "inMask", which is an array of all indices not mark as outliers by previous filters, may be used.
				     *
				     * If your ransac supports an optimization step, this function will be called again afterwards. Then, the resultIdx will be 0
				     *
				 * @param resultIdx the inliers of what result?
				 * @param inlierFlags when a correspondence is an inlier, the corresponding flag should be "true" and "false" otherwise.
				 * @param distVals fill in the distance of every sample to the computed model, if you want ties to be resolved.
				 */
				virtual void computeError( int resultIdx, vector<int> &inlierFlags, vector<double> &distVals ) = 0;

				/**
				 * This method is called right before the RANSAC iteration starts and gives you the opportunity the pull the emergency break.
				 * @param numIndices the number of points that will be taken into account in the following RANSAC procedure.
				 */
				virtual bool checkPrerequisitsAndInit( int numInliers, const vector<int> &indices ) = 0;

				/**
				 * a new sample is ready. This gives you the opportunity to prepare for the invokation of getNumberOfInliers(...)
				 * As the sample is a protected class member, no argument is necessary.
				 * @return please deliver the number of results. &lt; 0 should mark an error.
				 */
				virtual int applySample( const vector<int> &sample ) = 0;

				/**
				 * This method is called after ransac has finished.
				 * Subclasses may use this to tidy up.
				 */
				virtual void ransacDone()
				{
					;
				}

				/**
				 * the lo-ransac step. optimize the result by applying a local search or something
				 * shall return false if no optimization took place
				 */
				virtual bool optimize( int bestIdx ) = 0;

				double getError() const
				{
					return err;
				}

				int getNumIn() const
				{
					return numIn;
				}

			protected:

				void setNumIn( const int& theValue )
				{
					numIn = theValue;
				}

				void setError( double theValue )
				{
					err = theValue;
				}

			private:

				double err;
				int numIn;
		};

	}

}

#endif
