

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "assessor.h"
#include <adrcore/util/printmat.h>

namespace adore
{

	namespace core
	{


		ViewErrors::ViewErrors():nvToDiff( NULL, NULL ) {;}

		void ViewErrors::computeDiffs( ViewBase *v, NView &nview )
		{
			assert( v );

			Camera *c = nview.getCamera( v );
			if ( !c )
			{
				DBGOUT( DBGNORMAL, "Assessor: No Camera for view %p\n", v );
				return;
			}
			if ( nview.getNumPoints3D() < 1 )
			{
				DBGOUT( DBGNORMAL, "Assessor: No 3D points in NView\n" );
				return;
			}

			for ( NView::Point3DIterator it = nview.getPoints3DBegin(); it != nview.getPoints3DEnd(); ++it )
			{
				AVector3D *p = *it;
				assert( p );

				PointCorr *pc = nview.getPointCorr( p );
//				cout << pc->toDbgString() << endl;
				if ( pc )
				{
					FeaturePoint *nvp = pc->getFeature( v );
					if ( nvp )
					{
						AVector2D p2D = c->project( *p );
						assert_msg( fabs( p2D( 2 ) ) > 0, "Assessor: 3rd repro coor has been %f\n", p2D( 2 ) );
						p2D( 0 ) = p2D( 0 ) / p2D( 2 );
						p2D( 1 ) = p2D( 1 ) / p2D( 2 );
						p2D( 2 ) = 1.0;

						assert_msg( fabs( ( *nvp )( 2 ) ) > 0, "Assessor: 3rd feat coor has been %f\n", p2D( 2 ) );
						( *nvp )( 0 ) /= ( *nvp )( 2 );
						( *nvp )( 1 ) /= ( *nvp )( 2 );
						( *nvp )( 2 ) = 1.0;

//						if ( count < 3 )
//						{
//							printf( "3D-point in assessor:\n" );
//							core::printMat( *p );
//							printf( "camera in assessor:\n" );
//							core::printMat( c->getProjMat() );
//							printf( "norm: pt: recon | orig\n" );
//							core::printMats( p2D, *nvp );
//							count++;
//						}

						AVector2D *diff = new AVector2D( 2 );
						*diff = fabs( ( *nvp )( 0 ) - p2D( 0 ) ), fabs( ( *nvp )( 1 ) - p2D( 1 ) );
						nvToDiff.add( nvp, diff );

						if ( ( *diff )( 0 ) > 5 || ( *diff )( 1 ) > 5 )
						{
							DBGOUT( DBGCRITICAL, "Assessor: large Diff detected:\n" );
							DBGOUT( DBGCRITICAL, "Assessor: Camera:\n" );
							printMat( c->getProjMat() );
							DBGOUT( DBGCRITICAL, "Assessor: Point3D:\n" );
							printMat( *p );
							DBGOUT( DBGCRITICAL, "Assessor: 2D (orig/repro):\n" );
							printMats( ( *nvp ), p2D );
						}
					}
					else
					{
//						DBGOUT( DBGNORMAL, "Assessor: No feature of view %p in correspondence %p\n", v, pc );
					}
				}
				else
				{
					DBGOUT( DBGNORMAL, "Assessor: No correspondence for 3D point %p\n", p );
				}
			}
			DBGOUT( DBGALL, "Assessor: %d diffs computed\n", nvToDiff.getSize() );
		}

		ViewErrors::~ViewErrors() {reset();}

		void ViewErrors::reset()
		{
			for ( Bijectivity<FeaturePoint *, AVector2D *>::BIterator it = nvToDiff.getBsBegin();
			                it != nvToDiff.getBsEnd(); ++it )
			{
				delete *it;
			}
			nvToDiff.clear();

			reproErrorSum = 0;
			reproErrorMin = DBL_MAX;
			reproErrorMax = 0;
			reproErrorAvg = 0;
			reproErrorVar = 0;

			xErrorSum = 0;
			xErrorMin = DBL_MAX;
			xErrorMax = 0;
			xErrorAvg = 0;
			xErrorVar = 0;

			yErrorSum = 0;
			yErrorMin = DBL_MAX;
			yErrorMax = 0;
			yErrorAvg = 0;
			yErrorVar = 0;
		}

		void ViewErrors::computeResults()
		{
//			printf( "num entries: %d\n", nvToDiff.getSize() );
			double valsR[nvToDiff.getSize()];
			double valsX[nvToDiff.getSize()];
			double valsY[nvToDiff.getSize()];

			int idx = 0;
			for ( Bijectivity<FeaturePoint *, AVector2D *>::BIterator it = nvToDiff.getBsBegin();
			                it != nvToDiff.getBsEnd(); ++it, ++idx )
			{
				AVector2D *p = *it;

				double x = ( *p )( 0 );
				double y = ( *p )( 1 );
				double rSqr = x * x + y * y;

				valsR[idx] = rSqr;
				valsX[idx] = x;
				valsY[idx] = y;

				if ( x > xErrorMax ) xErrorMax = x;
				if ( x < xErrorMin ) xErrorMin = x;
				xErrorSum += x;
				if ( y > xErrorMax ) yErrorMax = y;
				if ( y < xErrorMin ) yErrorMin = y;
				yErrorSum += y;
				if ( rSqr > reproErrorMax ) reproErrorMax = rSqr;
				if ( rSqr < reproErrorMin ) reproErrorMin = rSqr;
				reproErrorSum += rSqr;
			}
			if ( idx == 0 )
			{
				DBGOUT( DBGNORMAL, "Assessor: No Points to asses the result\n" );
			}

			getAvgVar( valsR, nvToDiff.getSize(), reproErrorAvg, reproErrorVar,0 );
			getAvgVar( valsX, nvToDiff.getSize(), xErrorAvg, xErrorVar,0 );
			getAvgVar( valsY, nvToDiff.getSize(), yErrorAvg, yErrorVar,0 );

//			printf( "reproerrSum: %.20f\n", reproErrorSum );
//			printf( "reproerrAvg: %.20f\n", reproErrorAvg );
		}

		void ViewErrors::computeErrors( ViewBase *v, NView &nview )
		{
			reset();
			computeDiffs( v, nview );
			computeResults();
		}

		void ViewErrors::getXDifferences( vector<double> &diffs )
		{
			for ( Bijectivity<FeaturePoint *, AVector2D *>::BIterator it = nvToDiff.getBsBegin();
			                it != nvToDiff.getBsEnd(); ++it )
			{
				diffs.push_back( ( **it )( 0 ) );
			}
		}

		void ViewErrors::getYDifferences( vector<double> &diffs )
		{
			for ( Bijectivity<FeaturePoint *, AVector2D *>::BIterator it = nvToDiff.getBsBegin();
			                it != nvToDiff.getBsEnd(); ++it )
			{
				diffs.push_back( ( **it )( 1 ) );
			}
		}

		void ViewErrors::getReproErrors( vector<double> &diffs )
		{
			for ( Bijectivity<FeaturePoint *, AVector2D *>::BIterator it = nvToDiff.getBsBegin();
			                it != nvToDiff.getBsEnd(); ++it )
			{
				double x = ( **it )( 0 );
				double y = ( **it )( 1 );
				double rSqr = x * x + y * y;
				diffs.push_back( rSqr );
			}
		}

		void ViewErrors::getReproErrorsAndFeatures( vector< pair<FeaturePoint*, double> > & errs )
		{
			for ( Bijectivity<FeaturePoint *, AVector2D *>::BIterator it = nvToDiff.getBsBegin(); it != nvToDiff.getBsEnd(); ++it )
			{
				FeaturePoint * fp = nvToDiff.getA(*it);

				double x = ( **it )( 0 );
				double y = ( **it )( 1 );
				double rSqr = x * x + y * y;

				errs.push_back( make_pair(fp, rSqr) );
			}
		}

//		7


		Assessor::Assessor():viewToErrors( NULL, NULL )
		{
		}


		Assessor::~Assessor()
		{
			reset();
		}

		void Assessor::reset()
		{
			ViewErrors::reset();

			for ( Bijectivity<ViewBase *, ViewErrors *>::BIterator it = viewToErrors.getBsBegin();
			                it != viewToErrors.getBsEnd(); ++it )
			{
				delete *it;
			}
			viewToErrors.clear();
		}

		void Assessor::assess( NView &nview )
		{
			reset();

			for ( NView::ViewIterator vit = nview.getViewsBegin();
			                vit != nview.getViewsEnd(); ++vit )
			{
				ViewErrors *err = new ViewErrors();
				err->computeErrors( *vit, nview );
				viewToErrors.add( *vit, err );
			}

			double val;
			for ( Bijectivity<ViewBase *, ViewErrors *>::BIterator it = viewToErrors.getBsBegin();
			                it != viewToErrors.getBsEnd(); ++it )
			{
				ViewErrors *err = *it;

				val = err->getReproErrorSum();
				reproErrorSum += val;
				val = err->getReproErrorMin();
				if ( val < reproErrorMin ) reproErrorMin = val;
				val = err->getReproErrorMax();
				if ( val > reproErrorMax ) reproErrorMax = val;

				val = err->getXErrorSum();
				xErrorSum += val;
				val = err->getXErrorMin();
				if ( val < xErrorMin ) xErrorMin = val;
				val = err->getXErrorMax();
				if ( val > xErrorMax ) xErrorMax = val;

				val = err->getYErrorSum();
				yErrorSum += val;
				val = err->getYErrorMin();
				if ( val < yErrorMin ) yErrorMin = val;
				val = err->getYErrorMax();
				if ( val > yErrorMax ) yErrorMax = val;
			}

			vector<double> vals;
			vals.reserve( getSize() );

			getXDifferences( vals );
			getAvgVar( vals, xErrorAvg, xErrorVar );

			vals.clear();
			getYDifferences( vals );
			getAvgVar( vals, yErrorAvg, yErrorVar );

			vals.clear();
			getReproErrors( vals );
			getAvgVar( vals, reproErrorAvg, reproErrorVar );

			notInFront = computeCamErrors( nview );
		}

		void Assessor::computeErrors( ViewBase *v, NView &nview )
		{
			ADORE_UNUSED( v );
			ADORE_UNUSED( nview );
			DBGOUT( DBGALL, "not supported by Assessor\n" );
		}

		int Assessor::getSize()
		{
			uint size = 0;
			for ( Bijectivity<ViewBase *, ViewErrors *>::BIterator it = viewToErrors.getBsBegin();
			                it != viewToErrors.getBsEnd(); ++it )
			{
				size += ( *it )->getSize();
			}
			return size;
		}

		void Assessor::getXDifferences( vector<double> &diffs )
		{
			for ( Bijectivity<ViewBase *, ViewErrors *>::BIterator it = viewToErrors.getBsBegin();
			                it != viewToErrors.getBsEnd(); ++it )
			{
				( *it )->getXDifferences( diffs );
			}
		}

		void Assessor::getYDifferences( vector<double> &diffs )
		{
			for ( Bijectivity<ViewBase *, ViewErrors *>::BIterator it = viewToErrors.getBsBegin();
			                it != viewToErrors.getBsEnd(); ++it )
			{
				( *it )->getYDifferences( diffs );
			}
		}

		void Assessor::getReproErrors( vector<double> &diffs )
		{
			for ( Bijectivity<ViewBase *, ViewErrors *>::BIterator it = viewToErrors.getBsBegin();
			                it != viewToErrors.getBsEnd(); ++it )
			{
				( *it )->getReproErrors( diffs );
			}
		}

		void Assessor::computeDiffs( ViewBase *v, NView &nview )
		{
			ADORE_UNUSED( v );
			ADORE_UNUSED( nview );
			DBGOUT( DBGALL, "not supported by Assessor\n" );
		}

		void Assessor::computeResults()
		{
			DBGOUT( DBGALL, "not supported by Assessor\n" );
		}

		map<Camera*, int> Assessor::computeCamErrors( NView & nview )
		{
			/** BadPointRemover3 **/
			map<Camera*, int> removed;//bad name, they are not really removed
			NView::CorrIterator cit = nview.getPointCorrsBegin();
			while ( cit != nview.getPointCorrsEnd() )
			{
				PointCorr *pc = *cit; ++cit;
				if ( pc->size() > 0 )
				{
					NView::Point3D *pt3D = nview.getPoint3D ( pc );
					if ( pt3D )
					{
						for ( PointCorr::iterator it = pc->begin(); it != pc->end(); ++it )
						{
							FeaturePoint *fp = *it;
							Camera *cam = nview.getCamera ( fp->getView() );
							if ( !cam )
							{
								DBGOUT( DBGNORMAL, "computeCamErrors: missing Camera - stopping\n" );
								return map<Camera*, int>();
							}
							if ( removed.find( cam ) == removed.end() )
								removed[cam] = 0;
							if ( cam->getSign ( *pt3D ) <= 0 )
							{
								++removed[cam];
								break;
							}
						}
					}
				}
			}
			return removed;
		}


	}
}//namespace
