

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_TESTPROGRESSOBSERVER_H
#define ADORE_TESTPROGRESSOBSERVER_H

#include <vector>
#include <adrcore/test/progresslistener.h>

namespace adore
{

	namespace core
	{

		using namespace std;



		/**
		 
		*/
		class ProgressObserver
		{
			public:
				virtual ~ProgressObserver() {;}

				virtual void operator++() {;}
		};



		class ProgressObserverSimple: public ProgressObserver
		{
			public:
				ProgressObserverSimple( int m = 0 )
				{
					max = m;
				}

				virtual ~ProgressObserverSimple() {;}

				void setCurrent( const int& theValue )
				{
					current = theValue;
					notify();
				}

				int getCurrent() const
				{
					return current;
				}

				void setMax( const int& theValue )
				{
					max = theValue;
				}

				int getMax() const
				{
					return max;
				}

				void operator++() {inc( 1 );}

				void operator+=( int i ) {inc( i );}

				void inc( int step = 1 )
				{
					if ( current + step <= max )
					{
						current += step;
					}
					else
					{
						current = max;
					}
					notify();

				}

				void complete()
				{
					current = max;
					notify();
				}

				void addListener( ProgressListener *listener )
				{
					listeners.push_back( listener );
				}

				void clearListeners()
				{
					listeners.clear();
				}


				void reset()
				{
					current = 0;
					notify();
				}

			protected:

				void notify()
				{
					for ( uint i = 0; i < listeners.size(); ++i )
					{
						listeners[i]->progressChanged();
					}
				}

				int current;
				int max;
				vector<ProgressListener *> listeners;
		};

	}

}

#endif
