

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef ADORE_AARRAYVIEW_H
#define ADORE_AARRAYVIEW_H

#include "aarrayrange.h"
#include "aarrayinitializer.h"
#include "aarrayviewiterator.h"

#include <boost/multi_array.hpp>

namespace adore {
namespace core {

    template<typename T, int rank> class AArray;


    template<typename A, typename T, int rank1, int rank2> struct view_type_getter {};

    template<typename T, int rank1, int rank2> struct view_type_getter<AArray<T, rank1>, T, rank1, rank2 >
    {
        typedef typename boost::multi_array<T, rank1>::template array_view<rank2>::type view_t;
    };

    template<typename T, int rank1, int rank2> struct view_type_getter<const AArray<T, rank1>, T, rank1, rank2 >
    {
        typedef typename boost::multi_array<T, rank1>::template const_array_view<rank2>::type view_t;
    };

    template<typename A, int rank> class AArrayView
    {
        typedef AArrayView<A, rank> this_type;
        typedef typename A::size_type size_type;
        typedef typename A::reference reference;
		typedef typename A::const_reference const_reference;

        typedef typename view_type_getter<A, typename A::scalar_type, A::get_rank, rank>::view_t view_type;
        typedef boost::multi_array_types::index_gen indices_type;

        view_type m_data;

    public:

        typedef typename A::scalar_type scalar_type;
        typedef recursive_iterator<view_type, rank> iterator;
        typedef recursive_iterator<const view_type, rank> const_iterator;

        AArrayView(A& arr, const Range& r) :
            m_data( arr.internal_array()[ indices_type()[r] ] )
        {}

        AArrayView(A& arr, const Range& r0, const Range& r1) :
            m_data( arr.internal_array()[ indices_type()[r0][r1] ] )
        {}

        AArrayView(A& arr, const Range& r0, const int i1) :
            m_data( arr.internal_array()[ indices_type()[r0][i1] ] )
        {}

        AArrayView(A& arr, const int i0, const Range& r1 ) :
            m_data( arr.internal_array()[ indices_type()[i0][r1] ] )
        {}

        AArrayView(A& arr, const int i0, const Range& r1, const Range& r2 ) :
            m_data( arr.internal_array()[ indices_type()[i0][r1][r2] ] )
        {}

		view_type & internal_array() { return m_data; }

		const view_type & internal_array() const { return m_data; }

		iterator begin() { return iterator(m_data.begin()); }
        const_iterator begin() const { return const_iterator(m_data.begin()); }
        iterator end() { return iterator(m_data.end()); }
        const_iterator end() const { return const_iterator(m_data.end()); }

        reference operator() (size_type i0, size_type i1) { BOOST_STATIC_ASSERT(rank == 2); return m_data[i0][i1]; }
		const_reference operator() (size_type i0, size_type i1) const { BOOST_STATIC_ASSERT(rank == 2); return m_data[i0][i1]; }

		this_type& operator=(const AArray<scalar_type, rank>& rhs)
		{
			internal_array() = rhs.internal_array();
			return *this;
		}

		this_type& operator=(const this_type& rhs)
		{
			internal_array() = rhs.internal_array();
			return *this;
		}

        size_type size() const { return m_data.num_elements(); }

        const size_type* shape() const { return m_data.shape(); }

        Initializer<this_type> operator=(const scalar_type val)
        {
            return Initializer<this_type>((*this), val);
        }

		size_type rows() const
		{
			BOOST_STATIC_ASSERT(rank == 1 || rank == 2);
			return m_data.shape()[0];
		}

		size_type cols() const
		{
			BOOST_STATIC_ASSERT(rank == 1 || rank == 2);
			return m_data.shape()[1];
		}

		void setIdentity()
		{
			( *this ) = 0;
			for ( unsigned d = 0; d < std::min( rows(), cols() ); ++d )
			{
				( *this )( d, d ) = 1;
			}
		}

    };

    template<typename A, int rank>
    AArray<typename A::scalar_type, rank> operator*(const AArrayView<A, rank>& a, const typename A::scalar_type factor)
    {
        typedef typename A::scalar_type scalar_type;
        std::vector<size_t> ex; ex.assign( a.shape(), a.shape() + rank );
        AArray<scalar_type, rank> result(ex);
        typename AArrayView<A, rank>::const_iterator ita = a.begin(), enda = a.end();
        typename AArray<scalar_type, rank>::iterator itr = result.begin();
        for( ; ita != enda; ++ita, ++itr)
            *itr = *ita * factor;
        return result;
    }

    template<typename A, int rank>
    AArray<typename A::scalar_type, rank> operator*(const typename A::scalar_type factor, const AArrayView<A, rank>& a)
    {
        return a * factor;
    }

    template<typename A, int rank>
    AArray<typename A::scalar_type, rank> operator+(const AArrayView<A, rank>& a, const AArray<typename A::scalar_type, rank>& b)
    {
        typedef typename A::scalar_type scalar_type;
        std::vector<size_t> ex; ex.assign( a.shape(), a.shape() + rank );
        AArray<scalar_type, rank> result(ex);
        typename AArrayView<A, rank>::const_iterator ita = a.begin(), enda = a.end();
        typename AArray<scalar_type, rank>::const_iterator itb = b.begin();
        typename AArray<scalar_type, rank>::iterator itr = result.begin();
        for( ; ita != enda; ++ita, ++itb, ++itr)
            *itr = *ita + *itb;
        return result;
    }

} // core
} // adore

#endif // ADORE_AARRAYVIEW_H
