

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef AARRAY_RECURSIVE_ITERATOR_H
#define AARRAY_RECURSIVE_ITERATOR_H

namespace adore {
namespace core {

/**
  
 */

template <typename ArrayView, int rank> struct sub_iterator_type_gen
{
	typedef typename ArrayView::iterator iterator;
	typedef typename ArrayView::const_iterator const_iterator;
};

template <typename ArrayView> struct sub_iterator_type_gen<ArrayView, 1>
{
	typedef typename ArrayView::template subarray<1>::type::iterator iterator;
	typedef typename ArrayView::template subarray<1>::type::const_iterator const_iterator;
	typedef typename iterator::value_type scalar_type;
};

template <typename ArrayView, int rank> class recursive_iterator
{
	typedef typename sub_iterator_type_gen<ArrayView, rank>::iterator iterator;
	typedef typename sub_iterator_type_gen<ArrayView, rank>::const_iterator const_iterator;
	typedef recursive_iterator<ArrayView, rank-1> sub_iterator;

	typedef typename ArrayView::element scalar_type;
	typedef scalar_type& reference;
	typedef const scalar_type& const_reference;

	friend class recursive_iterator<ArrayView, rank+1>;

	iterator m_pos;
	sub_iterator m_sub_pos, m_sub_end;

	recursive_iterator() {}
public:

	recursive_iterator(iterator pos) : m_pos(pos), m_sub_pos((*pos).begin()), m_sub_end((*pos).end()) {}
	recursive_iterator(const_iterator pos) : m_pos(pos), m_sub_pos((*pos).begin()), m_sub_end((*pos).end()) {}

	reference operator*() { return *m_sub_pos; }
	const_reference operator*() const { return *m_sub_pos; }

	recursive_iterator& operator++()
	{
		++m_sub_pos;
		if(m_sub_pos == m_sub_end) {
			++m_pos;
			m_sub_pos = sub_iterator((*m_pos).begin());
			m_sub_end = sub_iterator((*m_pos).end());
	return *this;
		}
		return *this;
	}

	bool operator !=(const recursive_iterator& rhs) const
	{
		return m_sub_pos != rhs.m_sub_pos;
	}

	bool operator ==(const recursive_iterator& rhs) const
	{
		return m_sub_pos == rhs.m_sub_pos;
	}
};

template <typename ArrayView> class recursive_iterator<ArrayView, 1>
{
	typedef typename sub_iterator_type_gen<ArrayView, 1>::iterator iterator;
	typedef typename sub_iterator_type_gen<ArrayView, 1>::scalar_type scalar_type;

	typedef scalar_type& reference;
	typedef const scalar_type& const_reference;

	friend class recursive_iterator<ArrayView, 2>;

	iterator m_pos;

public:

	recursive_iterator(iterator pos) : m_pos(pos) {}
	static recursive_iterator find_end(ArrayView& view) { return recursive_iterator(view.end()); }

	reference operator*() { return *m_pos; }
	const_reference operator*() const { return *m_pos; }
	recursive_iterator& operator++() { ++m_pos; return *this; }
	bool operator !=(const recursive_iterator& rhs) const { return m_pos != rhs.m_pos; }
	bool operator ==(const recursive_iterator& rhs) const { return m_pos == rhs.m_pos; }
};

template <typename ArrayView> class recursive_iterator<const ArrayView, 1>
{
	typedef typename sub_iterator_type_gen<ArrayView, 1>::const_iterator const_iterator;
	typedef typename sub_iterator_type_gen<ArrayView, 1>::scalar_type scalar_type;

	typedef const scalar_type& const_reference;

	friend class recursive_iterator<const ArrayView, 2>;

	const_iterator m_pos;

public:

	recursive_iterator(const_iterator pos) : m_pos(pos) {}
	static recursive_iterator find_end(const ArrayView& view) { return recursive_iterator(view.end()); }

	const_reference operator*() const { return *m_pos; }
	recursive_iterator& operator++() { ++m_pos; return *this; }
	bool operator !=(const recursive_iterator& rhs) const { return m_pos != rhs.m_pos; }
	bool operator ==(const recursive_iterator& rhs) const { return m_pos == rhs.m_pos; }
};

} // core
} // adore

#endif // AARRAY_RECURSIVE_ITERATOR_H
