

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_FEATURESFEATUREPOINT_H
#define ADORE_FEATURESFEATUREPOINT_H

#include <stdint.h>
#include <adrcore/types/aarray.h>
#include <adrcore/types/feature.h>

namespace adore
{

	namespace core
	{

#ifndef NDEBUG
		template<class imgT> class ImagePlane;
#endif

		/**
		 
		*/
		class FeaturePoint : public Feature, public AVector
		{
			public:
				FeaturePoint( int typeID = 0 ):
						Feature( typeID ), AVector( 3 ),
#ifndef NDEBUG
						patch( NULL ),
#endif
						invAffine( 2, 2 )
				{
					operator()( 0 ) = 0;
					operator()( 1 ) = 0;
					operator()( 2 ) = 1;
					invAffine = INFINITY, INFINITY, INFINITY, INFINITY;
				}

				FeaturePoint( const AVector &vec, int typeID = 0 ):
						Feature( typeID ), AVector( vec ),
#ifndef NDEBUG
						patch( NULL ),
#endif
						invAffine( 2, 2 )
				{
					invAffine = INFINITY, INFINITY, INFINITY, INFINITY;
				}


				FeaturePoint( const double &x, const double &y, int typeID = 0 ):
						Feature( typeID ), AVector( 3 ),
#ifndef NDEBUG
						patch( NULL ),
#endif
						invAffine( 2, 2 )
				{
					operator()( 0 ) = x;
					operator()( 1 ) = y;
					operator()( 2 ) = 1;
					invAffine = INFINITY, INFINITY, INFINITY, INFINITY;
				}


				//taken out to prevent strengths interpreted as IDs
				/*FeaturePoint(const double &x, const double &y, const double &strength, int typeID = 0):
				        Feature(typeID, strength), AVector(3), invAffine(2, 2)
				{
				    operator()(0) = x;
				    operator()(1) = y;
				    operator()(2) = 1;
				    	invAffine = INFINITY, INFINITY, INFINITY, INFINITY;
				}*/

				~FeaturePoint();

				bool hasInvAffineMat() const {return invAffine( 0, 0 ) != INFINITY;}

				/**
				 * returns a 2x2 Matrix encoding affine region properties.
				 */
				AArray<float, 2> &getInvAffineMat() {return invAffine;}
				const AArray<float, 2>& getInvAffineMat() const { return invAffine; }

#ifndef NDEBUG
				ImagePlane<uint8_t> *patch;
#endif

				std::string toDbgString()
				{
					std::ostringstream oss;
					oss << "(" << operator()( 0 ) << ", " << operator()( 1 ) << ", " << operator()( 2 ) << ")";
					return oss.str();
				}

			protected:

				AArray<float, 2> invAffine;//co-varince matrix of the region; inv - inverted

		};

	}

}

#endif
