

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_COREIMAGECACHE_H
#define ADORE_COREIMAGECACHE_H

#include "view.h"
#include <adrcore/imgprocessing/imageplane.h>
#include <adrcore/util/scaleshifttrafo.h>
#include <map>

namespace adore
{

	namespace core
	{

		/**
		 * cache images to use inside views and switches them
			 
		*/
		template<class imgT>
		class ImageCache
		{
				typedef View<imgT> ViewT;
				typedef ImagePlane<imgT> ImageT;
				typedef vector<ImageT *> ImgColl;
				typedef std::map<ViewT *, ImgColl> ViewImgMap;
				typedef typename ViewImgMap::iterator mapIter;
				typedef typename ImgColl::iterator collIter;

				const int imgSpace;
				ViewImgMap map;

			public:
				ImageCache():imgSpace( 20 ) {;}

				void add( ViewT *v, ImageT *i, int pos )
				{
					mapIter it = map.find( v );
					if ( it == map.end() )
					{
						map[v] = ImgColl( imgSpace );
					}
					map[v][pos] = i;
				}

				ImageBase *get( ViewT *v, int pos )
				{
					return map[v][pos];
				}

				void switchTo( int pos )
				{
					for ( mapIter it = map.begin(); it != map.end(); ++it )
					{
						( *it ).first->setImage( ( *it ).second[pos] );
					}
				}

				void switchToAndAdjustAll( int pos )
				{
					for ( mapIter it = map.begin(); it != map.end(); ++it )
					{
						ViewT* view = ( *it ).first;
						bool denormalizeIt = false;
						if(!view->isNormalized()) {
							view->normalize();
							denormalizeIt = true;
						}

						ImageT* img = ( *it ).second[pos];
						view->setImage( img );

						double width = img->getWidth();
						double height = img->getHeight();
						getNormalizationTrafo( width, height, view->getTrafo() );
                        AMatrix & vInvTrafo = view->getInvTrafo();
						vInvTrafo = view->getTrafo();
						invertNormalizationMatrix ( vInvTrafo );

						if(denormalizeIt)
							view->denormalize();
					}
				}

				/**
				 * deletes every view and image
				 */
				void deleteEverything()
				{
					for ( mapIter it = map.begin(); it != map.end(); ++it )
					{
						for ( collIter cit = ( *it ).second.begin(); cit != ( *it ).second.end(); ++cit )
						{
							delete *cit;
						}
						DBGOUT( DBGNORMAL, "ImageCache: INFO: view %p deletes points: %s\n", ( *it ).first, ( ( *it ).first->getPointMode() == ViewBase::POINT_DELETE ) ? "true" : "false" );
						delete ( *it ).first;
					}
					map.clear();
				}

				/**
				 * deletes all associated images
				 */
				void deleteAllImages()
				{
					for ( mapIter it = map.begin(); it != map.end(); ++it )
					{
						for ( collIter cit = ( *it ).second.begin(); cit != ( *it ).second.end(); ++cit )
						{
							delete *cit;
						}
						( *it ).second.clear();
					}
				}

				/**
				 * deletes the associated at a specific position
				 */
				void deleteImagesAt( int pos )
				{
					for ( mapIter it = map.begin(); it != map.end(); ++it )
					{
						collIter eraseThis = ( *it ).second.begin() + pos;
						ImageT *img = *eraseThis;
						( *it ).second.erase( eraseThis );
						delete img;
					}
				}
		};

	}

}

#endif
