

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_UTILBLITZUTIL_H
#define ADORE_UTILBLITZUTIL_H

#include <vector>
#include <adrcore/types/aarray.h>

namespace adore
{

	namespace core
	{

		/**
			 
		*/
		template<class T_numtype>
		void addPts ( std::vector<AArray<T_numtype, 1> > &vec, int numPoints, int dim )
		{
			assert ( ( vec.size() == 0 ) || vec[0].rows() == dim );
			int size = vec.size();
			vec.resize ( size + numPoints );
			for ( int i = size; i < size + numPoints; ++i )
			{
				vec[i].resize ( dim );
			}
		}

		template<class T_numtype>
		void reallocPts ( std::vector<AArray<T_numtype, 1> *> &vec, int numPoints, int dim )
		{
			assert ( ( vec.size() == 0 ) || vec[0]->rows() == dim );
			int size = vec.size();

			if ( numPoints > size )
			{
				vec.resize ( numPoints );
				for ( int i = size; i < numPoints; ++i )
				{
					vec[i] = new AArray<T_numtype, 1> ( dim );
				}
			}
			else if ( numPoints < size )
			{
				for ( int i = numPoints; i < size; ++i )
				{
					delete vec[i];
				}
				vec.resize ( numPoints );
			}
		}

		template<class T_numtype>
		void reallocPts ( std::vector<AArray<T_numtype, 1> > &vec, int numPoints, int dim )
		{
                        assert ( ( vec.size() == 0 ) || (int) vec[0].rows() == dim );
			int size = vec.size();

			if ( numPoints > size )
			{
				vec.resize ( numPoints );
				for ( int i = size; i < numPoints; ++i )
				{
					vec[i].resize ( dim );
				}
			}
		}

		template<class T_numtype>
		inline void freePts ( std::vector<AArray<T_numtype, 1> *> &vec )
		{
			for ( typename std::vector<AArray<T_numtype, 1> *>::iterator it = vec.begin(); it != vec.end(); ++it )
			{
				delete *it;
			}
			vec.clear();
		}

		template<class T_numtype>
		void reallocMats ( std::vector<AArray<T_numtype, 2> *> &mat, int num, int dim1, int dim2 )
		{
                        assert ( ( mat.size() == 0 ) || ( (int) mat[0]->rows() == dim1 && (int) mat[0]->cols() == dim2 ) );
			int size = mat.size();

			if ( num > size )
			{
				mat.resize ( num );
				for ( int i = size; i < num; ++i )
				{
					mat[i] = new AArray<T_numtype, 2> ( dim1, dim2 );
				}
			}
			else if ( num < size )
			{
				for ( int i = num; i < size; ++i )
				{
					delete mat[i];
				}
				mat.resize ( num );
			}
		}

		template<class T_numtype>
		void reallocMats ( std::vector<AArray<T_numtype, 2> > &mat, int num, int dim1, int dim2 )
		{
			assert ( ( mat.size() == 0 ) || ( mat[0].rows() == dim1 && mat[0].cols() == dim2 ) );
			int size = mat.size();

			if ( num > size )
			{
				mat.resize ( num );
				for ( int i = size; i < num; ++i )
				{
					mat[i].resize ( dim1, dim2 );
				}
			}
		}

		template<class T_numtype>
		inline void freeMats ( std::vector<AArray<T_numtype, 2> *> &mat )
		{
			for ( typename std::vector<AArray<T_numtype, 2> *>::iterator it = mat.begin(); it != mat.end(); ++it )
			{
				delete *it;
			}

			mat.clear();
		}

		template <class T_numtype, class Comparator = std::less<T_numtype> >
		class LexCompareAVector
		{
			public:
				bool operator() ( const AArray<T_numtype, 1> *a, const AArray<T_numtype, 1> *b ) const
				{
					assert ( a );
					assert ( b );

					return operator() ( *a, *b );
				}

				bool operator() ( const AArray<T_numtype, 1> &a, const AArray<T_numtype, 1> &b ) const
				{
					assert ( a.size() == b.size() );
					for ( int i = 0; i < a.size(); ++i )
					{
						bool ab = comparator ( a ( i ), b ( i ) );
						bool ba = comparator ( b ( i ), a ( i ) );
						if ( ab != ba )
						{
							return ab;
						}
					}
					return false;
				}

			private:
				Comparator comparator;
		};

		template<class T_fromTypeIt, class T_toTypeIt>
		void typeCvt ( T_fromTypeIt sit, T_toTypeIt dit, uint num )
		{
			for ( int i = 0; i < num; ++i, ++sit, ++dit )
			{
				*dit = T_toTypeIt ( sit );
			}
		}

		template<class T_fromType, class T_toType, int T_dim>
		AArray<T_toType, T_dim> arCvt ( AArray<T_fromType, T_dim> &srcArr )
		{
			AArray<T_toType, T_dim> dstArr ( srcArr.shape() );
			assert ( srcArr.size() == dstArr.size() );

			typeCvt ( srcArr.begin(), dstArr.begin(), srcArr.size() );

			return dstArr;
		}

//		/**
//			lets the inner references of the second vector point to
//			the inner structures of the first vector
//		*/
//		template<class T>
//		inline void pointerToReference( std::vector<T*> &ptrVec, std::vector<T> &refVec )
//		{
//			for ( uint i = 0; i < refVec.size(); ++i )
//			{
//				refVec[i].reference( *ptrVec[i] );
//			}
//		}

	}

}

#endif
