

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef UTILAVGVAR_H
#define UTILAVGVAR_H

namespace adore
{
	namespace core
	{


		/**
		 * Computes the average and the variance of an array.
		 * @param array the array
		 * @param size the size of the array.
		 * @param avg will be altered to the average
		 * @param var will be altered to the variance
		 * @param mask an array to mask some values. These values are not taken into account.
		 */
		template <class T>
		void getAvgVar( T array[], int size, double &avg, double &var, bool *mask = NULL )
		{
			double sum = 0.0;
			int num = 0;

			for ( int i = 0 ; i < size; ++i )
			{
				if ( !mask || mask[i] )
				{
					sum += array[i];
					++num;
				}
			}
			avg = sum / double( num );

			sum = 0.0;
			for ( int i = 0 ; i < size; ++i )
			{
				if ( !mask || mask[i] ) sum += ( avg - array[i] ) * ( avg - array[i] );
			}
			var = sum / double( num );
		}

		/**
		 * Computes the average and the variance of a vector.
		 * @param array the vector
		 * @param avg will be altered to the average
		 * @param var will be altered to the variance
		 * @param mask a vector to mask some values. These values are not taken into account.
		 */
		template <class T>
		void getAvgVar( std::vector<T> &array, double &avg, double &var, std::vector<bool> &mask )
		{
			double sum = 0.0;
			int size = array.size();
			int num = 0;

			for ( int i = 0 ; i < size; ++i )
			{
				if ( mask[i] )
				{
					sum += array[i];
					++num;
				}
			}
			avg = sum / double( num );

			sum = 0.0;
			for ( int i = 0 ; i < size; ++i )
			{
				if ( mask[i] )
				{
					sum += ( avg - array[i] ) * ( avg - array[i] );
				}
			}
			var = sum / double( num );
		}

		/**
		 * Computes the average and the variance of a vector.
		 * @param array the vector
		 * @param avg will be altered to the average
		 * @param var will be altered to the variance
		 * @param mask a vector to mask some values. These values are not taken into account.
		 */
		template <class T>
		void getAvgVar( std::vector<T> &array, double &avg, double &var, std::vector<int> &mask )
		{
			double sum = 0.0;
			int size = array.size();
			int num = 0;

			for ( int i = 0 ; i < size; ++i )
			{
				if ( mask[i] )
				{
					sum += array[i];
					++num;
				}
			}
			avg = sum / double( num );

			sum = 0.0;
			for ( int i = 0 ; i < size; ++i )
			{
				if ( mask[i] )
				{
					sum += ( avg - array[i] ) * ( avg - array[i] );
				}
			}
			var = sum / double( num );
		}

		/**
		 * Computes the average and the variance of a vector.
		 * @param array the vector
		 * @param avg will be altered to the average
		 * @param var will be altered to the variance
		 */
		template <class T>
		void getAvgVar( std::vector<T> &array, double &avg, double &var )
		{
			double sum = 0.0;
			int size = array.size();
			int num = 0;

			for ( int i = 0 ; i < size; ++i )
			{
				sum += array[i];
				++num;
			}
			avg = sum / double( num );

			sum = 0.0;
			for ( int i = 0 ; i < size; ++i )
			{
				sum += ( avg - array[i] ) * ( avg - array[i] );
			}
			var = sum / double( num );
		}

		template <class IteratorT>
		void getAvgVar( IteratorT begin, int size, double &avg, double &var )
		{
			double sum = 0.0;
			int num = 0;

			IteratorT start = begin;
			for ( int i = 0 ; i < size; ++i )
			{
				sum += *start;
				++num;
				++start;
			}
			avg = sum / double( num );

			sum = 0.0;
			for ( int i = 0 ; i < size; ++i )
			{
				sum += ( avg - *begin ) * ( avg - *begin );
				++begin;
			}
			var = sum / double( num );
		}


		/**
		 * Computes the average of an array.
		 * @param array the array
		 * @param size the size of the array.
		 * @param avg will be altered to the average
		 * @param mask an array to mask some values. These values are not taken into account.
		 */
		template <class T>
		void getAvg( T array[], int size, double &avg, bool *mask = NULL )
		{
			double sum = 0.0;
			int num = 0;

			for ( int i = 0 ; i < size; ++i )
			{
				if ( !mask || mask[i] )
				{
					sum += array[i];
					++num;
				}
			}
			avg = sum / double( num );
		}

		/**
		 * Computes the average of a vector.
		 * @param array the vector
		 * @param avg will be altered to the average
		 * @param mask a vector to mask some values. These values are not taken into account.
		 */
		template <class T>
		void getAvg( std::vector<T> &array, double &avg, std::vector<bool> &mask )
		{
			double sum = 0.0;
			int size = array.size();
			int num = 0;

			for ( int i = 0 ; i < size; ++i )
			{
				if ( mask[i] )
				{
					sum += array[i];
					++num;
				}
			}
			avg = sum / double( num );
		}

		/**
		 * Computes the average of a vector.
		 * @param array the vector
		 * @param avg will be altered to the average
		 */
		template <class T>
		void getAvg( std::vector<T> &array, double &avg )
		{
			double sum = 0.0;
			int size = array.size();
			int num = 0;

			for ( int i = 0 ; i < size; ++i )
			{
				sum += array[i];
				++num;
			}
			avg = sum / double( num );
		}


		template <class IteratorT>
		double getAvg( IteratorT begin, int size )
		{
			double sum = 0.0;

			for ( int i = 0 ; i < size; ++i )
			{
				sum += *begin;
				++begin;
			}
			return sum / double( size );
		}

	}
}

#endif

