

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include <cassert>
#include "filelocator.h"
#include <libgen.h>
#include <sstream>
#include <cstdlib>
#include "stlutil.h"

namespace adore
{

	namespace core
	{

		string moveName( string oldPath, string newPath )
		{
			std::ostringstream oss;

			char *_base = strdup( oldPath.c_str() );
			char *base = basename( _base );

			if ( strcmp( oldPath.c_str(), base ) == 0 )
			{
				oss << newPath << "/" << base;
			}
			else
			{
				if ( pathIsRelative( newPath.c_str() ) )
				{
					char *_dir = strdup( oldPath.c_str() );
					oss << dirname( _dir ) << "/" << newPath << "/" << base;
					free( _dir );
				}
				else oss << newPath << "/" << base;
			}

			free( _base );
			return oss.str();
		}

		string replacePath( string oldPath, string newPath )
		{
			std::ostringstream oss;

			char *_base = strdup( oldPath.c_str() );
			char *base = basename( _base );
			oss << newPath << "/" << base;
			free( _base );
			return oss.str();
		}

		string extractPath( string file )
		{
			char *_base = strdup( file.c_str() );
			char *base = dirname( _base );
			string ret( base );
			free( _base );
			return ret;
		}

		string extractName( string file )
		{
			char *_base = strdup( file.c_str() );
			char *base = basename( _base );
			string ret( base );
			free( _base );
			return ret;
		}
		
		string removeExt( string file )
		{
			uint pos = file.rfind( '.' );
			if ( pos >= file.length() - 1 ) return file;
			return string( file, 0, pos );
		}

		/**
		 * returns lowercased file extension
		 */
		string fileExt( string file )
		{
			uint pos = file.rfind( '.' );
			if ( pos >= file.length() - 1 ) return "";
			++pos;
			string ext( file, pos, file.length() - pos );
			return stringToLower( ext );
		}

		/**
		 * moves a file. newFile may be constructed with moveName
		 * */
		bool moveFile( string oldFile, string newFile )
		{
			if ( rename( oldFile.c_str(), newFile.c_str() ) != 0 )
			{
				printf( "moveFile: Can't move file\n" );
				return false;
			}
			return true;
		}

		bool createDir( char *dir )
		{
			if ( !strlen( dir ) ) return false;
			if ( exists( dir ) ) return true;

			bool result = true;
			if ( mkdir( dir, 0777 ) != 0 )
			{
				char *copy = strdup( dir );
				dir = dirname( dir );
				if ( createDir( dir ) == 0 )//try with topdir
				{
					//success, try again
					if ( mkdir( copy, 0777 ) != 0 ) result = false;
				}
				else result = false;
				free( copy );
			}
			return result;
		}

		bool createDir( string dir )
		{
			char *d = strdup( dir.c_str() );
			bool result = createDir( d );
			free( d );
			return result;
		}


		//////////////////////////////////////////////////////////////////////////

		const int FileLocator::bufLen( FL_BUF__LEN );

		FileLocator::FileLocator()
		{
			buffer[bufLen - 1] = '\0';
		}


		FileLocator::~FileLocator()
		{
		}

		void FileLocator::addPath( const char *path )
		{
			if ( !path ) return;
			if ( strlen( path ) < 1 ) return;
			assert( strlen( path ) < bufLen - 1 );

			if ( !exists( path ) ) return;

			if ( S_ISDIR( s.st_mode ) )
			{
				std::string s( path );
				paths.push_back( s );
			}
			else
			{
				//chop filename and addPath
				strncpy( buffer, path, bufLen - 1 );
				addPath( dirname( buffer ) );
			}
		}

		std::string FileLocator::getLocation( const char *path ) const
		{
			printf( "FileLocator testing %s\n", path );
			if ( exists( path ) ) return std::string( path );

			for ( PathsT::const_iterator it = paths.begin(); it != paths.end(); ++it )
			{
				std::string test = *it + "/" + path;
				printf( "FileLocator testing %s\n", test.c_str() );
				if ( exists( test.c_str() ) ) return test;
			}

			return "";
		}

	}
}
