

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_UTILFLOATSHIFT_H
#define ADORE_UTILFLOATSHIFT_H

#include <cfloat>
#include <climits>
#include <cassert>
#include <cstring>
#include <cstdio>
#include <algorithm>
#include <cmath>

namespace adore
{

	namespace core
	{

		/**
			 
		*/

		/**
		 * returns the expo
		 */
		int getBinaryExponent( double d );

		int getBinaryExponent( float a );

		double shift( double d, int digits );

		float shift( float d, int digits );


		template<class IteratorT>
		bool getAverageBinaryExponent( IteratorT begin, IteratorT end, int &expo )
		{
			expo = 0;
			int dist = 0;
			for ( ; begin != end; ++begin )
			{
				if ( fabs( *begin ) > FLT_EPSILON )
				{
					expo += getBinaryExponent( *begin );
					++dist;
				}
			}
			if ( !dist ) return false;
			expo = expo / dist;
			return true;
		}

		template<class IteratorT>
		bool getMinMaxBinaryExponent( IteratorT begin, IteratorT end, int &min, int &max )
		{
			min = INT_MAX;
			max = INT_MIN;
			if ( begin == end ) return false;
			for ( ; begin != end; ++begin )
			{
				if ( fabs( *begin ) > FLT_EPSILON ) min = max = getBinaryExponent( *begin );
			}
			if ( begin == end ) return false;
			for ( ; begin != end; ++begin )
			{
				if ( fabs( *begin ) > FLT_EPSILON )
				{
					int expo = getBinaryExponent( *begin );
					min = expo < min ? expo : min;
					max = expo > max ? expo : max;
				}
			}
			return ( min != INT_MAX ) && ( max != INT_MIN );
		}

		/**
		 * let the average exponent be 1
		 */
		template<class IteratorT>
		bool normalizeExponentsAverage( IteratorT begin, IteratorT end )
		{
			int expo = 0;
			bool success = getAverageBinaryExponent( begin, end, expo );
			if ( success && expo )
			{
				double factor = shift( 2.0, -expo );
				for ( ; begin != end; ++begin )
				{
					*begin *= factor;
				}
			}
			return success;
		}

		/**
		 * let min and max exponent add up to zero
		 */
		template<class IteratorT>
		bool normalizeExponentsMinMax( IteratorT begin, IteratorT end )
		{
			int min, max;
			bool success = getMinMaxBinaryExponent( begin, end, min, max );
			int expo = ( min + max ) >> 1;
			if ( success && expo )
			{
				double factor = shift( 2.0, -expo );
				for ( ; begin != end; ++begin )
				{
					*begin *= factor;
				}
			}
			return success;
		}

		/**
		 * let the squared sum of all elements be equal to the number of elements
		 */
		template<class IteratorT>
		bool normalizeExponentsSqrSum( IteratorT begin, IteratorT end )
		{
			uint num = std::distance( begin, end );
			if ( num < 1 ) return false;

			double vals[num];
			IteratorT iter = begin;
			for ( int i = num - 1; i >= 0; ++iter, --i ) { vals[i] = *iter; }
			std::sort( &vals[0], &vals[num] );

			//sum from small to large
			double sum = 0.0;
			for ( uint i = 0; i < num; ++i ) { sum += vals[i] * vals[i]; }
			if ( sum <= DBL_EPSILON ) return false;

			double factor = sqrt( double( num )/sum );
			for ( ; begin != end; ++begin ) {*begin *= factor;}
			return true;
		}

		/**
		    	returns true on success
		    */
		template<class IteratorT>
		bool normalizeExponents( IteratorT begin, IteratorT end )
		{
			return normalizeExponentsAverage( begin, end );
//			return normalizeExponentsSqrSum( begin, end );
		}

	}

}

#endif
