

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORETEMPLATELIMITS_H
#define ADORETEMPLATELIMITS_H

#include <cfloat>
#include <adrcore/types/commontypes.h>
#include <limits.h>

namespace adore
{
	namespace core
	{

		/**
		 * base template for maxLimit. Specializations below.
		 * @return 0;
		 */
		template < class T >
		inline T maxLimit() {return T( 0 );}

		/**
		 * base template for minLimit. Specializations below.
		 * @return 0;
		 */
		template < class T >
		inline T minLimit() {return T( 0 );}

		template < class T >
		inline T epsilon() {return T( 0 );}

		template <>
		inline float epsilon<float>() {return FLT_EPSILON;}

		template <>
		inline double epsilon<double>() {return FLT_EPSILON /*sic!*/;}

		template <>
		inline long double epsilon<long double>() {return LDBL_EPSILON;}

		/**
		 * maxLimit
		 * @return the maximum possible value.
		 */
		template <>
		inline long double maxLimit<long double>() {return LDBL_MAX;}

		/**
		 * maxLimit
		 * @return the maximum possible value.
		 */
		template <>
		inline double maxLimit<double>() {return DBL_MAX;}

		/**
		 * maxLimit
		 * @return the maximum possible value.
		 */
		template <>
		inline float maxLimit<float>() {return FLT_MAX;}

		/**
		 * maxLimit
		 * @return the maximum possible value.
		 */
		template <>
		inline int8_t maxLimit<int8_t>() {return 127;}

		/**
		 * maxLimit
		 * @return the maximum possible value.
		 */
		template <>
		inline uint8_t maxLimit<uint8_t>() {return 255;}


		/**
		 * maxLimit
		 * @return the maximum possible value.
		 */
		template <>
		inline int16_t maxLimit<int16_t>() {return SHRT_MAX;}

		/**
		 * maxLimit
		 * @return the maximum possible value.
		 */
		template <>
		inline uint16_t maxLimit<uint16_t>() {return USHRT_MAX;}

		/**
		 * maxLimit
		 * @return the maximum possible value.
		 */
		template <>
		inline int32_t maxLimit<int32_t>() {return INT_MAX;}

		/**
		 * maxLimit
		 * @return the maximum possible value.
		 */
		template <>
		inline uint32_t maxLimit<uint32_t>() {return UINT_MAX;}


		/**
		 * minLimit
		 * @return the minimum possible value.
		 */
		template <>
		inline long double minLimit<long double>() {return -LDBL_MAX;}

		/**
		 * minLimit
		 * @return the minimum possible value.
		 */
		template <>
		inline double minLimit<double>() {return -DBL_MAX;}

		/**
		 * minLimit
		 * @return the minimum possible value.
		 */
		template <>
		inline float minLimit<float>() {return -FLT_MAX;}

		/**
		 * minLimit
		 * @return the minimum possible value.
		 */
		template <>
		inline int8_t minLimit<int8_t>() {return -128;}

		/**
		 * minLimit
		 * @return the minimum possible value.
		 */
		template <>
		inline uint8_t minLimit<uint8_t>() {return 0;}

		/**
		 * minLimit
		 * @return the minimum possible value.
		 */
		template <>
		inline int16_t minLimit<int16_t>() {return SHRT_MIN;}

		/**
		 * minLimit
		 * @return the minimum possible value.
		 */
		template <>
		inline uint16_t minLimit<uint16_t>() {return 0;}

		/**
		 * minLimit
		 * @return the minimum possible value.
		 */
		template <>
		inline int32_t minLimit<int32_t>() {return INT_MIN;}

		/**
		 * minLimit
		 * @return the minimum possible value.
		 */
		template <>
		inline uint32_t minLimit<uint32_t>() {return 0;}

	}
}
#endif
