

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_COREKDTREEMATCHER_H
#define ADORE_COREKDTREEMATCHER_H

#include "matcher.h"
#include <adrcore/types/descriptorbackref.h>

namespace adore
{

	namespace core
	{
		class MatchSorter;

		enum KDTREE_MATCH_MODE
		{
			KDTREE_SINGLE_BEST,
			KDTREE_BBF_SINGLE_BEST,
			KDTREE_RATIO_BEST,
			KDTREE_BBF_RATIO_BEST,
			KDTREE_BETTER_THAN_RATIO,
			KDTREE_BBF_BETTER_THAN_RATIO
		};

		/**
			 
		*/
		class KDTreeMatcher : public Matcher
		{
			public:
				typedef FeatureDescriptor::DescVecT DescVecT;
				typedef DescVecT::scalar_type T_numtype;
				typedef DescriptorBackRef<FeaturePoint *> DBackRef;
				typedef DBackRef::Ruler BackRefRuler;
				typedef vector<DBackRef> CollectionT;
				typedef CollectionT::iterator iterator;

			private:
				int maxBBFNodes;
				KDTREE_MATCH_MODE mode;
				CollectionT backrefs1;
				CollectionT backrefs2;

				float thresh; //theshold on matching badness
				int countThresh; //theshold on number of corrs
				float goodRatio;//dist1 / dist2 <= goodRatio to let the match pass

				MatchSorter *waste;

			public:
				KDTreeMatcher ( KDTREE_MATCH_MODE _mode = KDTREE_SINGLE_BEST ) :
						Matcher(),maxBBFNodes ( 100 ), mode ( _mode ), thresh ( FLT_MAX ), countThresh ( INT_MAX ), goodRatio ( 0.7F ), waste ( NULL ) {;}

				KDTreeMatcher ( const KDTreeMatcher& km ):Matcher( km ), maxBBFNodes( km.maxBBFNodes ), mode ( km.mode ), thresh ( km.thresh ), countThresh ( km.countThresh ), goodRatio ( km.goodRatio ), waste ( km.waste )
				{
					assert( waste == NULL );//todo: handle waste
				}

				~KDTreeMatcher() {;}

				virtual NViewOperator* clone() {return new KDTreeMatcher( *this );}

				void match ( ViewBase *v1, ViewBase *v2, NView *set );

				void setMode ( const KDTREE_MATCH_MODE theValue )
				{
					mode = theValue;
				}

				KDTREE_MATCH_MODE getMode() const
				{
					return mode;
				}

				void setDistThresh ( const float theValue )
				{
					thresh = theValue;
				}

				float getDistThresh() const
				{
					return thresh;
				}

				void setCountThresh ( int theValue )
				{
					countThresh = theValue;
				}

				int getCountThresh() const
				{
					return countThresh;
				}

				void setGoodRatio ( float theValue )
				{
					goodRatio = theValue;
				}

				float getGoodRatio() const
				{
					return goodRatio;
				}

				void setWasteBin ( MatchSorter *ms )
				{
					DBGOUT ( DBGNORMAL, "KDTreeMatcher: got wastebin\n" );
					waste = ms;
				}

				MatchSorter *getWasteBin() const
				{
					return waste;
				}

				void setMaxBBFNodes ( int theValue )
				{
					maxBBFNodes = theValue;
				}


				int getMaxBBFNodes() const
				{
					return maxBBFNodes;
				}


			private:
				void takeVecs ( ViewBase *v, CollectionT &backrefs );
				void match ( NView *set );

			protected:
				void matchDerived ( ViewT *v1, ViewT *v2, NView *set )
				{match ( static_cast<ViewBase *> ( v1 ), static_cast<ViewBase *> ( v2 ), set );}


		};



	}
}//Namespace

#endif
