

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include <adrcore/viewmatch/scaledogfinder.h>
#include <adrcore/types/view.h>
#include <adrcore/imgprocessing/kernelbinom_3_7__3_3.h>
#include <adrcore/imgprocessing/kernelbinom_delta_3_3.h>
#include <adrcore/imgprocessing/convolver2d.h>
#include <adrcore/imgprocessing/scalespacetraverser.h>
#include <adrcore/imgprocessing/rgbtogray.h>
#include <adrcore/imgprocessing/borderfiller.h>
#include <adrcore/features/featurefilter.h>
#include <adrcore/imgprocessing/scalespacedog.h>
#include <memory>

namespace adore
{
	namespace core
	{

		ScaleDOGFinder::ScaleDOGFinder ( uint ID, int _numImages, calcT minHarrisThresh, int maxPoints , float minDistance, float minFrac )
				: FeatureFinder(),
				fid ( ID ),
				patchImg ( 25, 25, 1 ),
				extremaG ( minHarrisThresh ),
				extremaL ( -minHarrisThresh ),
				candidates ( false ),//don't delete points
				numSmooths ( _numImages + 1 ),
				numImages ( _numImages ),
				descriptorcreator ( NULL ),
				maxPts ( maxPoints ),
				minDist ( minDistance ),
				minFraction ( minFrac ),
				currentView ( 0 ),
				patchFactor ( 2 ),
				snsRemove ( false ),
				snsMat ( 2, 2 )
		{
			assert ( numImages > 0 );
			currentView.setPointMode ( ViewBase::POINT_FORGET );//prevent Featurepoint deletion
			snsMat.setIdentity();
		}

		ScaleDOGFinder::ScaleDOGFinder ( const ScaleDOGFinder &sdf )
				:FeatureFinder(),
				fid ( sdf.fid ),
				patchImg ( 25, 25, 1 ),
				extremaG ( sdf.extremaG.getThresh() ),
				extremaL ( sdf.extremaL.getThresh() ),
				candidates ( false ),//don't delete points
				numSmooths ( sdf.numSmooths ),
				numImages ( sdf.numImages ),
				descriptorcreator ( NULL ),
				maxPts ( sdf.maxPts ),
				minDist ( sdf.minDist ),
				minFraction ( sdf.minFraction ),
				currentView ( 0 ),
				patchFactor ( 2 ),
				snsRemove ( false ),
				snsMat ( 2, 2 )
		{
			assert ( numImages > 0 );
			currentView.setPointMode ( ViewBase::POINT_FORGET );//prevent Featurepoint deletion
			snsMat.setIdentity();
		}

		unsigned int ScaleDOGFinder::findFeaturesDerived ( ViewT *v )
		{

			ImagePlane<imgT> *chan = v->getImage();
			assert ( chan );

			assert ( !descriptorcreator || descriptorcreator->getFeatureID() == fid );

			candidates.init ( 0, 0, chan->getWidth(), chan->getHeight() );

			ImagePlane<imgT> *gray = NULL;
			if ( chan->getNumChannels() ==3 )
			{
				gray = new ImagePlane<imgT> ( ( int ) v->getImage()->getWidth(), ( int ) v->getImage()->getHeight(),1 );
				RGBtoGray<imgT> converter;
				Channel<imgT>::Set in ( true );
				chan->createChannels ( in );
				converter.process ( in, gray );
				chan=gray;
			};
			auto_ptr< ImagePlane<imgT> > grayDeletor ( gray ); //does nothing if gray is NULL
			dogImg.init ( gray->getWidth(), gray->getHeight(), 1 );

			//this convolver is used to generate the scale space
			typedef Convolver2D<KernelBinom_3_7__3_3<uint8_t> > Convolver2DT;
			Convolver2DT convolver2d;
			float delta = 3.0F / 7.0F;
			scale = getPreSmoothingAmount ( delta );

			//this convolver is used for a pre-smoothing
			typedef Convolver2D<KernelBinom_delta_3_3<uint8_t> > PreConvolver2DT;
			PreConvolver2DT preConvolver2d;
			preConvolver2d.getKernel().setDelta ( scale );

			//build the traverser
			typedef ScaleSpaceTraverser<Convolver2DT, PreConvolver2DT> ScaleSpaceTraverserT;
			ScaleSpaceTraverserT sstv ( chan, ScaleSpaceTraverserT::STATE_SMOOTH, //TODO PRESMOOTH?
			                            numSmooths /*smooths until the first subsampling*/,
			                            numSmooths /*smooths between subsamplings*/,
			                            2 /*number of images cached*/ );
			sstv.setConvolver ( &convolver2d );
			sstv.setPreConvolver ( &preConvolver2d );

			ImagePlane<imgT> *plane = chan;
			ImagePlane<imgT> *lastPlane = chan;

			scale = 1.0F;//TODO
			DBGOUT ( DBGNORMAL, "computing %d dogs\n", numImages );
			for ( int i = 0; i < numImages; ++i )
			{
				plane = sstv.next();
				lastPlane = sstv ( 1 );
				calcDOG ( lastPlane, plane, &dogImg );

//				dogImg.save( "dog.png" ); pressKey();

				//prepare the view for the descriptorcreator
				//and for affine matrix creation
				if ( descriptorcreator )
				{
					DBGOUT ( DBGSDF, "resetting view\n" );
					currentView.reset();
				}
				DBGOUT ( DBGSDF, "setting image\n" );
				currentView.setImage ( plane );
				DBGOUT ( DBGSDF, "adopting dimensions\n" );
				currentView.adoptDimensionsFromImage();
				DBGOUT ( DBGSDF, "initializing\n" );
				currentView.initialize();

				spacing = sstv.getSpacing();

				//adds the features to currentVIew and to candidates
				extremaG ( dogImg, *this );
//				DBGOUT( DBGSDF, "G: %d features\n", candidates.getSize() );
//				pressKey();
				extremaL ( dogImg, *this );
//				DBGOUT( DBGSDF, "L: %d features\n", candidates.getSize() );
//				pressKey();

//				harrisImg.save( "harry.png" );pressKey();

				if ( descriptorcreator )
				{
					descriptorcreator->performAction ( &currentView );
					//move features and add to candidates TODO
					for ( ViewT::PointsIterator it = currentView.getPointsBegin ( fid );
					                it != currentView.getPointsEnd ( fid ); ++it )
					{
						//scale now
						( **it ) ( 0 ) = getPixelPosition( ( **it ) ( 0 ), spacing );
						( **it ) ( 1 ) = getPixelPosition( ( **it ) ( 1 ), spacing );
						candidates.add ( *it );
					}
				}

				DBGOUT ( DBGSDF, "%d features at scale %f (maxStrength is %f at spacing %d)\n", candidates.getSize(), scale, ( *candidates.getBegin() )->getStrength(), spacing );
				scale += sstv.getScaleStep ( 1, delta );
//				pressKey();
			}
			currentView.reset();//clean currentview finally

			DBGOUT ( DBGSDF, "ScaleDog: filtering features\n" );
			filterFeatures ( candidates, minDist, minFraction, maxPts );
			DBGOUT ( DBGSDF, "%d features after filtering\n", candidates.getSize() );

			FeatureGrid::Iterator it = candidates.getBegin();
			FeatureGrid::Iterator endIt = candidates.getEnd();
			while ( it != endIt )
			{
				FeaturePoint *fp = *it;
				fp->setView ( v );

				FeaturePoint *fpSame = v->pick ( ( *fp ) ( 0 ), ( *fp ) ( 1 ), fid );
				if ( fpSame )
				{
					DBGOUT ( DBGSDF, "point occupied by scale %f (tried %f)\n", fpSame->getScale(), fp->getScale() );
				}

				if ( v->add ( fp ) != GRID_SUCCESS )
				{
					++it;
					delete fp;
					continue;
				}

				++it;
			}

			DBGOUT ( DBGSDF, "view has %d feature points\n", v->getNumFeaturePoints() );
			return v->getNumFeaturePoints();
		}

	}

}
