

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "viewknitter.h"
#include <adrcore/types/nview.h>
#include <map>

namespace adore
{

	namespace core
	{

		void ViewKnitter::uniSection ( VTuple *v1, VTuple *v2, VTuple &uni, VTuple &sec )
		{
			uni.reserve ( v1->size() + v2->size() );
			sec.reserve ( MIN ( v1->size(), v2->size() ) );

			//add views of t1
			for ( VTuple::iterator it = v1->begin(); it != v1->end(); ++it )
			{
				uni.push_back ( *it );
			}
			//add views of t2, that are not present in t1
			for ( VTuple::iterator it = v2->begin(); it != v2->end(); ++it )
			{
				bool unknown = true;
				for ( VTuple::iterator tit = v1->begin(); tit != v1->end(); ++tit )
				{
					if ( *tit == *it )
					{
						unknown = false;
						break;
					}
				}
				if ( unknown ) {uni.push_back ( *it );}
				else {sec.push_back ( *it );}
			}
		}

		//another dumb implementation
		//for those subclasses that dont come up with a more clever one
		int ViewKnitter::combineBasic ( int overlap )
		{
			assert ( curPos > 0 );
			assert ( ( int ) tuples.size() >= curPos );//equal is okay, because of the check in addTuple
			int usePos = curPos - 1;
			assert ( tuples[usePos].size() > 0 );
			VTuple uni, sec;

			int tupleCount = 0;
			DBGOUT ( DBGNORMAL, "ViewKnitter - combining (largest is %d, overlap shall be %d, usePos: %d, num outer tuples: %zd)\n", getNumViewsOfUnion ( usePos ), overlap, usePos, tuples[usePos].size() );
			ViewKnitter::iterator it1_end = end ( usePos ); --it1_end;
			for ( ViewKnitter::iterator it1 = begin ( usePos ); it1 != it1_end; ++it1 )
			{
				ViewKnitter::iterator it2 = it1; ++it2;
				for ( ; it2 != end ( usePos ); ++it2 )
				{
					VTuple *vt1 = ( *it1 ).get<2>();
					VTuple *vt2 = ( *it2 ).get<2>();
					uni.clear(); sec.clear();
					uniSection ( vt1, vt2, uni, sec );
//					DBGOUT( DBGNORMAL, "%d - %d\n", uni.size(), sec.size() );
					if ( ( int ( sec.size() ) == overlap ) || ( overlap == -1 && sec.size() > 0 && uni.size() > vt1->size() && uni.size() > vt2->size() ) )
					{
						DBGOUT ( DBGALL, "ViewKnitter - adding (%zd, %zd)->%zd/%zd\n", vt1->size(), vt2->size(), uni.size(), sec.size() );

						this->addTuple ( vt1, vt2, uni, sec );
						++tupleCount;
					}
				}
			}

			return tupleCount;
		}

		void ViewKnitter::addTuple ( ViewBase *v1, ViewBase *v2 )
		{
			vtuples.push_back ( VTuple ( 1, v1 ) );
			VTuple &v1t = vtuples.back();
			vtuples.push_back ( VTuple ( 1, v2 ) );
			VTuple &v2t = vtuples.back();
			vtuples.push_back ( VTuple ( 2 ) );
			VTuple &t = vtuples.back();
			VTuple &empty = vtuples.front();
			t[0] = v1;
			t[1] = v2;
			if ( ( int ) tuples.size() < curPos + 1 ) tuples.push_back ( InnerTupleCollT() );
			tuples[curPos].push_back ( make_tuple ( &v1t, &v2t, &t, &empty ) );
			DBGOUT ( DBGNORMAL, "ViewKnitter::addTuple: curPos: %d, size: %d\n", curPos, ( int ) tuples.size() );
		}

		void ViewKnitter::createPlan ( int overlap, bool rewind, int maxSteps, bool filterFromTop )
		{
			assert ( getPos() == 0 );//start with pairs
			int nodes = 0;
			do
			{
				next();//increments curPos
				nodes = combine ( overlap );
			}
			while ( nodes > 0 && --maxSteps);

			//if the top combination does not span all views, combine them despite the large overlap
			if ( curPos > 0 && maxSteps)
			{
				int numViews = countViews();
				int lastoverlap = ( getNumViewsOfUnion( curPos - 1 ) << 1 ) - numViews;
				DBGOUT ( DBGNORMAL, "%d views counted, lastoverlap: %d\n", numViews, lastoverlap );
				assert( lastoverlap > overlap );
				nodes = combine ( lastoverlap );//needs no next(), because the last combine failed
			}
			if ( nodes == 0 ) prev(); //not successful

			DBGOUT ( DBGNORMAL, "createPlan created %d steps (%d knits before filtering)\n", curPos, size() );

			//filter superfluous tuples that may exists
			if(filterFromTop) confirmPlan();

			if ( rewind ) this->rewind();
			
			DBGOUT ( DBGNORMAL, "createPlan leaves at knit step %d with %d knits to do\n", curPos, size() );
		}

		void ViewKnitter::confirmPlanBasic()
		{
			vector<VTuple *> goodOnes[2];
			goodOnes[0].reserve ( 50 );
			goodOnes[1].reserve ( 50 );
			int which = 0;

			//confirm the first top tuple as solution, the other remove
			int s = numSteps() - 1;
			iterator it = begin ( s ), it_end = end ( s );
			Tuple &tup = *it; ++it;
			goodOnes[which].push_back ( tup.get<0>() );
			goodOnes[which].push_back ( tup.get<1>() );
			while ( it != it_end )
			{
				iterator oldIt = it; ++it;
				tuples[s].erase ( oldIt );
			}

			//mark all tuples referenced by top tuples, the others may perish
			for ( s = numSteps() - 2; s >= 1; --s )
			{
				it = begin ( s ); it_end = end ( s );
				while ( it != it_end )
				{
					iterator oldIt = it; ++it;

					Tuple &tup = *oldIt;
					VTuple *testee = tup.get<2>();

					if ( std::find ( goodOnes[which].begin(), goodOnes[which].end(), testee ) == goodOnes[which].end() )
					{
						//remove it
						tuples[s].erase ( oldIt );
					}
					else
					{
						goodOnes[1 - which].push_back ( tup.get<0>() );
						goodOnes[1 - which].push_back ( tup.get<1>() );
					}
				}
				goodOnes[which].clear();
				which = 1 - which;
			}
		}

		int ViewKnitter::countViews()
		{
			set<ViewBase *> views;
			for ( iterator it = begin( 0 ); it != end( 0 ); ++it )
			{
				VTuple *v1 = ( *it ).get<0>();
				assert( v1->size() == 1 );
				VTuple *v2 = ( *it ).get<1>();
				assert( v2->size() == 1 );
				views.insert( ( *v1 )[0] );
				views.insert( ( *v2 )[0] );
			}
			return views.size();
		}

		void ViewKnitter::createPairs( NViewMap &map )
		{
			ViewRelations &vrela = map.getViewRelations();
			//printf("num views in viewrelations: %d\n", vrela.getNumViews());
			for ( uint i = 0; i < vrela.getNumViews(); ++i )
			{
				ViewBase *v1 = vrela.getView( i );
				for ( ViewRelations::IdIterator it = vrela.begin( i ); it != vrela.end( i ); ++it )
				{
					ViewBase *v2 = vrela.getView( *it );
					if ( *it > i ) addTuple ( v1, v2 );
				}
			}
		}

	}
}
