

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "viewrelatethumbsproxy.h"
#include <adrcore/imgprocessing/thumbnailer.h>
#include <adrcore/viewmatch/scaleharrisfinder.h>
#include <adrcore/viewmatch/siftdescriptor.h>
#include <adrcore/surf/surffinder.h>
#include <adrcore/surf/surfdescriptor.h>
#include <adrcore/util/recommendations.h>

namespace adore
{

	namespace core
	{

		//creates thumbnails and uses them to find close images
		void ViewRelateThumbsProxy::createRelations ( ViewBase **vit, int numViews, ViewRelations &vrela )
		{
			assert( relater );
			if ( !relater ) return;
			if ( numViews < 1 ) return;

//			const int numPoints = 400;
			const int extend = 400;

			const int minExt = 50;
			const int numShrinks = int ( log2 ( extend / minExt ) + 0.5 );
			ADORE_UNUSED( numShrinks );
//			const int numSmooths = 4;

			DBGOUT ( DBGNORMAL, "KnitProxy: extend: %d, numShrinks = %d\n", extend, numShrinks );

			vector<View<uint8_t> *> thumbs ( numViews );
			createThumbViews ( vit, numViews, extend, thumbs.begin() );

			//now the thumbs are created - find features

			DBGOUT ( DBGNORMAL, "KnitProxy: thumb(%f, %f)\n", ( *thumbs.begin() )->getWidth(), ( *thumbs.begin() )->getHeight() );

//			ScaleHarrisFinder featfind ( 0/*id*/, numSmooths, numSmooths * ( numShrinks + 1 ), ( int ) getGoodMinStrength<ScaleHarrisFinder>() /*harris strength*/, numPoints/*maxPts*/ );
//			SiftDescriptor sifter;
//			featfind.setDescriptorCreator ( &sifter );
//			featfind.setSnsRemove( false );
//			featfind.findFeatures ( thumbs.begin(), numViews );


			SurfFinder finder( 0, 1/*octave*/, 3/*images*/, ( int )getGoodMinStrength< SurfFinder >() );
			SurfDescriptor surfer;
			finder.setDescriptorCreator( &surfer );
			finder.findFeatures( thumbs.begin(), numViews );

			DBGOUT ( DBGNORMAL, "KnitProxy: features computed\n" );

			relater->createRelations ( thumbs.begin(), numViews, vrela );

			//delete thumbs
			for ( vector<View<uint8_t> *>::iterator it = thumbs.begin(); it != thumbs.end(); ++it )
			{
				delete *it;
			}

			//modify views inside vrela to point to the non-thumb-views
			for ( int i = 0; i < numViews; ++i ) vrela.replaceView( i, vit[i] );
		}

	}
}

