

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "glvisualizerdots.h"

namespace adore {
namespace visual {

GLVisualizerDots::GLVisualizerDots()
 : GLVisualizerMouse(),
 inited(false),
 displayCams(true),
 points(0),
 cams(0),
 camOff(1000000),
nv_ref(0)
{
}


GLVisualizerDots::~GLVisualizerDots()
{
}

int GLVisualizerDots::settings()
{
	return DISPLAY_VERTICES | DISPLAY_CAMERAS;
}

void GLVisualizerDots::settings(int flags)
{
    PARAM_UNUSED(flags);
}

void GLVisualizerDots::init()
{

}

void GLVisualizerDots::fakeBillboardBegin()
{
    float modelview[16];
    
    // save the current modelview matrix
    glPushMatrix();

    // get the current modelview matrix
    glGetFloatv(GL_MODELVIEW_MATRIX , modelview);

    // undo all rotations
    // beware all scaling is lost as well 
    modelview[ 0] = 1.0;
    modelview[ 5] = 1.0;
    modelview[10] = 1.0;
    modelview[1] = modelview[2] = modelview[4] = 0.0;
    modelview[6] = modelview[8] = modelview[9] = 0.0;

    // set the modelview with no rotations
    glLoadMatrixf(modelview);
}

void GLVisualizerDots::fakeBillboardEnd()
{
    // restore the previously 
    // stored modelview matrix
    glPopMatrix();
}

void GLVisualizerDots::createDotDL() {
    dotID = glGenLists(1);
    glNewList(dotID,GL_COMPILE);
    drawDot();
    glEndList();
}

void GLVisualizerDots::createCamDL() {
    camID = glGenLists(1);
    glNewList(camID,GL_COMPILE);
    drawCam();
    glEndList();
}

void GLVisualizerDots::drawDot() {
    float d = 1.0f / sqrt(2.0);
    glTranslatef(0.0f ,0.0f, 0.0f);
    glBegin(GL_TRIANGLE_FAN);
    glVertex3f( 0.0f,  0.0f, 0.0f);
    glVertex3f( 1.0f,  0.0f, 0.0f);
    glVertex3f(    d,     d, 0.0f);
    glVertex3f( 0.0f,  1.0f, 0.0f);
    glVertex3f(   -d,     d, 0.0f);
    glVertex3f(-1.0f,  0.0f, 0.0f);
    glVertex3f(   -d,    -d, 0.0f);
    glVertex3f( 0.0f, -1.0f, 0.0f);
    glVertex3f(    d,    -d, 0.0f);
    glVertex3f( 1.0f,  0.0f, 0.0f);
    glEnd();            
}

void GLVisualizerDots::drawCam() {
    glTranslatef(0.0f ,0.0f, 0.0f);
    glBegin(GL_TRIANGLE_FAN);
    glColor3f(1.0f,0.5f,0.0f);                        //orange
    glVertex3f(  0.0f,  0.0f, 0.0f);
    glColor3f(0.0f,0.0f,1.0f);                        //blue
    glVertex3f(1.0f, 1.0f, 2.5f);
    glVertex3f(-1.0f, 1.0f, 2.5f);
    glVertex3f(-1.0f, -1.0f, 2.5f);
    glVertex3f(1.0f, -1.0f, 2.5f);
    glVertex3f(1.0f, 1.0f, 2.5f);
    glEnd();
}

void GLVisualizerDots::focus()
{
    GLVisualizerMouse::focus();
    if(!inited)
    {
//         lowColor[0] = 0.5;
//         lowColor[1] = 0.8;
//         lowColor[2] = 0.5;
//         lowColor[3] = 1.0;
//         
//         highColor[0] = 0.8;
//         highColor[1] = 1.0;
//         highColor[2] = 0.8;
//         highColor[3] = 1.0;
//         
        lowColor[0] = 1.0;
        lowColor[1] = 1.0;
        lowColor[2] = 1.0;
        lowColor[3] = 1.0;
        
        highColor[0] = 0.5;
        highColor[1] = 0.5;
        highColor[2] = 0.5;
        highColor[3] = 0.5;
        
        dotScale = 1.0;
        //             
        lastHighCam = -1;
        lastHighPoint = -1;
        
        createDotDL();
        createCamDL();
        inited = true;
    }
}

void GLVisualizerDots::attach(NView*nv) 
{
	nv_ref = nv;
	if(!nv) { //reset
		vector< AVector3D* > pts;
		vector< Camera* > cams;
		drawPointsAndCams(pts, cams);
		return;
	}


    vector< AVector3D* > pts;
    for(NView::Point3DIterator it = nv->getPoints3DBegin(); it != nv->getPoints3DEnd(); ++it)
    {
        pts.push_back(*it);
    }
    
    vector< Camera* > cams;
    for(NView::CameraIterator it = nv->getCamerasBegin(); it != nv->getCamerasEnd(); ++it)
    {
        cams.push_back(*it);
    }
    
    drawPointsAndCams(pts, cams);
}

void GLVisualizerDots::select(const core::ViewBase* const, const bool) {/** UNUSED **/}

void GLVisualizerDots::drawPointsAndCams(vector<AVector3D* > &pts, vector<Camera* > &cam)
{
    points.resize(pts.size());
//     if(pts.size() < 1) return;
    
    double sumX = 0.0;
    double sumY = 0.0;
    double sumZ = 0.0;
    int nPoints = 0;
    
    for(uint i = 0; i < pts.size(); ++i)
    {
        if((*pts[i])(3) != 0)
        {
            points[i].x = (*pts[i])(0) / (*pts[i])(3);
            points[i].y = (*pts[i])(1) / (*pts[i])(3);
            points[i].z = (*pts[i])(2) / (*pts[i])(3);
            sumX += points[i].x;
            sumY += points[i].y;
            sumZ += points[i].z;
            points[i].setColor(lowColor);
            points[i].disabled = false;
            points[i].point = pts[i];
            ++nPoints;
//             printf("GLViz-Dots: 3D-Point: (%f, %f, %f)\n", points[i].x, points[i].y, points[i].z);
        }
        else
        {
            points[i].disabled = true;
        }
    }
    //put cameras in the appropriate list
    if(displayCams) cams.resize(cam.size());
    else cams.clear();
    for(uint i = 0; i < cams.size(); ++i)
    {
        Camera *c = cam[i];
        c->decompose();
        
        if(!isRightHand(c->getRotationMatrix()))
        {
            cams[i].disabled = true;
            continue;
        }
        
        cams[i].disabled = false;
        
        AVector3D loc = c->getLocation();
        
        cams[i].x = loc(0);
        cams[i].y = loc(1);
        cams[i].z = loc(2);
        AVector2D axis = anax.matToAngleAxis(c->getRotationMatrix(), cams[i].angle);
        cams[i].angle *= -180.0 / M_PI;
        cams[i].rx = axis(0);
        cams[i].ry = axis(1);
        cams[i].rz = axis(2);
        
        cams[i].setColor(lowColor);
        cams[i].cam = c;
        
		if(pts.size() < 1)
		{
			sumX += cams[i].x;
    	    sumY += cams[i].y;
	        sumZ += cams[i].z;
        	++nPoints;
//         	printf("Camera: (%f, %f, %f)\n", cams[i].x, cams[i].y, cams[i].z);
		}
    }
    
    sumX /= double(nPoints);
    sumY /= double(nPoints);
    sumZ /= double(nPoints);
    
    //setting lookat
    centroid[0] = camera[3] = sumX;
    centroid[1] = camera[4] = sumY;
    centroid[2] = camera[5] = sumZ;
    
    //calculating good distance from lookat
    double maxX = 0.0;
    double maxY = 0.0;
    double maxZ = 0.0;
    for(uint i = 0; i < points.size(); ++i)
    {
        double val;
        val = (points[i].x - sumX) * (points[i].x - sumX);
        if(val > maxX) maxX = val;
        val = (points[i].y - sumY) * (points[i].y - sumY);
        if(val > maxY) maxY = val;
        val = (points[i].z - sumZ) * (points[i].z - sumZ);
        if(val > maxZ) maxZ = val;
    }
	if(pts.size() < 1)
	{
		for(uint i = 0; i < cams.size(); ++i)
		{
			double val;
			val = (cams[i].x - sumX) * (cams[i].x - sumX);
			if(val > maxX) maxX = val;
			val = (cams[i].y - sumY) * (cams[i].y - sumY);
			if(val > maxY) maxY = val;
			val = (cams[i].z - sumZ) * (cams[i].z - sumZ);
			if(val > maxZ) maxZ = val;
		}
	}
    camera[0] = sumX;
    camera[1] = sumY;
    maxZ = sqrt(maxZ);
    maxY = sqrt(maxY);
    maxX = sqrt(maxX);
    //double dist = max(maxZ, maxX/(tan(0.5 * fov/180.0 * M_PI)));
    maxDist = 2.0 * max(max(maxX, maxY), maxZ);
    camera[2] = sumZ - maxDist - 1.0;
    
    //claculate clipping range
    cliprangeFar = 3.0 * maxDist;
    
    
//     printf("centroid: (%f, %f, %f)\n", sumX, sumY, sumZ);
//     printf("maxAb: (%f, %f, %f)\n", maxX, maxY, maxZ);
//     printf("clipping range (far): %f\n", cliprangeFar);
    
    //calculate good dot size via nearest neighbour
    //extremely dumb, so please improve it.
    //KDTree< AVector3D > nn(pts.begin(), pts.end(), 3);
    nearestDotDist = DBL_MAX;
	if(points.size() > 0)//to prevent uint underflow (watch the -1)
	{
    	for(uint i = 0; i < points.size() - 1; ++i)
    	{
        	for(uint j = i; j < points.size(); ++j)
        	{
            	double val = (points[i].x - points[j].x) * (points[i].x - points[j].x);
            	val += (points[i].y - points[j].y) * (points[i].y - points[j].y);
            	val += (points[i].z - points[j].z) * (points[i].z - points[j].z);
            	if(val > FLT_EPSILON && val < nearestDotDist) nearestDotDist = val;
        	}
    	}
	}
	
	if(points.size() > 0)//to prevent uint underflow (watch the -1)
	{
		for(uint i = 0; i < cams.size() - 1; ++i)
		{
			for(uint j = i; j < cams.size(); ++j)
			{
				double val = (cams[i].x - cams[j].x) * (cams[i].x - cams[j].x);
				val += (cams[i].y - cams[j].y) * (cams[i].y - cams[j].y);
				val += (cams[i].z - cams[j].z) * (cams[i].z - cams[j].z);
				if(val > FLT_EPSILON && val < nearestDotDist) nearestDotDist = val;
			}
		}
	}
	
    nearestDotDist = sqrt(nearestDotDist);
    dotScale = nearestDotDist * 0.25;
//     printf("nearest distance: %f\n", nearestDotDist);
//     printf("current scale: %f\n", dotScale);
    cliprangeNear = std::max(FLT_EPSILON, nearestDotDist * 0.1F);
//     printf("clipping range (near): %f\n", cliprangeNear);
    
    
    //after finding good values for clipRange etc. set them
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    setPerspective();    
    glMatrixMode(GL_MODELVIEW);
}

void GLVisualizerDots::setDotSizePercentage(float percent)
{
    if(percent < 0) percent = 0;
    if(percent > 100) percent  = 100;
    dotScale = nearestDotDist * 0.02 * percent;
    paint();
}

void GLVisualizerDots::paintScene()
{
    for(vector<glPoint>::iterator it = points.begin(); it != points.end(); ++it)
    {
        if((*it).disabled) continue;
        
        /*
        glPushMatrix();
        glColor4f((*it).r, (*it).g, (*it).b, (*it).a);
        glTranslatef((*it).x, (*it).y, (*it).z);
        glScalef(dotScale, dotScale,dotScale);
        glCallList(dotID);
        glPopMatrix();*/
        glPushMatrix();
        glColor4f((*it).r, (*it).g, (*it).b, (*it).a);
        glTranslatef((*it).x, (*it).y, (*it).z);
        fakeBillboardBegin();
        glScalef(dotScale, dotScale, 1.0);
        glCallList(dotID);
        fakeBillboardEnd();
        glPopMatrix();
    }    
    for(vector<glCamera>::iterator it = cams.begin(); it != cams.end(); ++it)
    {
        if((*it).disabled) continue;
        glPushMatrix();
        glColor4f((*it).r, (*it).g, (*it).b, (*it).a);
        glTranslatef((*it).x, (*it).y, (*it).z);
        glRotated((*it).angle, (*it).rx, (*it).ry, (*it).rz);
        glScalef(dotScale, dotScale,dotScale);
        //glCallList(camID);
		drawCam();
        glPopMatrix();
    }
//     glPushMatrix();
//     glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
//     glTranslatef(centroid[0], centroid[1], centroid[2]);
//     glScalef(dotScale, dotScale,dotScale);
//     glCallList(dotID);
//     glPopMatrix();
    
    glPushMatrix();
    glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
    glTranslatef(centroid[0], centroid[1], centroid[2]);
    fakeBillboardBegin();
    glScalef(dotScale, dotScale, 1.0);
    glCallList(dotID);
    fakeBillboardEnd();
    glPopMatrix();
    
    glPushMatrix();
    glColor4f(0.8f, 0.2f, 0.2f, 0.1f);
    glBegin(GL_POLYGON);
    glVertex3f(100, 0, 100);
    glVertex3f(100, 0, -100);
    glVertex3f(-100, 0, -100);
    glVertex3f(-100, 0 ,100);
    glEnd();
    glPopMatrix();
}

void GLVisualizerDots::select(int winx, int winy)
{
    GLint viewport[4];
    glSelectBuffer(pickBufferSize, pickBuffer);
    glGetIntegerv(GL_VIEWPORT, viewport);
    glRenderMode(GL_SELECT);
    glInitNames();
    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    GLdouble x = winx; //window coordinates; window coordinates in OpenGL have the origin at the bottom left corner of the viewport
    GLdouble y = viewport[3] - winy;
    gluPickMatrix(x, y, 5, 5, viewport);
    //gluPerspective(fov,aspectratio,1,cliprange);
    setPerspective();
    glMatrixMode(GL_MODELVIEW);
    
    //rendering
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glLoadIdentity();
    gluLookAt(  camera[0], camera[1], camera[2], camera[3], camera[4], camera[5], camera[6], camera[7], camera[8]);
    setModelView();
    
    GLuint i = 0;
    for(vector<glPoint>::iterator it = points.begin(); it != points.end(); ++it, ++i)
    {
        if((*it).disabled) continue;
        glPushName(i);
        
//         glPushMatrix();
//         glTranslatef((*it).x, (*it).y, (*it).z);
//         glScalef(dotScale, dotScale,dotScale);
//         glCallList(dotID);
//         glPopMatrix();
        
        glPushMatrix();
        glColor4f((*it).r, (*it).g, (*it).b, (*it).a);
        glTranslatef((*it).x, (*it).y, (*it).z);
        fakeBillboardBegin();
        glScalef(dotScale, dotScale, 1.0);
        glCallList(dotID);
        fakeBillboardEnd();
        glPopMatrix();
        
        glPopName();
    }
    i = camOff;
    for(vector<glCamera>::iterator it = cams.begin(); it != cams.end(); ++it, ++i)
    {
        if((*it).disabled) continue;
        glPushName(i);
        glPushMatrix();
        glTranslatef((*it).x, (*it).y, (*it).z);
        glRotated((*it).angle, (*it).rx, (*it).ry, (*it).rz);
        glScalef(dotScale, dotScale,dotScale);
        //glCallList(camID);
		drawCam();
        glPopMatrix();
        glPopName();
    }    
    //rendering done
    
    //leaving selection mode
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();
    glMatrixMode(GL_MODELVIEW);
    glFlush();
    
    uint hits = glRenderMode(GL_RENDER);
    
    //printf ("hits = %d of %d\n", hits, i);
    if(hits > 0)
    {
        processHits(hits);    
    }
}

//picking
void GLVisualizerDots::processHits (uint hits)
{
    hits = hits;
    //just the first entry is taken.
    int numNames = pickBuffer[0];
    if(numNames)
    {
        int name = pickBuffer[3];
        if(name >= camOff)
        {
            int idx = name - camOff;
            if (idx != lastHighCam)
            {
                highlightCam(idx);
                notifyListeners(cams[idx].cam);
            }
        }
        else
        {
            if (name != lastHighPoint)
            {
                highlightPoint(name);
                notifyListeners(points[name].point);
            }
        }
    }
}

void GLVisualizerDots::highlightPoint(int i)
{
    if(lastHighPoint >= 0)
    {
        points[lastHighPoint].setColor(lowColor);
    }
    lastHighPoint = i;
    points[lastHighPoint].setColor(highColor);    
}

void GLVisualizerDots::highlightCam(int i)
{
    if(lastHighCam >= 0)
    {
        cams[lastHighCam].setColor(lowColor);
    }
    lastHighCam = i;
    cams[lastHighCam].setColor(highColor);    
}


}}
