

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_OPENGLGLVISUALIZERDOTS_H
#define ADORE_OPENGLGLVISUALIZERDOTS_H

#include "glvisualizermouse.h"
#include <adrcore/parameterization/angleaxis.h>

namespace adore {
namespace visual {
    
    using namespace adore::core;

/**
 
*/
class GLVisualizerDots : public GLVisualizerMouse
{
    class glPoint
    {
        public:
            float x, y, z;
            float r, g, b, a;
            bool disabled;
        
            void setColor(float col[4])
            {
                r = col[0];
                g = col[1];
                b = col[2];
                a = col[3];
            }
        
            AVector3D *point;
    };

    class glCamera
    {
        public:
            float x, y, z;
            float rx, ry, rz;
            double angle;
            float r, g, b, a;
            bool disabled;
            
            void setColor(float col[4])
            {
                r = col[0];
                g = col[1];
                b = col[2];
                a = col[3];
            }
            
            Camera *cam;
    };

public:
    GLVisualizerDots();

    ~GLVisualizerDots();

    int settings();

    void settings(int flags);

    void init();
    
//     void resize(int w, int h);
    
    void paintScene();
    
    void attach(NView *nv);
	virtual core::NView* nview()const { return nv_ref; }

	void select(const core::ViewBase* const v, const bool reset);
    
    void setDisplayCams(bool theValue)
    {
        displayCams = theValue;
    }

    bool getDisplayCams() const
    {
        return displayCams;
    }

    void setDotSizePercentage(float percent);
    
    virtual void focus();
 
	 virtual void getTranslation(float (&)[3], uint) {}

protected:
    
    AngleAxis anax;
    bool inited;
    
    bool displayCams;
    
    float highColor[4];
    float lowColor[4];
    float dotScale;
    float nearestDotDist;

    int lastHighPoint;
    int lastHighCam;
    vector<glPoint> points;
    vector<glCamera> cams;
    const int camOff;
    
    GLint dotID;
    GLint camID;

    void createDotDL();
    void drawDot();
    void createCamDL();
    void drawCam();

    void processHits (uint hits);
    void highlightPoint(int i);
    void highlightCam(int i);
    void drawPointsAndCams(vector<AVector3D* > &pts, vector<Camera* > &cam);
    void select(int winx, int winy);
    
    void fakeBillboardBegin();
    void fakeBillboardEnd();

	NView* nv_ref;
};

}}

#endif
