

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_OPENGLGLVISUALIZERMOUSE_H
#define ADORE_OPENGLGLVISUALIZERMOUSE_H

#include <adrcore/types/nview.h>
#include <adrcore/types/camera.h>
#include "selectlistener.h"
#include "glvisualizer.h"
#include <GL/gl.h>
#include <GL/glu.h>

namespace adore {
namespace visual {

    using namespace std;
    using namespace adore::core;
    using namespace adore::core;

#define PARAM_UNUSED(x) ((x) = (x))
/**
 
*/
class GLVisualizerMouse:public GLVisualizer{
public:
    GLVisualizerMouse();

    virtual ~GLVisualizerMouse();
    
    /** Initialization, usually called after Visualizer::attach
        */
    virtual void init();

    /** Set maximum edge deviation
     *
     *  Faces with edges that deviate from the mean edge length by more
     *  than the given value are not rendered.
     *  @param value Deviation value
     */
    virtual void deviation(float value)
    {
        PARAM_UNUSED(value);
    }

    /** Must be called before painting sequence
     *
     *   This function sets up presentation parametres which
     *   may differ per Visualizer but stay constant each
     *   successive paint operation.
     */
    virtual void focus();

    /** Must be called when another Visualizer gains focus
        */
    virtual void unfocus(){;}

    /** Paint scene
        */
    virtual void paint();

    /** Resize viewport
     *   @param width Viewport witdth in pixels
     *   @param height Viewport height in pixels
    */
    virtual void resize(int width, int height);

    /** Determine the object at position (x,y) and notify the listeners
     *   @param x Coordinate x of the picking cursor
     *   @param y Coordinate y of the picking cursor
        */
    virtual void pick(int x, int y);

    /** Set up the projection attributes
     *   @param fov Field of view
        */
    virtual void perspective(float fov)
    {
        PARAM_UNUSED(fov);
    }

    /** Set up the camera position
     *   @param eye An eye location in world space
     *   @param look An object location in world space
     *   @param up A vector pointing up in world space
        */
    virtual void lookAt(const float eye[3], const float aim[3], const float up[3])
    {
        PARAM_UNUSED(eye);
        PARAM_UNUSED(aim);
        PARAM_UNUSED(up);
    }


    /** Roll the camera
     *   @param angle Roll angle in radians
        */
    virtual void roll(float angle)
    {
        PARAM_UNUSED(angle);
    }


    /** Yaw the camera
     *   @param angle Yaw angle in radians
        */
    virtual void yaw(float angle)
    {
        PARAM_UNUSED(angle);
    }


    /** Increase the pitch of the camera
     *   @param angle Pitch angle in radians
    */
    virtual void pitch(float angle)
    {
        PARAM_UNUSED(angle);
    }


    /** Slide the camera
     *   @param delta Distance to slide in camera space
    */
    virtual void slide(const float delta[3]);
    

    /** Turn the camera around the z-axis keeping the up vector collinear to it
    */
    virtual void turn(float angle)
    {
        PARAM_UNUSED(angle);
    }

    virtual void tilt(float angle, float gap)
    {
        PARAM_UNUSED(angle);
        PARAM_UNUSED(gap);
    }

    /** Translate the scene
     *   @param delta Distance to translate in world space
     *   @param absolute Absolute or relative translation
    */
    virtual void translate(const float delta[3], bool absolute)
    {
        PARAM_UNUSED(delta);
        PARAM_UNUSED(absolute);
    }


    /** Rotate the scene
     *   @param axis Rotation axis in world space
     *   @param angle rotation angle in radians
     *   @param absolute Absolute or relative rotation
    */
    virtual void rotate(const float axis[3], float angle, bool absolute);
	
	virtual void turnTable(float angle, bool absolute);
	
	virtual void setTurnTableSpine ( const float axis[3] ){;}

    /** Scale the scene
     *   @param scale Scaling factor for 3 axes in object space
     *   @param absolute Absolute or relative scaling
    */
    virtual void scale(const float factor[3], bool absolute)
    {
        PARAM_UNUSED(factor);
        PARAM_UNUSED(absolute);
    }

    /** Reset all scene transformations to defaut values
    */
    virtual void identity()
    {
    }

    /** Set background color
     *  @param color Background color in RGBA form
    */
    virtual void bgcolor(const unsigned char color[4])
    {
        PARAM_UNUSED(color);
    }

    /** Set pen size and color
     *  @param color Color of points and lines in RGBA form
     *  @param size Point size and line thickness value
    */
    virtual void pen(const unsigned char color[4], float size)
    {
        PARAM_UNUSED(color);
        PARAM_UNUSED(size);
    }

    void setDistancePercentage(float percent);
    
protected:
    
    virtual void select(int x, int y) = 0;
    
    virtual void paintScene() = 0;
    
    void setPerspective();
    void setModelView();
    
    float fov;
    float cliprangeNear;
    float cliprangeFar;
    float aspectratio;
    float maxDist;

    float camera[9];
    int xAngle;
    int yAngle;
    float centroid[3];

    GLuint *pickBuffer;
    GLsizei pickBufferSize;
};

}}

#endif
