

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_OPENGLIMAGEDISPLAYCLIENT_H
#define ADORE_OPENGLIMAGEDISPLAYCLIENT_H

#include <adrcore/imgprocessing/bilinearinterpolator.h>
#include <adrcore/imgprocessing/imageplane.h>
#include <adrcore/imgprocessing/channel.h>
#include <adrcore/imgprocessing/channelfunctions.h>
#include "displayclient.h"

namespace adore {
namespace visual {

    using namespace adore::core;
    
/**
	 
*/
    template <class imgType>
            class ImageDisplayClient : public DisplayClient
    {
        public:
        
        ImageDisplayClient(Channel<imgType> *channel1, Channel<imgType> *channel2, Channel<imgType> *channel3):
            chan1(channel1),
        chan2(channel2),
        chan3(channel3),
        width(chan1->getWidth()),
        height(chan1->getHeight())
        {
            imgType min, max, *locMin, *locMax;
            adore::core::minMaxLoc(*chan1, min, max, locMin, locMax);
            mini = min;
            maxi =  max;
            adore::core::minMaxLoc(*chan2, min, max, locMin, locMax);
            mini = adore::core::min(mini, min);
            maxi = adore::core::max(maxi, max);
            adore::core::minMaxLoc(*chan3, min, max, locMin, locMax);
            mini = adore::core::min(mini, min);
            maxi = adore::core::max(maxi, max);
        }
        
        ~ImageDisplayClient()
        {
        }
        
        private:
        
            void drawSceneImpl()
            {
                int winWidth, winHeight;
                int vPort[4];
                glGetIntegerv(GL_VIEWPORT, vPort);
                winWidth = vPort[2];
                winHeight = vPort[3];
                ImagePlane<uint8_t> glImage(winWidth, winHeight, 3);
            
                if(mini == maxi)
                {
                    glImage.fill(0);
                }
                else
                {
                    double imgAspect;
                    imgAspect = (double) width / (double) height;
                    int resWidth = (int) ((double) winHeight * imgAspect);
                    int resHeight = winHeight;
                    if(resWidth > winWidth)
                    {
                        resWidth = winWidth;
                        resHeight = (int) (double(winWidth) / imgAspect);
                    }
                    int hBorder = (winWidth - resWidth) >> 1;
                    int vBorder = (winHeight - resHeight) >> 1;
                
                    glImage.fill(255);
                    glImage.setROI( hBorder, vBorder, resWidth, resHeight);
                    BilinearInterpolator<imgType> billy;
                
                    const int s = 2; //leave a border because of interpolation
                    for(int y = s; y < resHeight - s; ++y)
                    {
                        int x = s;
                        uint8_t *ptr = glImage.getPointer(x + glImage.getROI().getX(), y + glImage.getROI().getY());
                        for(; x < resWidth - s; ++x)
                        {
//                         float srcX = float(width * x) / float(resWidth);
//                         float srcY = float(height) - float(height * y) / float(resHeight);
//                         printf("(%d, %d)->(%f, %f)\n", x, y, srcX, srcY); pressKey();
                        /*billy.init(srcX, srcY);
                            *ptr = (billy.valueOf(*chan1) - mini) * 255 / (maxi - mini);
                            ++ptr;
                            *ptr = (billy.valueOf(*chan2) - mini) * 255 / (maxi - mini);
                            ++ptr;
                            *ptr = (billy.valueOf(*chan3) - mini) * 255 / (maxi - mini);
                            ++ptr;*/
                        
                            int srcX = int(float(width * x) / float(resWidth));
                            int srcY = int(float(height) - float(height * y) / float(resHeight));
                            *ptr = (*chan3->getPointer(srcX, srcY) - mini) * 255 / (maxi - mini);
                            ++ptr;
                            *ptr = (*chan2->getPointer(srcX, srcY) - mini) * 255 / (maxi - mini);
                            ++ptr;
                            *ptr = (*chan1->getPointer(srcX, srcY) - mini) * 255 / (maxi - mini);
                            ++ptr;
                        }
                    }
                    glImage.resetROI();
                }
            
                glPixelStorei(GL_PACK_ALIGNMENT, 1);
                glPixelStorei(GL_PACK_ROW_LENGTH, glImage.getWStep());
//             printf("width: %d, wStep: %d, height: %d\n", glImage.getWidth(), glImage.getWStep(), glImage.getHeight());
                glDrawPixels(glImage.getWidth(), glImage.getHeight(), GL_RGB, GL_UNSIGNED_BYTE, getImgArrayData<char>(glImage.getImgArray()));
            
            }
        
        private:
        
            Channel<imgType> *chan1;
            Channel<imgType> *chan2;
            Channel<imgType> *chan3;
            int width, height;
            imgType mini, maxi;
    };

}}

#endif
