

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef ADORE_OPENGLRENDERNVIEW_H
#define ADORE_OPENGLRENDERNVIEW_H

#include "renderitem.h"
#include <adrcore/types/aarray.h>
#include <adrcore/types/camera.h>
#include <adrcore/types/nview.h>

namespace adore {
namespace visual {

        using namespace adore::core;

/**
	 
*/
class RenderNView : public RenderItem
{
    class glPoint
    {
        public:
            float x, y, z;
            float r, g, b, a;
            bool disabled;
        
            void setColor(float col[4])
            {
                r = col[0];
                g = col[1];
                b = col[2];
                a = col[3];
            }
        
            AVector3D *point;
    };

    class glCamera
    {
        public:
            float x, y, z;
            float rx, ry, rz;
            double angle;
            float r, g, b, a;
            bool disabled;
            
            void setColor(float col[4])
            {
                r = col[0];
                g = col[1];
                b = col[2];
                a = col[3];
            }
            
            Camera *cam;
    };

    public:
        RenderNView();
    
        ~RenderNView();

        void attach(NView *nv);
        
        void render();
        
        void renderSelect(GLuint lastId);
        
        void setDotScale(float scale)
        {
            dotScale = scale;
        }
        
        void setDisplayCams(bool d)
        {
            displayCams = d;
        }
        
        void setDisplayDots(bool d)
        {
            displayDots = d;
        }
        
    private:
            
        void createDotDL();
        void drawDot();
        void createCamDL();
        void drawCam();
        void highlightPoint(int i);
    
        void highlightCam(int i);
        
        void drawCams(NView::CameraIterator it, uint size);
        void drawPoints(NView::Point3DIterator it, uint size);
            
        void fakeBillboardBegin();
        void fakeBillboardEnd();

        bool displayCams;
        bool displayDots;
        
        float highColor[4];
        float lowColor[4];
        float dotScale;
        
        int lastHighCam;
        int lastHighDot;

        GLint dotID;
        GLint camID;
        
        vector<glPoint> points;
        vector<glCamera> cams;

        AVector centroidDots;
        AVector centroidCams;
};

}}

#endif
