

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#include "renderviewgraph.h"

namespace adore {
namespace visual {

RenderViewgraph::RenderViewgraph()
    : RenderItem(), displayLines(true)
{
}


RenderViewgraph::~RenderViewgraph()
{
}

void RenderViewgraph::render()
{
    if(displayLines)
    {
        glBegin(GL_LINES);
        for(vector<glLink>::iterator it = links.begin(); it != links.end(); ++it)
        {
            glPushMatrix();
            glColor4f((*it).r, (*it).g, (*it).b, (*it).a);
            glVertex3f((*it).x1, (*it).y1, (*it).z1);
            glVertex3f((*it).x2, (*it).y2, (*it).z2);
            glPopMatrix();
        }
        glEnd();
    }
}
        
void RenderViewgraph::renderSelect(GLuint lastId)
{
    ADORE_UNUSED(lastId);
}

void RenderViewgraph::attach(ViewGraph *vg)
{
    graph_traits<ViewGraph>::vertex_iterator vi, vi_end;
    for (tie(vi, vi_end) = vertices(*vg); vi != vi_end; ++vi)
    {
        Camera *cam1 = (*vg)[*vi].cam;
        cam1->decompose();
        AVector loc1 = cam1->getLocation();
        assert(loc1(3) == 1.0);
        
        graph_traits<ViewGraph>::adjacency_iterator avi, avi_end;
        for (tie(avi, avi_end) = adjacent_vertices(*vi, *vg); avi != avi_end; ++avi)
        {
            Camera *cam2 = (*vg)[*avi].cam;
            cam2->decompose();
            AVector loc2 = cam2->getLocation();
            assert(loc2(3) == 1.0);
            
            glLink link;
            link.r = 1.0;
            link.g = 0.5;
            link.b = 0.5;
            link.a = 1.0;
            
            link.x1 = loc1(0);
            link.y1 = loc1(1);
            link.z1 = loc1(2);
            
            link.x1 = loc2(0);
            link.y2 = loc2(1);
            link.z2 = loc2(2);
            
            links.push_back(link);
        }
    }
}

}}
