

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming
/// Author: Sergey Cheremukhin


#include <iostream>
#include <string>
#include <vector>
#include <limits>
#include <cassert>

#include <adrcore/persistence/persist.h>
#include <adrcore/persistence/persist_boost.h>

#include <adrcore/types/nviewmap.h>
#include <adrcore/viewmatch/nviewdependencygraph.h>
#include <adrcore/parameterization/quaternion.h>

using namespace std;
using namespace persist;
using namespace adore::core;

int getInt( int min, int max, const string& q )
{
	int retval;
	bool ok = false;
	do
	{
		cout << q;
		cin >> retval;
		if ( !cin.good() )
		{
			cin.clear();
			cin.ignore( numeric_limits<streamsize>::max(), '\n' );
		}
		else
		{
			if ( min <= retval && max >= retval )
				ok = true;
		}
	}
	while ( !ok );
	return retval;
}

uint menu( const vector<string>& m, const string& q )
{
	size_t cnt = m.size();
	assert( cnt );
	for ( size_t i = 0; i < cnt; ++i )
		cout << i << ": " << m[i] << endl;
	return ( uint )getInt( 0, cnt - 1, q );
}

void extract( NViewMap& nviewmap )
{
	int numNviews = nviewmap.getNumEntries();
	cout << "numEntries returned: " << numNviews << "\n";

	vector<NView*> nviews;
	vector<string> m;

	NViewMap::EntryIterator nvit = nviewmap.getEntriesBegin();
	NViewMap::EntryIterator nvend = nviewmap.getEntriesEnd();
	for ( int i = 0; nvit != nvend; ++nvit, ++i )
	{
		NView* nv = *nvit;
		nviews.push_back( nv );

		stringstream helper;
		string s_helper;
		helper << nv->getNumViews();
		helper >> s_helper;
		m.push_back( string() + s_helper + " Views" );
	}

	int num = menu( m, "select NView(number): " );
	m.clear();
	m.push_back( "extract NView" );
	m.push_back( "extract point cloud" );
	m.push_back( "extract in sur-format" );
	int choice = menu( m, "coose action: " );

	string path_to_save;
	cout << "enter path: ";
	cin >> path_to_save;

	NView *nv = nviews[num];
	switch ( choice )
	{
		case 0:
		{
			NViewMap map;
			map.setNViewResetMode( KEEP_NVIEW );
			map.setViewResetMode( KEEP_VIEW );
			map.setNView( nv->getViewsBegin(), nv->getViewsEnd(), nv );
			DBGOUT( DBGNORMAL, "Saving NView Wrapped in NViewMap\n" );
			Persist<PersistBoost> database_save( path_to_save );
			database_save.saveNViewMap( map, "_dummy" );
		}
		break;
		case 1:
		{
			ofstream of( path_to_save.c_str() );
			NView::Point3DIterator pit = nv->getPoints3DBegin();
			NView::Point3DIterator pend = nv->getPoints3DEnd();
			for ( ; pit != pend; ++pit )
			{
				AVector* p3d = *pit;
				double w = ( *p3d )( 3 );
				if ( p3d && fabs( w ) > FLT_EPSILON )
				{
					double x = ( *p3d )( 0 ) / w;
					double y = ( *p3d )( 1 ) / w;
					double z = ( *p3d )( 2 ) / w;
					of << x << " " << y << " " << z << endl;
				}
			}
			of.flush();
		}
		break;
		case 2:
		{
			ofstream of( path_to_save.c_str() );
			size_t num_pts = nv->getNumPoints3D();
			of << num_pts << endl;
			vector<double> coords; coords.reserve( num_pts * 3 );
			double l_b[3] = {0, 0, 0}, r_t[3] = {0, 0, 0};
			bool initialized = false;

			NView::Point3DIterator pit = nv->getPoints3DBegin();
			NView::Point3DIterator pend = nv->getPoints3DEnd();
			for ( ; pit != pend; ++pit )
			{
				AVector* p3d = *pit;
				double w = ( *p3d )( 3 );
				if ( p3d && fabs( w ) > FLT_EPSILON )
				{
					double x = ( *p3d )( 0 ) / w; coords.push_back( x );
					double y = ( *p3d )( 1 ) / w; coords.push_back( y );
					double z = ( *p3d )( 2 ) / w; coords.push_back( z );
					if ( !initialized )
					{
						l_b[0] = r_t[0] = x;
						l_b[1] = r_t[1] = y;
						l_b[2] = r_t[2] = z;
						initialized = true;
					}
					else
					{
						if ( x < l_b[0] ) l_b[0] = x;
						else if ( x > r_t[0] ) r_t[0] = x;
						if ( y < l_b[1] ) l_b[1] = y;
						else if ( y > r_t[1] ) r_t[1] = y;
						if ( z < l_b[2] ) l_b[2] = z;
						else if ( z > r_t[2] ) r_t[2] = z;
					}
				}
				else --num_pts;
			}
			if ( num_pts )
			{
				of << l_b[0] << " " << l_b[1] << " " << l_b[2] << " "
				<< r_t[0] << " " << r_t[1] << " " << r_t[2] << endl;
			}
			for ( size_t i = 0; i < num_pts; ++i )
			{
				size_t idx = i * 3;
				of << coords[idx] - l_b[0] << " " << coords[idx + 1] - l_b[1]
				<< " "<< coords[idx + 2] - l_b[2] << endl;
			}
			of.flush();

			size_t num_cams = nv->getNumCameras();
			of << num_cams << endl;
			NView::ViewIterator vit = nv->getViewsBegin();
			NView::ViewIterator vend = nv->getViewsEnd();
			for ( ; vit != vend; ++vit )
			{
				ViewBase* v = *vit;
				Camera* cam = nv->getCamera( v, false );//don't create
				if ( cam )
				{
					cam->decompose();
					const AVector3D& pos( cam->getLocation() );
					double inv_w = 1.0 / pos( 3 );
					of << pos( 0 ) * inv_w << " " << pos( 1 ) * inv_w
					<< " " << pos( 2 ) * inv_w << " ";
					const AMatrix& rotmat( cam->getRotationMatrix() );
					Quaternion quat;
					quat.fromRotationMatrix( rotmat );
					of << quat( 0 ) << " " << quat( 1 ) << " " << quat( 2 ) << " " << quat( 3 )<< " ";
					of << v->getImagePath().c_str() << endl;
				}
			}
		}
		break;
		default:
			cout << "unknown action - quitting\n";
	}
}

void patchPaths( NViewMap& nviewmap, const string& save_path,
                 const string& obj_name )
{

	NViewMap::ViewIterator vit = nviewmap.getAllViewsBegin();
	NViewMap::ViewIterator vend = nviewmap.getAllViewsEnd();
	for ( ; vit != vend; ++vit )
	{
		ViewBase* view = ( *vit );
		cout << view->getImagePath() << endl;
	}
	string path_to_save;
	while ( path_to_save.empty() )
	{
		cout << "enter new path: ";
		cin >> path_to_save;
	}
	size_t len = path_to_save.length();
	const char* slash = "/";
	if ( *slash != path_to_save[len - 1] )
		path_to_save += slash;

	vit = nviewmap.getAllViewsBegin();
	for ( ; vit != vend; ++vit )
	{
		ViewBase* view = ( *vit );
		string new_path = view->getImagePath();
		size_t pos = new_path.find_last_of( slash );
		new_path = path_to_save + new_path.substr( pos + 1 );
		view->setImagePath( new_path );
	}

	cout << "saving..." << endl;
	Persist<PersistBoost> database_save( save_path );
	database_save.saveNViewMap( nviewmap, obj_name );
}

int main( int argc, char *argv[] )
{
	if ( argc < 2 )
	{
		cout << "I need a path.\n";
		return -1;
	}

	NViewMap nviewmap;

	string path( argv[1] );
	Persist<PersistBoost> database( path );
	vector<string> names( database.objectNames() );
	string obj_name( default_obj_name );

	vector<string> m;
	if ( !names.empty() )
	{
		uint cnt = names.size();
		if ( 1 < cnt )
		{
			int num = menu( names, "select object(number): " );
			obj_name = names[num];
		}
		else obj_name = names[0];
	}
	cout << "selected object: " << obj_name.c_str() << "\n";

	bool ok = database.loadNViewMap( nviewmap, obj_name );
	if ( !ok )
	{
		cerr << "load nviemap failed!\n";
		return -1;
	}

	cout << "What do You want to do?" << endl;
	m.clear();
	m.push_back( "Change paths to images" );
	m.push_back( "Extract something" );
	uint result = menu( m, "Select one, please: " );
	if ( !result ) patchPaths( nviewmap, path, obj_name );
	else extract( nviewmap );

	return EXIT_SUCCESS;
}
