

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Klaus Haeming


#ifndef CAMERATOURIST_H
#define CAMERATOURIST_H

#include <vector>
#include <cstdio>
#include <string>
#include <map>
#include <adrvisual/glcamera.h>
#include <adrvisual/gltransform.h>
#include <cstdio>
#include <string>

namespace adore
{
	namespace visual
	{
		class GLRenderer;
	}
}

class MovementAccu
{
	public:
		//speedFactor relates the integer speed number to the actual angle
		static const float rotFactor;

		//slidefactor relates the integer speed number to the actual step
		static const float slideFactor;

		float rotXAccu;
		float rotYAccu;
		float slideAccu;

		MovementAccu():rotXAccu( 0 ), rotYAccu( 0 ), slideAccu( 0 ) {}

		void reset() {rotXAccu = rotYAccu = slideAccu = 0.0f;}
		void rotX( float amount, bool report = false )
		{
			rotXAccu += amount;
			if ( report ) std::printf( "X rotation: %d\n", getRotXSteps() );
		}
		void rotY( float amount, bool report = false )
		{
			rotYAccu += amount;
			if ( report ) std::printf( "Y rotation: %d\n", getRotYSteps() );
		}
		void slide( float amount, bool report = false )
		{
			slideAccu += amount;
			if ( report ) std::printf( "cam slide : %d\n", getSlideSteps() );
		}

		int getRotXSteps() {return int( rotXAccu / rotFactor );}
		int getRotYSteps() {return int( rotYAccu / rotFactor );}
		int getSlideSteps() {return int( slideAccu / slideFactor );}
};

/**
	 
*/
class CameraTourist
{
	private:
		class CameraAction
		{
			public:
				int what;
				int numRepeats;
				CameraAction( int w, int n ):what( w ), numRepeats( n ) {;}
		};

		typedef std::vector<CameraAction> Script;
		typedef std::map<std::string, int> StringId;
		typedef StringId::iterator StringIdIter;
		Script script;
		StringId actionIds;
		adore::visual::GLCamera startCam;
		adore::visual::GLTransform worldTrafo;
		adore::visual::Vector3D turnSpine;

		int curStep;
		int subStep;
		int intSpeed;
		bool doRepeat;
		bool doReturn;//begin every loop with startCam?
		int dotTicks;
		int triTicks;

	public:
		CameraTourist();

		~CameraTourist();

		void reset();

		void readScript( const char *file );

		bool step ( adore::visual::GLRenderer *visu );

		void setCamera( const adore::visual::GLCamera &cam ) {startCam = cam;}

		void printWorldTrafo() {printWorldTrafo( worldTrafo );}

		void printWorldTrafo( const adore::visual::GLTransform trafo );

		void printTurnTableSpine( adore::visual::Vector3D spine );

		void printCamera() {printCamera( startCam );}

		void printCamera( const adore::visual::GLCamera cam );

};

#endif
