

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include <QComboBox>
#include <QByteArray>

#include "combodelegate.h"

ComboDelegate::ComboDelegate( QObject * parent )
		: QItemDelegate( parent )
{
}

QWidget * ComboDelegate::createEditor( QWidget * parent,
                                       const QStyleOptionViewItem & option, const QModelIndex & index ) const
{
	QComboBox* combo = new QComboBox( parent );
	QVariant var( index.data( data_idx ) );
	QByteArray bytes( var.toByteArray() );
	QDataStream stream( bytes );
	QList<QString> strings; stream >> strings;
	int cnt = strings.size();
	for ( int i = 0; i < cnt; i++ )
		combo->addItem( strings[i] );
	connect( combo, SIGNAL( activated( int ) ), this, SLOT( slotCommitData( int ) ) );
	return combo;
}

void ComboDelegate::updateEditorGeometry( QWidget * editor,
                const QStyleOptionViewItem & option, const QModelIndex & index ) const
{
	editor->setGeometry( option.rect );
}

void ComboDelegate::setEditorData( QWidget * editor, const QModelIndex & index ) const
{
	QString value = index.model()->data( index, text_idx ).toString();
	QComboBox* combo = static_cast<QComboBox*>( editor );
	int idx = combo->findText( value );
	if ( idx < 0 ) return;
	combo->setCurrentIndex( idx );
}

void ComboDelegate::setModelData( QWidget * editor, QAbstractItemModel * model,
                                  const QModelIndex & index ) const
{
	QComboBox* combo = static_cast<QComboBox*>( editor );
	int idx = combo->currentIndex();
	int lastidx = model->data( index, idx_idx ).toInt();
	if ( idx == lastidx ) return;

	QString value = combo->currentText();
	model->setData( index, value, text_idx );
	model->setData( index, idx, idx_idx );
}

void ComboDelegate::slotCommitData( int idx )
{
	emit sigCommitData( idx );
}
