

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef PARSER_H
#define PARSER_H

#include <string>
#include <vector>

#include "configconstants.h"
typedef unsigned int uint;

namespace batch
{

	struct _enum_
	{
		std::string name;
		std::vector< std::string> enums;
	};

	struct _opt_
	{
		std::string type;
		std::string name;
		std::vector < std::string > values;
	};

	struct _helper_
	{
		std::string name;
		std::vector< _opt_ > opts;
		std::string getType( const std::string& opt_name )const
		{
			std::string retval;
			int cnt = opts.size();
			for ( int i = 0; i < cnt; ++i )
			{
				if ( opt_name == opts[i].name )
					return opts[i].type;
			}
			return retval;
		}
		std::string getValue( std::string opt_name, uint idx = 0 )const
		{
			std::string retval;
			int cnt = opts.size();
			for ( int i = 0; i < cnt; ++i )
			{
				if ( opt_name == opts[i].name && idx < opts[i].values.size() )
				{
					return opts[i].values[idx];
				}
			}
			return retval;
		}
	};

	struct _alg_ : public _helper_
	{

	};

	struct _cmd_ : public _helper_
	{
		std::vector< _alg_ > algs;
	};

	struct _cfg_
	{
		std::vector< _cmd_ > cmds;
		std::vector< _enum_ > types;
	};

	class Parser
	{
		public:
			Parser() { }

			bool parseOptions( const std::string& optstr, std::vector< batch::_opt_>& options );
			bool parseConfig( const std::string& cfgstr, batch::_cfg_& cfg );
			void makeConform( const std::string& cfgstr, batch::_cfg_& cfg );

			static std::vector< std::string > getCommands();
			static bool hasCmd( const std::string& cmd );
			static std::vector< std::string > getAlgorithms( const std::string& cmd );
			static int algIndex( const std::string& cmd, const std::string& alg );
			static std::string getOptions( const std::string& cmd );
			static std::string getOptions( const std::string& cmd, const std::string& alg );
			static std::vector< std::string > getEnums( const std::string& name );
		private:
			static Parser* static_instance;
			static void init_instance();
			bool parseAll();

			void parseAllHit( const char* what, char const* str, char const* end );
			_cfg_ config;
			std::string next, switch_helper;
			_cmd_ cur_cmd;
			_alg_ cur_alg;
			_opt_ cur_opt;
			void cmd_begin();
			void alg_begin();
			void opt_begin();
			void _type();
			void _name();
			void _value();
			void opt_end();
			void alg_end();
			void cmd_end();

			_enum_ cur_enum;
			void enum_begin();
			void enum_val_begin();
			void enum_val_end();
			void enum_end();

			void parseOptionsHit( const char* what, char const* str, char const* end );
			std::vector< batch::_opt_> cur_options;

			static int cmdIndex( const std::string& cmd );
	};

} // namespace batch

#endif
