

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include "dirmodel.h"

DirModel::DirModel( Mode mode ) :
		workMode(mode)
{
	setSourceModel( &model );
	if(workMode == Files) {
		model.setFilter( QDir::Dirs | QDir::NoDotAndDotDot | QDir::Files | QDir::Readable );
		model.setSorting( QDir::Name | QDir::DirsFirst );
	} else {
		model.setFilter( QDir::Dirs | QDir::NoDotAndDotDot );
		model.setSorting( QDir::Name );
	}
}

bool DirModel::filterAcceptsRow( int source_row, const QModelIndex& source_parent ) const
{
	QModelIndex idx( model.index( source_row, 0, source_parent ) );
	if ( model.isDir( idx ) ) return true;
	QString path = idx.data().toString();

	int cnt = _filters.size();
	if ( !cnt ) return true;
	for ( int i = 0; i < cnt; ++i )
	{
		QRegExp rx( _filters[i] );
		rx.setPatternSyntax( QRegExp::Wildcard );
		if ( rx.exactMatch( path ) )
			return true;
	}
	return false;
}

QVariant DirModel::data ( const QModelIndex & proxyIndex, int role ) const
{
	if(role != Qt::ToolTipRole)
		return QSortFilterProxyModel::data(proxyIndex, role);
	QModelIndex idx(mapToSource(proxyIndex));
	if ( !model.isDir(idx) )
		return QString("");
	QDir dir(model.fileInfo(idx).absoluteFilePath());
	QStringList filtered( dir.entryList( _filters, QDir::Files | QDir::NoSymLinks | QDir::Readable ) );
	return QString::number(filtered.size());
}

