

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include "imagegl.h"

#include <QtGui/QImage>
#include <QtOpenGL/QGLWidget>

ImageGL::ImageGL(const QString& imgpath) :
		m_img_path(imgpath),
		m_texid(0)
{
}

void ImageGL::initialize(QGLWidget* binder)
{
	if(m_texid != 0) return;
	QImage img(m_img_path);
	if(img.isNull()) return;

	m_texid = binder->bindTexture(img);
	if(m_texid) {
		size = img.size();
	}
}

void ImageGL::cleanup()
{
	if(m_texid) {
		glDeleteTextures(1, &m_texid);
		m_texid = 0;
	}
}

void ImageGL::paint() const
{
	if(m_texid == 0)
		return;

	glTranslated(position.x(), position.y(), 0.0);

	glEnable( GL_TEXTURE_2D );
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glBindTexture(GL_TEXTURE_2D, m_texid);

	glBegin( GL_QUADS );
	glTexCoord2f( 0.0, 0.0 ); glVertex3f( 0, 0, 0.0 );
	glTexCoord2f( 1.0, 0.0 ); glVertex3f( width(), 0, 0.0 );
	glTexCoord2f( 1.0, 1.0 ); glVertex3f( width(), height(), 0.0 );
	glTexCoord2f( 0.0, 1.0 ); glVertex3f( 0, height(), 0.0 );
	glEnd();

	glBindTexture(GL_TEXTURE_2D, 0);

	glTranslated(-position.x(), -position.y(), 0.0);
}
