

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef PIXMAPPLANE_H
#define PIXMAPPLANE_H

#include <QtCore/QObject>

#include "item2d.h"

#include "constants.h"

#include "GL/gl.h"

#include <vector>

class QColor;
class QMouseEvent;

class PixmapPlane : public QObject, public Item2d
{
		Q_OBJECT

	enum { MaxTextureExtent = 1024 };

	public:
		PixmapPlane( ViewT* view );
		~PixmapPlane();

		ViewT* getView() const { return view; }

		void paint() const;

		void setSelected( bool on );
		bool isSelected() const { return selected; }
		void setFocused( bool on );
		bool isFocused() const { return focused; }
		void setHighlighted( bool on );
		bool isHighlighted() const { return highlighted; }

		void mousePressEvent( const QPointF& p, Qt::MouseButton button );

		void setEpiline1( const adore::core::AVector& line );
		void setEpiline2( const adore::core::AVector& line );
		void removeEpilines();

	private:
		enum { frameW = 8 };

		ViewT* view;
		GLuint texId, iconId;
		std::vector<QPointF> epilinePoints;

		void loadView();
		void generateFocusIcon();

		bool selected, focused, highlighted;

		void drawSelectionMark(const QRectF& imgRect) const;
		void drawHighlightMark(const QRectF& imgRect) const;
		void drawFocusMark(const QRectF& imgRect) const;
		QRectF calculateFocusMark(const QRectF& imgRect) const;
		void drawEpilines() const;

		void drawFrame( const QRectF& rect, const QColor& color, double width ) const;
		void drawRect( double x, double y, double w, double h ) const;
		void drawLineGl( const QPointF& p0, const QPointF& p1, double thickness ) const;

	signals:
		void signalSelected( const ViewT* const view, bool on );
		void signalFocused( const ViewT* const view );
};

#endif // PIXMAPPLANE_H
