

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#ifndef SINGLETONHOLDER_H
#define SINGLETONHOLDER_H

#include <boost/thread/mutex.hpp>

template <typename T> class ClassLevelMutex {
	typedef boost::mutex Mutex;
	static Mutex classLevelMutex;
public:
	static Mutex& getMutex() { return classLevelMutex; }
};
template <typename T> typename ClassLevelMutex<T>::Mutex ClassLevelMutex<T>::classLevelMutex;


template <typename T> class SingletonHolder {

	typedef ClassLevelMutex<T> MutexHolder;
	static MutexHolder mutexHolder;

	typedef T* InstancePtr;
	static InstancePtr instancePtr;

	static void createInstance();

public:
	static T& instance() {
		if(NULL == instancePtr) {
			createInstance();
		}
		return *instancePtr;
	}
	static const T& const_instance() {
		return instance();
	}
};

template <typename T> typename SingletonHolder<T>::InstancePtr SingletonHolder<T>::instancePtr = NULL;
template <typename T> typename SingletonHolder<T>::MutexHolder SingletonHolder<T>::mutexHolder;

template <typename T> void SingletonHolder<T>::createInstance() {
	boost::mutex::scoped_lock lock(mutexHolder.getMutex());
	if(NULL == instancePtr) {
		instancePtr = new T;
	}
}

#endif // SINGLETOTONHOLDER_H
