

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include "slidergl.h"
#include "qglhelper.h"

#include <cmath>

SliderGL::SliderGL(Type t, int min, int max) :
		m_type(t),
		m_min(min),
		m_max(max),
		m_pos(min)
{
}

void SliderGL::setValue(int val)
{
	val = std::max(val, m_min);
	val = std::min(m_max, val);
	m_pos = val;
}

void SliderGL::initialize(QGLWidget* )
{
	if(m_type == Horizontal)
		size.setHeight(Thickness);
	else
		size.setWidth(Thickness);
}

void SliderGL::paint() const
{
	glTranslated(position.x(), position.y(), 0.0);
	
	glEnable( GL_TEXTURE_2D );
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	double thickness(Thickness);
	double x1 = m_type == Horizontal ? 0 : thickness * 0.5;
	double y1 = m_type == Horizontal ? thickness * 0.5 : 0;
	double x2 = m_type == Horizontal ? width() : thickness * 0.5;
	double y2 = m_type == Horizontal ? thickness * 0.5 : height();
	
	glColor4ub(0, 127, 0, 128);
	QGLHelper::aaGlLineWidth(thickness);
	QGLHelper::aaGlBeginLines();
	QGLHelper::aaGlVertex2(x1, y1);
	QGLHelper::aaGlVertex2(x2, y2);
	QGLHelper::aaGlEndLines();

	double sliderPos = double(m_pos) / (m_max - m_min);
	if(m_type == Horizontal) {
		sliderPos = 0.5 * SliderLength + sliderPos * (width() - SliderLength);
	} else {
		sliderPos = 0.5 * SliderLength + (1.0 - sliderPos) * (height() - SliderLength);
	}
	x1 = m_type == Horizontal ? sliderPos - 0.5 * SliderLength : x1;
	y1 = m_type == Horizontal ? y1 : sliderPos - 0.5 * SliderLength;
	x2 = m_type == Horizontal ? x1 + SliderLength : x1;
	y2 = m_type == Horizontal ? y1 : y1 + SliderLength;
	glColor4ub(0, 255, 0, 200);
	QGLHelper::aaGlLineWidth(SliderWidth);
	QGLHelper::aaGlBeginLines();
	QGLHelper::aaGlVertex2(x1, y1);
	QGLHelper::aaGlVertex2(x2, y2);
	QGLHelper::aaGlEndLines();

	glTranslated(-position.x(), -position.y(), 0.0);
}

void SliderGL::click(double x, double y)
{
	double len = (m_type == Horizontal ? width() : height() ) - SliderLength;
	double pos = m_type == Horizontal ? x - (this->x() + 0.5 * SliderLength) : len - ( y - (this->y() + 0.5 * SliderLength) );
	if(pos <= 0) m_pos = m_min;
	else if(pos >= len) m_pos = m_max;
	else {
		pos /= len;
		pos *= double(m_max - m_min);
		m_pos = int(round(pos));
	}
	emit valueChanged(m_pos);
}

void SliderGL::beginDragging(double x, double y)
{
	click(x, y);
}

void SliderGL::drag(double dx, double dy)
{
	double sliderPos = double(m_pos) / (m_max - m_min);
	if(m_type == Horizontal) {
		sliderPos = 0.5 * SliderLength + sliderPos * (width() - SliderLength);
		click(sliderPos + dx + x(), 0);
	} else {
		sliderPos = 0.5 * SliderLength + (1.0 - sliderPos) * (height() - SliderLength);
		click(0, sliderPos + dy + y());
	}
}

void SliderGL::setLength(double len)
{
	if(m_type == Horizontal)
		size.setWidth(len);
	else
		size.setHeight(len);
}

void SliderGL::move( double delta )
{
	int old_pos(m_pos);
	if(m_type == Vertical) delta = -delta;
	m_pos += int(delta);
	m_pos = std::max(m_pos, m_min);
	m_pos = std::min(m_pos, m_max);
	if(old_pos != m_pos)
		emit valueChanged(m_pos);
}

