

/// FernUniversität in Hagen, Lehrgebiet Mensch-Computer-Interaktion
/// License: Creative Commons Attribution-Noncommercial-Share Alike 3.0  Germany, see README file that comes with this work
/// Author: Sergey Cheremukhin


#include <QtGui/QApplication>
#include <QtGui/QFont>
#include <QtGui/QFontMetrics>
#include <QtGui/QImage>
#include <QtGui/QPainter>
#include <QtOpenGL/QGLWidget>

#include "textlabelgl.h"

TextLabelGL::TextLabelGL(const QString& txt) :
		m_text(txt), m_texture(0),
		m_width(0), m_height(0)
{
}

void TextLabelGL::buildTexture(uint foregroundArgb, QGLWidget* binder)
{
	QFont font(QApplication::font());
	font.setPointSizeF(font.pointSizeF() + 1.0);
	QFontMetrics fm(font);
	m_width = fm.width(m_text);
	m_height = fm.height();
	QImage img(m_width, m_height, QImage::Format_ARGB32);
	img.fill(0x00000000);

	QPainter painter;
	painter.begin(&img);
	painter.setPen(QColor(foregroundArgb));
	painter.setFont(font);
	painter.drawText(0, 0, m_width, m_height, Qt::AlignCenter, m_text);
	painter.end();

	QImage glImg( QGLWidget::convertToGLFormat(img) );

	if(m_texture != 0) {
		binder->deleteTexture(m_texture);
		m_texture = 0;
	}
	m_texture = binder->bindTexture(glImg);

	glBindTexture( GL_TEXTURE_2D, 0 );
}

void TextLabelGL::releaseTexture()
{
	if(m_texture != 0) {
		glDeleteTextures(1, &m_texture);
		m_texture = 0;
	}
}

void TextLabelGL::draw() const
{
	if(m_texture == 0) return;

	glBindTexture( GL_TEXTURE_2D, m_texture );

	glBegin( GL_QUADS );
	glTexCoord2f( 0.0, 0.0 ); glVertex2f( 0.0, 0.0 );
	glTexCoord2f( 1.0, 0.0 ); glVertex2f( m_width, 0.0 );
	glTexCoord2f( 1.0, 1.0 ); glVertex2f( m_width, m_height );
	glTexCoord2f( 0.0, 1.0 ); glVertex2f( 0.0, m_height );
	glEnd();

	glBindTexture( GL_TEXTURE_2D, 0 );
}
